#========================================================================================
# package      DE_EPAGES::Database::API::DAL
# state        private
#----------------------------------------------------------------------------------------
# description  database abstraction layer with caching.
#               Provides access to database-level attributes of an object. The
#               attributes can be distributed over multiple tables compromising
#               the OO inheritance path.
#               For example, for objects of class "Product", the attributes are stored
#               in the tables "object", "contentobject" and "product".
#----------------------------------------------------------------------------------------
# example      my $DAL = DE_EPAGES::Database::API::DAL->load('DE_EPAGES::Product::API::Object::Product');
#               my $ProductID = $DAL->insert({
#                   parentID => $ProductFolder->id, # object
#                   Alias => '0815',                # object
#                   ...
#                   IsVisible => 1,                 # contentobject
#                   ...
#                   Weight => 12.3,                 # product
#                   TaxClassID => $TaxClass->id,    # product
#               });
#               my $hProduct = $DAL->info($ProductID);
#               print "Alias: $hProduct->{Alias}\n";
#               print "Weight: $hProduct->{Weight}\n";
#               $DAL->update($ProductID, { Weight => 12.5 });
#               $DAL->delete($ProductID);
#               # find all products with a given tax class
#               my $ahProducts = $DAL->get('ByTaxClassID', { TaxClassID => $TaxClass->id });
#========================================================================================
package DE_EPAGES::Database::API::DAL;

use strict;
use DE_EPAGES::Core::API::File qw ( ExistsFile );
use DE_EPAGES::Core::API::PerlTools qw (ElementOf UsePackage);
use DE_EPAGES::DataCache::API::Cache qw ( GetCache UpdateCache );
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Database::API::XML::DALImportHandler qw (ImportDAL);

our %DAL; # cache

#========================================================================================
# function     load
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DAL = $Package->load($PackageName );
# example      $ObjectDAL = $Package->load('DE_EPAGES::Object::API::Object::Object');
#----------------------------------------------------------------------------------------
# description  load dal class, imports mapping and build internal structure.
#----------------------------------------------------------------------------------------
# input        $PackageName | package name which supports new dal | string
# return       $DAL | new DAL object | DE_EPAGES::Database::API::DAL
#========================================================================================
sub load {
    my $class = shift;
    my ($PackageName) = @_;

    my $driver = GetCurrentDBHandle()->driverName;
    my $dal = $DAL{$driver}{$PackageName};

    if (not defined $dal) {
        my @Cartridge = split (/::/, $PackageName);
        my $ClassName = pop @Cartridge;

        my $XMLDirectory   = join '/', $ENV{'EPAGES_CARTRIDGES'}, @Cartridge[0,1], 'Database/XML';
        my $DriverFileName = join '/', $XMLDirectory, $driver, $ClassName .'.xml';
        my $SQLFileName    = join '/', $XMLDirectory, 'SQL', $ClassName .'.xml';
        my $hOptions = ImportDAL($SQLFileName);
        ImportDAL($DriverFileName, $hOptions ) if ExistsFile($DriverFileName);

        $hOptions->{'DAL_PACKAGE'} = __PACKAGE__.'::'.$driver unless defined $hOptions->{'DAL_PACKAGE'};
        UsePackage( $hOptions->{'DAL_PACKAGE'} );
        my $nonCachedDal = $hOptions->{'DAL_PACKAGE'}->new( $hOptions );
        $dal = $class->new({
            'NONCACHED_DAL' => $nonCachedDal,
            'CLASS'         => $nonCachedDal->class,
            'IDENTIFIER'    => $nonCachedDal->{'IDENTIFIER'},
            'IDENTIFIER_SLAVE' => $nonCachedDal->table($nonCachedDal->class)->{'IDENTIFIER'},
        });
        $DAL{$driver}{$PackageName} = $dal;
    }
    return $dal;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DAL = $Package->new($hOptions);
#----------------------------------------------------------------------------------------
# description  bless options. use <function load> to get a new dal.
#----------------------------------------------------------------------------------------
# input        $hOptions | data for object| ref.hash
# return       $DAL | new DAL object | DE_EPAGES::Database::API::DAL
#========================================================================================
sub new {
    my $class = shift;
    my ($hOptions) = @_;

    return bless $hOptions, $class;
}

#========================================================================================
# function     nonCachedDal
# state        private
#----------------------------------------------------------------------------------------
# syntax       $NonCachedDAL = $DAL->nonCachedDal;
#----------------------------------------------------------------------------------------
# description  Returns the database abstraction layer without caching.
#----------------------------------------------------------------------------------------
# return       $NonCachedDAL | non-cached DAL object | DE_EPAGES::Database::API::DAL::SQL
#========================================================================================
sub nonCachedDal  { shift->{'NONCACHED_DAL'} }

## no critic qw(RequireArgUnpacking)
sub class         { shift->{'CLASS'} }
sub table         { shift->{'NONCACHED_DAL'}->table(@_) }
sub identifier    { shift->{'IDENTIFIER'} }
sub mapObjects2IDs{ shift->{'NONCACHED_DAL'}->mapObjects2IDs(@_) }
## use critic qw(RequireArgUnpacking)

#========================================================================================
# function     cacheUpdate
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DAL->cacheUpdate($Identfier, $hInfo, $IsDeleteOrInsert);
# example      $DAL->cacheUpdate($ObjectID, { 'Alias'=> $NewAlias }, 0);
#----------------------------------------------------------------------------------------
# description  Resets the cache entries, depending on $hInfo also IndexCache entries
#               will be updated.
#----------------------------------------------------------------------------------------
# input        $Identifier | object identifier | integer
# input        $hInfo | updated columns, incl. values | ref.hash
# input        $IsDeleteOrInsert | is statement insert or delete | boolean
#========================================================================================
sub cacheUpdate {
    my $self = shift;
    my ($Identifier, $hInfo, $IsDeleteOrInsert) = @_;
    my $dbi = GetCurrentDBHandle();
    my $cacheKey = $dbi->cacheKey;
    my @ClassNames = $self->{'NONCACHED_DAL'}->classNames;
    my $hCacheInfo = undef;
    my $hMergedCacheInfo = undef;

    # On insert or delete, don't fetch old values but simply store the
    #  new values.
    if( $IsDeleteOrInsert ) {
        $hCacheInfo = $hInfo;
        foreach my $key (keys %$hInfo) {
            $hMergedCacheInfo->{$key} = $dbi->encodeSpaces($hInfo->{$key});
        }
    }
    # In all other cases, merge the cached values in $hCacheInfo with the
    #  new ones in $hInfo into $hMergedCacheInfo. This one is used as
    #  last parameter of UpdateCache to update the cache in one step.
    else {
        $hCacheInfo = $self->info($Identifier);
        $hInfo = $hCacheInfo unless defined $hInfo;

        foreach my $key (keys %$hCacheInfo) {
            $hMergedCacheInfo->{$key} = $hCacheInfo->{$key};
        }
        foreach my $key (keys %$hInfo) {
            $hMergedCacheInfo->{$key} = $dbi->encodeSpaces($hInfo->{$key});
        }
    }

    foreach my $ClassName (@ClassNames) {
        # update 'Info' caches
        # $hMergedCacheInfo contains all merged new and old values and is used
        # to prevent another SELECT inside the UpdateCache method to gather old values.
        my $Table = $self->{'NONCACHED_DAL'}->table($ClassName);
        if( defined $hMergedCacheInfo ) {
            my $hColumns = $Table->{COLUMN};
            my %CacheInfo = map { $_ => $hMergedCacheInfo->{$_} }
                grep { exists $hColumns->{$_} }
                keys %$hMergedCacheInfo;
            UpdateCache($cacheKey,'Info'.$ClassName, $Identifier, \%CacheInfo);
        }
        else {
            UpdateCache($cacheKey,'Info'.$ClassName, $Identifier);
        }

        UpdateCache($cacheKey,'All'.$ClassName) if $IsDeleteOrInsert;

        if (defined $hInfo) {
            # update index caches
            foreach my $IndexName (keys %{$Table->{'INDEX'}}) {
                my $DoUpdate = 0;
                # CacheParts1 includes new data, @CacheParts2 includes old data
                my (@CacheParts1, @CacheParts2) = ();
                foreach my $Column (@{$Table->{'INDEX'}{$IndexName}{'COLUMNREF'}}) {
                    if (exists $hInfo->{$Column}) {
                        $DoUpdate = 1;
                        push @CacheParts1, $hInfo->{$Column};
                    } else {
                        push @CacheParts1, $hCacheInfo->{$Column};
                    }
                    push @CacheParts2, $hCacheInfo->{$Column};
                }
                if ($DoUpdate) {
                    # convert both arrays of cache entries to a string and compare
                    # if different, update cache with new index parameter
                    my $CacheString1 = join '<||>', map { defined $_ ? $_ : ''} @CacheParts1;
                    my $CacheString2 = join '<||>', map { defined $_ ? $_ : ''} @CacheParts2;

                    UpdateCache($cacheKey, $ClassName.$IndexName, $CacheString1);
                    UpdateCache($cacheKey, $ClassName.$IndexName, $CacheString2) if ($CacheString1 ne $CacheString2);
                    if ($ClassName ne $self->{'CLASS'}) {
                        UpdateCache($cacheKey, $self->{'CLASS'}.$IndexName, $CacheString1);
                        UpdateCache($cacheKey, $self->{'CLASS'}.$IndexName, $CacheString2) if $CacheString1 ne $CacheString2;
                    }
                }
            }
        } else {
            # delete without cache info
            UpdateCache($cacheKey, $ClassName.$_) foreach keys %{$Table->{'INDEX'}};
        }
    }
}

#========================================================================================
# function     cacheUpdateTable
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DAL->cacheUpdateTable;
#----------------------------------------------------------------------------------------
# description  Resets all caches (info, all, indeces) of this dal.
#========================================================================================
sub cacheUpdateTable {
    my $self = shift;

    my $cacheKey = GetCurrentDBHandle()->cacheKey;
    foreach my $ClassName ($self->{'NONCACHED_DAL'}->classNames) {
        # update 'Info' caches
        UpdateCache($cacheKey,'All'.$ClassName);
        UpdateCache($cacheKey,'Info'.$ClassName);
        my $Table = $self->{'NONCACHED_DAL'}->table($ClassName);
        UpdateCache($cacheKey, $ClassName.$_) foreach keys %{$Table->{'INDEX'}};
    }
}

#========================================================================================
# function     _cachePush
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DAL->_cachePush($ahInfo);
#----------------------------------------------------------------------------------------
# description  push given infos in to the cache.
#----------------------------------------------------------------------------------------
# input        $ahInfo | result set | ref.array.hash
#========================================================================================
sub _cachePush {
    my $self = shift;
    my ($ahResults) = @_;

    my $IdentifierName = $self->{'IDENTIFIER'};
    my $cacheKey = GetCurrentDBHandle()->cacheKey;

    my @ClassNames = $self->{'NONCACHED_DAL'}->classNames;
    foreach my $ClassName (@ClassNames) {
        my $Table = $self->{'NONCACHED_DAL'}->table($ClassName);
        my $CacheRef = GetCache($cacheKey, 'Info'.$ClassName);
        foreach my $hRow (@$ahResults) {
            my %Result = ();
            $Result{$_} = $hRow->{$_} foreach @{$Table->{'CACHE_NAMES'}};
            $CacheRef->{$hRow->{$IdentifierName}} = \%Result;
        }
    }
    return;
}

#========================================================================================
# function     _cacheGet
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ahInfo = $DAL->_cacheGet($aIndentifier);
# example      $ahInfo = $DAL->_cacheGet([1, 2, 3]);
#----------------------------------------------------------------------------------------
# description  returns infos from cache, if one info wasn't found the function returns
#               undef.
#----------------------------------------------------------------------------------------
# input        $aIdentifier | list of identifier | ref.array.integer
# return       $ahInfo | result set (optional) | ref.array.hash
#========================================================================================
sub _cacheGet {
    my $self = shift;
    my ($aIdentifier) = @_;

    # get caches
    my $cacheKey = GetCurrentDBHandle()->cacheKey;
    my @ClassCaches = map { GetCache($cacheKey, "Info$_") } $self->{'NONCACHED_DAL'}->classNames;
    my $IndentifierMaster = $self->{'IDENTIFIER'};
    my $IndentifierSlave =  $self->{'IDENTIFIER_SLAVE'};

    # collect cache data
    return [
        map {
            my $ID = $_;
            +{
                $IndentifierMaster => $ID,
                $IndentifierSlave  => $ID,
                map {
                    my $hInfo = $_->{$ID};
                    return undef unless defined $hInfo;
                    %$hInfo;
                } @ClassCaches
            };
        } @$aIdentifier
    ];
}

#========================================================================================
# function     insert
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Identifier = $DAL->insert($hParameter);
#----------------------------------------------------------------------------------------
# description  Insert a new object in to database. The identifier can be part of
#               parameter.
#----------------------------------------------------------------------------------------
# input        $hParameter | key value pairs (column name => value) | ref.hash
# return       $Identifier | object identifier | integer
#========================================================================================
sub insert {
    my $self = shift;
    my ($hParameter) = @_;

    my $Identifier = $self->{'NONCACHED_DAL'}->insert($hParameter);

    # update cache
    $hParameter->{$self->{'IDENTIFIER'}} = $Identifier;
    $self->cacheUpdate($Identifier, $hParameter,1);

    return $Identifier;
}

#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DAL->update($Identifier, $hParameter);
#----------------------------------------------------------------------------------------
# description  Update an existing object in database.
#----------------------------------------------------------------------------------------
# input        $Identifier | object identifier | integer
# input        $hParameter | key value pairs (column name => value) | ref.hash
#========================================================================================
sub update {
    my $self = shift;
    my ($Identifier, $hParameter) = @_;

    my $hInfo = $self->info($Identifier);
    my $ChangedValues = {};

    foreach my $Key (keys %$hParameter) {
        next if not defined $hInfo->{$Key} and not defined $hParameter->{$Key};

        if ( defined $hParameter->{$Key} and defined $hInfo->{$Key}) {
            # fix for "Cannot compare a datetime to a regular scalar" in DateTime.pm
            if( 'DateTime' eq ref $hParameter->{$Key} && ! ref $hInfo->{$Key} ) {
                my $OldValue = GetCurrentDBHandle()->unquoteDateTime($hInfo->{$Key});
                $ChangedValues->{$Key} = $hParameter->{$Key} unless $hParameter->{$Key} == $OldValue;
                next;
            }

            if ($hInfo->{$Key} ne $hParameter->{$Key}) {
                my ($hColumnInfo) = $self->{'NONCACHED_DAL'}->column($Key);
                $ChangedValues->{$Key} = $hParameter->{$Key} unless
                    ElementOf($hColumnInfo->{'Type'}, 'Integer', 'Float', 'Money', 'DateTime')
                    and $hInfo->{$Key} eq $hParameter->{$Key}
            }
        } else {
            $ChangedValues->{$Key} = $hParameter->{$Key};
        }
    }
    return unless scalar keys %$ChangedValues;

    # update cache
    $self->cacheUpdate($Identifier, $ChangedValues,0);
    $self->{'NONCACHED_DAL'}->update($Identifier, $ChangedValues);
    return;
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DAL->delete($Identifier);
#----------------------------------------------------------------------------------------
# description  Update an existing object in database.
#----------------------------------------------------------------------------------------
# input        $Identifier | object identifier | integer
#========================================================================================
sub delete {
    my $self = shift;
    my ($Identifier) = @_;

    # update cache
    $self->cacheUpdate($Identifier, undef, 1);

    $self->{'NONCACHED_DAL'}->delete($Identifier);
    return;
}

#========================================================================================
# function     get
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahObjects = $DAL->get($IndexName, $hParameter);
# syntax       $ahObjects = $DAL->get($IndexName, $aParameter);
#----------------------------------------------------------------------------------------
# description  Get objects from database.
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $hParameter | parameter of index | ref.hash
# input        $aParameter | parameter of index | ref.array
# return       $ahObjects | object data | ref.array.hash
#========================================================================================
sub get {
    my $self = shift;
    my ($IndexName, $Parameter) = @_;

    my $aParameter;
    if (ref $Parameter eq 'HASH') {
        $aParameter = [map { $Parameter->{$_} } @{$self->{'NONCACHED_DAL'}->index($IndexName)->{'COLUMNREF'}}];
    } else {
        $aParameter = $Parameter;
    }
    my $CacheString = join '<||>', @$aParameter;

    my $dbi = GetCurrentDBHandle();
    my $IndexCache = GetCache($dbi->cacheKey, $self->{'CLASS'}.$IndexName);
    my $aCacheResult = $IndexCache->{$CacheString};

    # found in cache, collect rows
    if (defined $aCacheResult) {
        my $ahResults = $self->_cacheGet($aCacheResult);
        return $ahResults if defined $ahResults;
    }

    my $ahResult = $self->{'NONCACHED_DAL'}->get($IndexName, $aParameter);
    $self->_cachePush($ahResult);
    my $Identifier = $self->{'IDENTIFIER'};
    my $IdentifierSlave = $self->{'IDENTIFIER_SLAVE'};
    $_->{$IdentifierSlave} = $_->{$Identifier} foreach @$ahResult;
    $IndexCache->{$CacheString} = [ map { $_->{$Identifier} } @$ahResult ];
    return $ahResult;
}

#========================================================================================
# function     iterator
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $DAL->iterator($IndexName, $hParameter, $aOrderBy);
# syntax       $Iterator = $DAL->iterator($IndexName, $aParameter);
#----------------------------------------------------------------------------------------
# description  Returns object ids from the database as an iterator.
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $hParameter | parameter of index | ref.hash
# input        $aParameter | parameter of index | ref.array
# input        $aOrderBy | (optional) attributes for sorting,
#                           e.g. [{ Column => 'Position', Direction => 'ASC' }]
#                         | ref.array.hash
# return       $Iterator | object id iterator | DE_EPAGES::Core::API::Iterator
#========================================================================================
sub iterator {
    my $self = shift;
    my ($IndexName, $aParameter, $aOrderBy) = @_;

    return $self->{'NONCACHED_DAL'}->iterator($IndexName, $aParameter, $aOrderBy);
}

#========================================================================================
# function     getFirst
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hObject = $DAL->getFirst($IndexName, $hParameter);
# syntax       $hObject = $DAL->getFirst($IndexName, $aParameter);
#----------------------------------------------------------------------------------------
# description  Get object from database (useful for unique keys).
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $hParameter | parameter of index | ref.hash
# input        $aParameter | parameter of index | ref.array
# return       $hObject | object data | ref.hash
#========================================================================================
sub getFirst {
    my $self = shift;
    my ($IndexName, $Parameter) = @_;

    my $ahResult = $self->get($IndexName, $Parameter);
    return undef unless scalar @$ahResult;
    return $ahResult->[0];
}

#========================================================================================
# function     uncachedInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hObject = $DAL->uncachedInfo($Identifier);
#----------------------------------------------------------------------------------------
# description  Get object from database (direct with out cache).
#----------------------------------------------------------------------------------------
# input        $Identifier | object identifier | integer
# return       $hObject | object data | ref.hash
#========================================================================================
sub uncachedInfo {
    my $self = shift;
    my ($Identifier) = @_;

    my $result = $self->{'NONCACHED_DAL'}->info($Identifier);
    return undef unless defined $result;
    $result->{$self->{'IDENTIFIER_SLAVE'}} = $Identifier;
    $result->{$self->{'IDENTIFIER'}} = $Identifier;
    $self->_cachePush([$result]);
    return $result;
}

#========================================================================================
# function     info
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hObject = $DAL->info($Identifier);
#----------------------------------------------------------------------------------------
# description  Get object from database (tries to get data from cache).
#----------------------------------------------------------------------------------------
# input        $Identifier | object identifier | integer
# return       $hObject | object data | ref.hash
#========================================================================================
sub info {
    my $self = shift;
    my ($Identifier) = @_;

    # look at dal data cache
    my $ahResults = $self->_cacheGet([$Identifier]);
    return $ahResults->[0] if defined $ahResults;
    return $self->uncachedInfo($Identifier);
}

#========================================================================================
# function     multi
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahObject = $DAL->multi($aIdentifier);
#----------------------------------------------------------------------------------------
# description  Get objects from database.
#----------------------------------------------------------------------------------------
# input        $aIdentifier | list of object identifier | ref.array.integer
# return       $ahObject | object data | ref.array.hash
#========================================================================================
sub multi {
    my $self = shift;
    my ($aIdentifier) = @_;

    # look at dal data cache
    my (%ResultSet, @UnLoadedIDs);
    foreach my $Identifier (@$aIdentifier) {
        my $aResults = $self->_cacheGet([$Identifier]);
        if (defined $aResults) {
            $ResultSet{$Identifier} = $aResults->[0];
        } else {
            push @UnLoadedIDs, $Identifier;
        }
    }

    # load objects, which were not found at cache
    my $Count = scalar @UnLoadedIDs;
    if ($Count) {
        # get statement
        my $ahResult = $self->{'NONCACHED_DAL'}->multi(\@UnLoadedIDs);
        for(my $i=0;$i<$Count;$i++) {
            $ResultSet{$UnLoadedIDs[$i]} = $ahResult->[$i];
        }
        # cache info
        $self->_cachePush($ahResult);
    }
    my $Identifier = $self->{'IDENTIFIER'};
    my $IdentifierSlave = $self->{'IDENTIFIER_SLAVE'};
    $_->{$IdentifierSlave} = $_->{$Identifier} foreach values %ResultSet;
    return [ @ResultSet{@$aIdentifier} ];
}

#========================================================================================
# function     exists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hObject = $DAL->exists($Identifier);
#----------------------------------------------------------------------------------------
# description  Exists object in database.
#----------------------------------------------------------------------------------------
# input        $Identifier | object identifier | integer
# return       $Exists | true or false | boolean
#========================================================================================
sub exists {
    my $self = shift;
    my ($Identifier) = @_;

    my $aResults = $self->_cacheGet([$Identifier]);
    return 1 if defined $aResults;

    return $self->{'NONCACHED_DAL'}->exists($Identifier);
}

#========================================================================================
# function     nextid
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ID = $DAL->nextid;
#----------------------------------------------------------------------------------------
# description  Returns a new unique identifier or undef for auto_increment columns.
#----------------------------------------------------------------------------------------
# return       $ID | next identifier | integer
#========================================================================================
sub nextid {
    my $self = shift;
    return $self->{'NONCACHED_DAL'}->nextid;
}

#========================================================================================
# function     all
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahObjects = $DAL->all();
#----------------------------------------------------------------------------------------
# description  Get all objects from database.
#----------------------------------------------------------------------------------------
# return       $ahObjects | object data | ref.array.hash
#========================================================================================
sub all {
    my $self = shift;

    my $CacheString = '1';
    my $dbi = GetCurrentDBHandle();
    my $IndexCache = GetCache($dbi->cacheKey, 'All'.$self->{'CLASS'});
    my $aCacheResult = $IndexCache->{$CacheString};

    # found in cache, collect rows
    if (defined $aCacheResult) {
        my $ahResults = $self->_cacheGet($aCacheResult);
        return $ahResults if defined $ahResults;
    }
    my $ahResult = $self->{'NONCACHED_DAL'}->all();
    $self->_cachePush($ahResult);
    my $Identifier = $self->{'IDENTIFIER'};
    $IndexCache->{$CacheString} = [ map { $_->{$Identifier} } @$ahResult];
    return $ahResult;
}

1;
