use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Installer::API::GitPatchBuilder;

# A test for this script is defined in the infra repo:
# /epages-infra/scripts/epages/test/build/test_patch_code_generation.sh

sub Main {

    local $| = 1;

    my $OldVersion;
    my $NewVersion;
    my $Branch;
    my $Help;
    my $PropertiesFile;
    my $Cartridge;
    my $NoOutput = 0;
    my $Verbose = 1;


    GetOptions(
        'oldversion=s'          => \$OldVersion,
        'newversion=s'          => \$NewVersion,
        'branch=s'              => \$Branch,
        'p=s'                   => \$PropertiesFile,
        'help'                  => \$Help,
        'cartridge=s'           => \$Cartridge,
        'nooutput'              => \$NoOutput,
    );

    my $time1 = time;

    usage() if $Help;
    $Verbose = 0 if($NoOutput);

    unless( $OldVersion && $NewVersion ) {
        print "missing parameter -oldversion or -newversion\n";
        usage();
    }

    unless(defined $Branch) {
        print "missing parameter -branch\n";
        usage();
    }

    # trigger patch file generation
    my $hBuildOptions = {
        'verbose'       => $Verbose,
        'branch'        => $Branch,
        'build_only_cartridge' => $Cartridge,
    };
    my $GitPatchBuilder = DE_EPAGES::Installer::API::GitPatchBuilder->new({'verbose' => $Verbose});
    $GitPatchBuilder->buildCartridgePatches($OldVersion, $NewVersion, $hBuildOptions);

    my $time2 = time;
    printf "\n\nneed %.3f seconds\n", $time2-$time1;
}


sub usage {
    print <<END_USAGE;
This script triggers the generation of the patch code for the Cartridges repo.

The source branch for the patch file generation is the current branch of the
Cartridges repository.

Usage:
    perl $0 [options] [flags]

options:
    -oldversion       version number of the previous release
                          (e.g. 6.17.36)
    -newversion       version number of the target release
                          (e.g. 6.17.36.1)
    -p                the properties file for branch/commit tracking
                          (e.g. /tmp/patch.properties)
    -branch           the branch on which the patch files will be commited
                          (e.g. generated/patch_6_17_36)
    -cartridge        optional; the name of the cartridge for which the patch files are to be
                      generated. By default all cartridges are being used.
                          (e.g. Product)
    -nooutput         optional; set this flag to avoid printing of log messages
    -help             optional; set this flag to print out usage information for this script

Examples:
    # Create patch files for all cartridges
    perl $0 -oldversion 6.17.22 -newversion 6.17.23 -branch generated/patch_6_17_36 \
    -p /tmp/patch.properties
    # Create patch files for only one cartridge for testing purposes
    perl $0 -oldversion 6.17.22 -newversion 6.17.23 -cartridge Shop -branch test/patch_6_17_36 \
    -p /tmp/patch.properties

END_USAGE
    exit 2;
}

RunScript(
    'Debug' => 0,
    'Message' => 'short',
    'Sub' => \&Main
);

1;
