use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Installer::API::CVSTagHandler;
use DE_EPAGES::Installer::API::SystemPatchBuilder;
use DE_EPAGES::Core::API::File qw (WriteFile);


sub Main {

    local $| = 1;

    my $CleanUp;
    my $OldVersion;
    my $NewVersion;
    my $Branch;
    my $TargetBranch;
    my $BaseBranch;
    my $UserName;
    my $UserToken;
    my $UserCrossRepo;
    my $Remote;
    my $Help;
    my $Email;
    my $AutoMode = 0;
    my $Rebuild;
    my $CompleteBuild;
    my $PropertiesFile;
    my $NotUseGenerated = 0;
    my $Cartridge;
    my $useGit = 0;
    my $mergePR = 0;
    my $Verbose = 1;


    GetOptions(
        'oldversion=s'          => \$OldVersion,
        'newversion=s'          => \$NewVersion,
        'branch=s'              => \$Branch,
        'targetbranch=s'        => \$TargetBranch,
        'baseBranch=s'          => \$BaseBranch,
        'userName=s'            => \$UserName,
        'userToken=s'           => \$UserToken,
        'userCrossRepo=s'       => \$UserCrossRepo,
        'remote=s'              => \$Remote,
        'cleanup'               => \$CleanUp,
        'automode'              => \$AutoMode,
        'email=s'               => \$Email,
        'rebuild'               => \$Rebuild,
        'complete_build'        => \$CompleteBuild,
        'p=s'                   => \$PropertiesFile,
        'help'                  => \$Help,
        'not_use_generated'     => \$NotUseGenerated,
        'cartridge=s'           => \$Cartridge,
        'mergePR'               => \$mergePR,
        'useGit'                => \$useGit,
        'verbose'               => \$Verbose,
    );

    my $time1 = time;

    usage() if $Help;

    unless( $OldVersion && $NewVersion ) {
        print "missing parameter -oldversion or -newversion\n";
        usage();
    }

    if( $CleanUp and ($Rebuild or $CompleteBuild)) {
        print "option 'cleanup' not able in combination with 'rebuild' or 'complete_build'\n";
        usage();
    }

    if($useGit and not defined $Branch) {
        print "option 'useGit' requires a 'branch' value\n";
        usage();
    }

    #set git specific value for AutoMode to decide whether to push or not
    if($useGit) {
        if(defined $Remote) {
            $AutoMode = 1;
        }
        else {
            $AutoMode = 0;
        }
    }

    #get system patch builder
    my $SystemPatchBuilder = DE_EPAGES::Installer::API::SystemPatchBuilder->new({'verbose' => 1});

    #get options
    my $hCleanOptions = {
        'verbose' => 1,
    };
    #check if cleanup is activated
    if ($CleanUp){
        $SystemPatchBuilder->cleanUpPatchReleaseTags($NewVersion,$AutoMode,$hCleanOptions,$useGit);
        return 1;
    }

    #set options
    my $hTagOptions = {
        'verbose' => 1,
        'email'   => $Email,
    };

    #updateReleasePatchTags
    if(!$useGit) {
        $SystemPatchBuilder->updateReleasePatchTags($OldVersion,$NewVersion,$AutoMode,$hTagOptions);
    }

    #set options
    my $hBuildOptions = {
        'verbose'       => $Verbose,
        'email'         => $Email,
        'branch'        => $Branch,
        'targetbranch'  => $TargetBranch,
        'remote'        => $Remote,
        'reconstruct'   => $Rebuild,
        'build_only_cartridge'     => $Cartridge,
        'build_all_cartridges' => $CompleteBuild,
        'use_generate'  => not($NotUseGenerated),
    };

    #build cartridge patches
    if (not $SystemPatchBuilder->buildCartridgePatches($OldVersion,$NewVersion,$AutoMode,$hBuildOptions,$useGit)) {
        die 'Build of Cartridge patches was not successful. Aborting now...';
    }

    #create and merge pull request
    my $FeatureBranch = $TargetBranch;
    $FeatureBranch = $Branch unless (defined $TargetBranch);

    #set pr options
    my $hPROptions = {
        'userName'      => $UserName,
        'userCrossRepo' => $UserCrossRepo,
        'userToken'     => $UserToken,
        'featureBranch' => $FeatureBranch,
        'baseBranch'    => $BaseBranch,
        'repository'    => 'Cartridges'
    };
    $FeatureBranch = $UserCrossRepo.':'.$FeatureBranch if(defined $UserCrossRepo);
    if($useGit and (defined $BaseBranch) and ($FeatureBranch ne $BaseBranch)) {
        my $PullRequestId = $SystemPatchBuilder->createPullRequest($hPROptions);
        if ($mergePR) {
            $SystemPatchBuilder->mergePullRequest($PullRequestId, $hPROptions);
        }
    }

    if($PropertiesFile){
        #get Tags from versions
        my $ReleaseTag = $SystemPatchBuilder->getNewReleaseTag($NewVersion,$AutoMode,$hBuildOptions);
        my $PatchTag = $SystemPatchBuilder->getPatchTag($NewVersion,$AutoMode,$hBuildOptions);

        my $Content = "PATCH_TAG=$PatchTag\nRELEASE_TAG=$ReleaseTag";

        WriteFile($PropertiesFile,\$Content,{'create_directory' => 1})
    }

    my $time2 = time;
    printf "\n\nneed %.3f seconds\n", $time2-$time1;

}


sub usage {
    print <<END_USAGE;
This script sets the patch_* tag for release_*, that means, all files that
have changed content from oldrelease to newrelease get the patch tag.
The Script also build all necessary patch files and commit changes.

Git version:
The patch files will be build from the given old version until the last commit of your current branch.
Be sure to set your <branch> to the correct commit, before running this script.

Usage:
    perl $0 [options] [flags]

options:
    -oldversion       version number of the previous release
    -newversion       version number of the target release database user password
    -email            email address for patch info mail, have to be an external address
    -p                property file, used by jenkins to get used patch and release tags
    -cartridge        just handle cartridge
git options:
    -branch           git only; branch to build and commit patch files; given -targetbranch:
                      branch to build patch files and to create targetbranch, commits patch files to targetbranch (required if useGit is enabled)
    -targetbranch     git only; branch to commit patch files (optional if useGit is enabled)
    -remote           git only; remote to push generated patch files to (optional if useGit is enabled)
pull request options:
    -baseBranch       git only; if defined, creates and merge a pull request via GitHub API;
                      pull and merge generated patch files into baseBranch
    -userName         git only; required, name of a GitHub user account to create pull requests
    -userToken        git only; required, personal access token or password for the corresponding userName
    -userCrossRepo    git only; optional, name of a GitHub user account;
                      create PR of a branch from a repository that does not belong to <userName>

flags:
    -help             show the command line options
    -cleanup          remove all patch and release tags
    -rebuild          ignores already builded patch files
    -automode         cvs only; automatic mode, operates in HEAD and uses temporary source and target tags
    -complete_build   build patch files for all cartridges, even if there are no changes in current tag process
    -noverbose        dont print messages (optional)
    -not_use_generated  with -automode, commit branch is HEAD instead of B_generated_patch_code
    -useGit           use Git version to build patch package
    -mergePR          git only; after patch code generation a PR

Examples:
    perl $0 -oldversion 6.17.3 -newversion 6.17.4
    perl $0 -oldversion 6.17.3 -newversion 6.17.4 -email patch[at]epages.com -automode -rebuild -complete_build

Examples -useGit:
    use branch develop, commit files to develop, no push
    perl $0 -oldversion 6.17.22 -newversion 6.17.23 -cartridge Shop -useGit -branch develop
    use branch develop, commit files to test/PatchBuildGit, no push
    perl $0 -oldversion 6.17.22 -newversion 6.17.23 -cartridge Shop -useGit -branch develop -targetbranch test/PatchBuildGit
    use branch develop, commit files to test/PatchBuildGit, push to upstream
    perl $0 -oldversion 6.17.22 -newversion 6.17.23 -cartridge Shop -useGit -branch develop -targetbranch test/PatchBuildGit -remote upstream
END_USAGE
    exit 2;
}

RunScript(
    'Debug' => 0,
    'Message' => 'short',
    'Sub' => \&Main
);

1;
