#========================================================================================
# package      DE_EPAGES::MeinPaket::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::MeinPaket::API::Cartridge;
use base DE_EPAGES::ProductPortal::API::PortalInstaller;

use strict;
use DE_EPAGES::Core::API::File qw ( GetFileContent SetAccessRights );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle RunOnStore );
use DE_EPAGES::Database::API::DBInterface qw ( ProcessSqlGo );
use DE_EPAGES::Database::API::OrderFile qw ( GetSqlFileOrder );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::MeinPaket::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::MeinPaket.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ( $class, %options ) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/MeinPaket',
        'Version'            => '7.11.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;
    ImportObjects( $self->xmlDirectory . '/MailTypes.xml');
    $self->installMeinPaketCategories;
    $self->installMeinPaketVariantConfigurations;
    $self->finishInstall;
    return;
}

#========================================================================================
# function     installMeinPaketCategories
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->installMeinPaketCategories( );
#----------------------------------------------------------------------------------------
# description  Installs MeinPaketCategory structure
#========================================================================================
sub installMeinPaketCategories {
    my $self = shift;

    RunOnStore(
        Store => 'MeinPaketCategories',
        Sub   => sub {
            my $dbi = GetCurrentDBHandle();
            $dbi->transaction(sub{
                if (    not $dbi->existsTable('meinpaketsite') or
                        not $dbi->existsTable('meinpaketcategory') or
                        not $dbi->existsTable('meinpaketcategorytemp') ) {
                    $self->uninstallMeinPaketCategories($dbi);
                    my $Dir = join '/',$ENV{'EPAGES_CARTRIDGES'},$self->cartridgeDirectory,'Database',$dbi->driverName;
                    my $hValidFileNames = {
                        "$Dir/Tables/meinpaketsite.sql"         => 1,
                        "$Dir/Tables/meinpaketcategory.sql"     => 1,
                        "$Dir/Tables/meinpaketcategorytemp.sql" => 1,
                    };
                    my $aFiles = GetSqlFileOrder($Dir);
                    foreach my $File (@$aFiles) {
                        next unless exists $hValidFileNames->{$File};
                        my $rContent = GetFileContent($File);
                        ProcessSqlGo($$rContent);
                    }
                    SetAccessRights("$ENV{EPAGES_CONFIG}/MeinPaketCategories.db");
                }
            });
        }
    );
}

#========================================================================================
# function     uninstallMeinPaketCategories
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstallMeinPaketCategories( );
#----------------------------------------------------------------------------------------
# description  Uninstalls MeinPaketCategory structure
#========================================================================================
sub uninstallMeinPaketCategories {
    my $self = shift;
    my ($dbi) = @_;

    my $Dir = join '/', $ENV{'EPAGES_CARTRIDGES'}, $self->cartridgeDirectory, 'Database', $dbi->driverName;
    my $aFiles = GetSqlFileOrder($Dir);
    foreach my $File (@$aFiles) {
        next unless $File =~ /\/delete_meinpaketcategory/;
        my $rContent = GetFileContent($File);
        ProcessSqlGo($$rContent);
    }
}

#========================================================================================
# function     installMeinPaketVariantConfigurations
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->installMeinPaketVariantConfigurations( );
#----------------------------------------------------------------------------------------
# description  Installs MeinPaketVariantConfiguration structure
#========================================================================================
sub installMeinPaketVariantConfigurations {
    my $self = shift;

    RunOnStore(
        Store => 'MeinPaketVariantConfigurations',
        Sub   => sub {
            my $dbi = GetCurrentDBHandle();
            $dbi->transaction(sub{
                if (    not($dbi->existsTable('meinpaketvariantconfigurationsite')) or
                        not($dbi->existsTable('meinpaketvariantconfiguration')) or
                        not($dbi->existsTable('meinpaketrequiredattribute')) or
                        not($dbi->existsTable('meinpaketvariantattributevalue')) or
                        not($dbi->existsTable('meinpaketvariantconfigurationtemp')) or
                        not($dbi->existsTable('meinpaketrequiredattributetemp')) or
                        not($dbi->existsTable('meinpaketvariantattributevaluetemp')) ) {
                    $self->uninstallMeinPaketVariantConfigurations($dbi);
                    my $Dir = join '/',$ENV{'EPAGES_CARTRIDGES'},$self->cartridgeDirectory,'Database',$dbi->driverName;
                    my $hValidFileNames = {
                        "$Dir/Tables/meinpaketvariantconfigurationsite.sql"  => 1,
                        "$Dir/Tables/meinpaketvariantconfiguration.sql"      => 1,
                        "$Dir/Tables/meinpaketrequiredattribute.sql"         => 1,
                        "$Dir/Tables/meinpaketvariantattributevalue.sql"     => 1,
                        "$Dir/Tables/meinpaketvariantconfigurationtemp.sql"  => 1,
                        "$Dir/Tables/meinpaketrequiredattributetemp.sql"     => 1,
                        "$Dir/Tables/meinpaketvariantattributevaluetemp.sql" => 1,
                    };
                    my $aFiles = GetSqlFileOrder($Dir);
                    foreach my $File (@$aFiles) {
                        next unless exists $hValidFileNames->{$File};
                        my $rContent = GetFileContent($File);
                        ProcessSqlGo($$rContent);
                    }
                    SetAccessRights("$ENV{EPAGES_CONFIG}/MeinPaketVariantConfigurations.db");
                }
            });
        }
    );
}

#========================================================================================
# function     uninstallMeinPaketVariantConfigurations
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstallMeinPaketVariantConfigurations( );
#----------------------------------------------------------------------------------------
# description  Uninstalls MeinPaketVariantConfiguration structure
#========================================================================================
sub uninstallMeinPaketVariantConfigurations {
    my $self  = shift;
    my ($dbi) = @_;

    my $Dir = join '/', $ENV{'EPAGES_CARTRIDGES'}, $self->cartridgeDirectory, 'Database', $dbi->driverName;
    my $aFiles = GetSqlFileOrder($Dir);
    foreach my $File (@$aFiles) {
        next unless $File =~ /\/delete_meinpaketvariantconfiguration/;
        my $rContent = GetFileContent($File);
        ProcessSqlGo($$rContent);
    }
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    RunOnStore(
        Store => 'MeinPaketCategories',
        Sub   => sub {
            $self->uninstallMeinPaketCategories(GetCurrentDBHandle());
        }
    );
    RunOnStore(
        Store => 'MeinPaketVariantConfigurations',
        Sub   => sub {
            $self->uninstallMeinPaketVariantConfigurations(GetCurrentDBHandle());
        }
    );
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# function     patchDBStructure_6_15_4
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_15_4;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.15.4' to version '6.16.0'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.16.0' | string
#========================================================================================
sub patchDBStructure_6_15_4 {
    my $self = shift;

    #call data base structure changes from patch.xml file
    my $dbi = GetCurrentDBHandle();
    $dbi->transaction(sub{
        if( $dbi->driverName() eq 'mysql' and not($dbi->existsTable('meinpaketvariantconfigurationmap')) ) {
            $self->importPatchDir('Patch_6_15_4', 'PatchDBStructure');
        }
    });

    GetLog()->info('add sqlite db MeinPaketVariantConfigurations');
    $self->installMeinPaketVariantConfigurations();
    return '6.16.0';
}

1;
