#========================================================================================
# package      DE_EPAGES::Object::API::BaseAttribute
# state        public
# description  Base class for implementing custom attributes.
#               Normally you override at least the function <function getAttributes>.
#               If your atrributes are writeable, also override <function setAttributes>.
#========================================================================================
package DE_EPAGES::Object::API::BaseAttribute;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw( LogDebug );
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );

#ERRORDEFINITION
sub ErrorDefinition { return {
    'NoResults' => 'Object doesn\'t exist ( ObjectID=#ObjectID, Package=#Package )',
}}
#ERRORDEFINITION END

#========================================================================================
# function     getAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $Package->getAttribute( $Object, $Name, $LanguageID );
#----------------------------------------------------------------------------------------
# description  Returns a single attribute value. Don't call this function directly.
#               Use $Object->get() instead.
#               The default implementation calls <function getAttributes>
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# input        $Name       | attribute name  | string
# input        $LanguageID | language id | int
# return       $Value      | attribute value | string
#========================================================================================
sub getAttribute {
    my $self = shift;
    my ($Object, $Name, $LanguageID) = @_;
    return $self->getAttributes( $Object, [ $Name ], $LanguageID )->{$Name};
}

#========================================================================================
# function     getAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->getAttributes( $Object, $aNames, $LanguageID );
# example      sub getAttributes {
#                   my $self = shift;
#                   my ($Object, $aNames, $LanguageID) = @_;
#
#                   my %Values;
#                   foreach my $Name ( @$aNames ) {
#                       if( $Name eq 'Name' ) {
#                           my $aRows = GetCurrentDBHandle()->execute(
#                               'SELECT name FROM group WHERE groupid = ?',
#                               [ $Object->id ] );
#                           $Values{$Name} = $aRows->[0]->[0];
#                       }
#                   }
#                   return \%Values;
#               }
#----------------------------------------------------------------------------------------
# description  Returns multiple attribute values. Don't call this function directly.
#               Use $Object->get() instead.
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# input        $aNames     | list of attribute names | array ref
# input        $LanguageID | language id | int
# return       $hValues    | attribute name => value hash | hash ref
#========================================================================================
sub getAttributes {
    shift;
    my ($Object, $aNames) = @_;
    my $hInfo = $Object->dal->info($Object->id);
    Error('NoResults', {'ObjectID' => $Object->id, 'Package'=>ref($Object)}) unless defined $hInfo;
    return $hInfo;
}

#========================================================================================
# function     getAllAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->getAllAttributes( $Object, $aNames );
#----------------------------------------------------------------------------------------
# description  Returns multiple attribute values. Don't call this function directly.
#               Use $Object->get() instead.
#               The default implementation calls <function getAttributes>.
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# input        $aNames     | list of attribute names (optional) | array ref
# return       $hValues    | attribute name => value hash | hash ref
#========================================================================================
sub getAllAttributes {
    my $self = shift;
    my ($Object, $aNames) = @_;

    LogDebug('getAllAttributes', [ $Object, $aNames ]);

    my $hValues = $self->getAttributes($Object, $aNames);

    TriggerHook('API_GetAllAttributes', {
        'Object' => $Object,
        'Values' => $hValues,
        'Names' => $aNames,
    });

    return $hValues;
}

#========================================================================================
# function     setAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->setAttribute( $Object, $Name, $Value, $LanguageID );
#----------------------------------------------------------------------------------------
# description  Sets a single attribute value. Don't call this function directly.
#               Use $Object->set() instead.
#               The default implementation calls <function setAttribute>.
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# input        $Name       | attribute name  | string
# input        $Value      | attribute value | string
# input        $LanguageID | language id | int
#========================================================================================
sub setAttribute {
    my $self = shift;
    my ($Object, $Name, $Value, $LanguageID) = @_;

    $self->setAttributes( $Object, { $Name => $Value }, $LanguageID );
    return;
}

#========================================================================================
# function     setAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->setAttributes( $Object, $hValues, $LanguageID );
# example      sub setAttributes {
#                   my $self = shift;
#                   my ($Object, $hValues, $LanguageID) = @_;
#
#                   foreach my $Name (keys %$hValues) {
#                       if( $Name eq 'Name' ) {
#                           GetCurrentDBHandle()->do( 'UPDATE group SET name = ? WHERE groupid = ?',
#                               [ $hValues->{'Name'}, $Object->id ] );
#                       }
#                   }
#                   return;
#               }
#----------------------------------------------------------------------------------------
# description  Sets multiple attribute values. Don't call this function directly.
#               Use $Object->set() instead.
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# input        $hValues    | attribute name => value hash | hash ref
# input        $LanguageID | language id (optional) | integer
#========================================================================================
sub setAttributes {
    shift;
    my ($Object, $hUpdate) = @_;

    $Object->dal->update($Object->id, $hUpdate);
    return;
}

#========================================================================================
# function     deleteObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->deleteObject( $Object );
# example      sub deleteObject {
#                   my $self = shift;
#                   my ($Object) = @_;
#
#                   GetCurrentDBHandle()->do( 'DELETE FROM groupmember WHERE groupid = ?',
#                       [ $Object->id ] );
#                   GetCurrentDBHandle()->do( 'DELETE FROM group WHERE groupid = ?',
#                       [ $Object->id ] );
#                   return;
#               }
#----------------------------------------------------------------------------------------
# description  Called before the object is deleted.
#               Used to clean up any data that is related to the object.
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
#========================================================================================
sub deleteObject {}

#========================================================================================
# function     canDeleteObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->canDeleteObject( $Object, $aWarnings );
# example      sub canDeleteObject {
#                   my $self = shift;
#                   my ($Object, $aWarnings) = @_;
#
#                   my $Warning = DE_EPAGES::Core::API::Warning->new(
#                       Code => $Code,
#                       Vars => $hVars
#                   );
#                   push @$aWarnings, $Warning;
#                   return;
#               }
#----------------------------------------------------------------------------------------
# description  Tests if the object can be deleted.
#               If there is any reason why the object should not be deleted, these
#               reasons are appended to the @$aWarnings list.
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# input        $aWarnings  | array of Warning Objects | array.ref
#========================================================================================
sub canDeleteObject {}

#========================================================================================
# function     deleteAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->deleteAttribute( $Attribute );
# example      sub deleteAttribute {
#                   my $self = shift;
#                   my ($Attribute) = @_;
#
#                   if( $Attribute->alias eq 'GroupMembers' ) {
#                       GetCurrentDBHandle()->do( 'TRUNCATE TABLE groupmembers' );
#                   }
#               }
#----------------------------------------------------------------------------------------
# description  Called when the attribute is deleted from the class.
#               Used to clean up all data that is related to this attribute.
#----------------------------------------------------------------------------------------
# input        $Attribute  | attribute object | object
#========================================================================================
sub deleteAttribute {}

#========================================================================================
# function     defaultAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->defaultAttributes( $Parent, $aNames );
# example      sub defaultAttributes {
#                   my $self = shift;
#                   my ($Parent, $aNames) = @_;
#
#                   return {
#                       LocaleID => $Parent->getSite()->get('LocaleID'),
#                       IsVisible => 0,
#                   };
#               }
#----------------------------------------------------------------------------------------
# description  Returns default values for a new object.
#               It is possible to get extra information from the parent object.
#----------------------------------------------------------------------------------------
# input        $Parent | parent of new or existing object | object
# input        $aNames | attribute names | ref.array.string
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
#========================================================================================
sub defaultAttributes { return {}; }

#========================================================================================
# function     importXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->importXML($Handler, $Class, $aNames, $hAttributes);
# example      sub importXML {
#                   my $self = shift;
#                   my ($Handler, $Class, $aNames, $hAttributes) = @_;
#
#                   if( exists $hAttributes->{'Class'} ) {
#                       $hAttributes->{'ActionClass'} = LoadClassByAlias($hAttributes->{'Class'});
#                       delete $hAttributes->{'Class'};
#                   }
#               }
#----------------------------------------------------------------------------------------
# description  Converts attributes from the XML start tag to object attribute values.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $Class | class of imported object | object
# input        $aNames | attribute names | ref.array.string
# input        $hAttributes | imported attributes (attribute name => value).
#                   On input, the values are taken from the XML start tag.
#                   This function can modify the values, for example to load the
#                   object value by name. | ref.hash
#========================================================================================
sub importXML {}

#========================================================================================
# function     exportXMLAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->exportXMLAttributes($Driver, $Object, $aNames, $hAttributes);
# example      sub exportXMLAttributes {
#                   my $self = shift;
#                   my ($Driver, $Object, $aNames, $hAttributes) = @_;
#
#                   foreach my $Name (@$aNames) {
#                       if( $Name eq 'ActionClass' ) {
#                           $hAttributes->{'Class'} = $Object->get('ActionClass')->alias;
#                       }
#                   }
#               }
#----------------------------------------------------------------------------------------
# description  Adds attributes to the XML start tag when the object is exported.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver | object
# input        $Object | object to export | object
# input        $aNames | attribute names | ref.array.string
# input        $hAttributes | imported attributes | ref.hash
#========================================================================================
sub exportXMLAttributes {}

#========================================================================================
# function     exportXMLElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->exportXMLElements($Driver, $Object, $aNames, $hAttributes);
# example      sub exportXMLElements {
#                   my $self = shift;
#                   my ($Driver, $Object, $aNames, $hAttributes) = @_;
#
#                   foreach my $Name (@$aNames) {
#                       if( $Name eq 'Members' ) {
#                           $Driver->addObject('Member', $_) foreach @{$Object->get('Members')};
#                       }
#                   }
#               }
#----------------------------------------------------------------------------------------
# description  Adds sub elements to the XML file when the object is exported.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver | object
# input        $Object | object to export | object
# input        $aNames | attribute names | ref.array.string
# input        $hAttributes | imported attributes | ref.hash
#========================================================================================
sub exportXMLElements {}

#========================================================================================
# function     cloneAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->cloneAttributes( $Object, $Clone );
#----------------------------------------------------------------------------------------
# description  copy object attributes to clone.
#----------------------------------------------------------------------------------------
# input        $Object | source object | object
# input        $Clone | clone | object
#========================================================================================
sub cloneAttributes {}

1;
