#========================================================================================
# package      DE_EPAGES::Object::API::Object::Object
# state        public
# base         DE_EPAGES::Object::API::Object::DALObject
#----------------------------------------------------------------------------------------
# description  object interface for Object.
#               Includes methods for accessing attributes, child objects, and parents.
#               Serialization is supported via XML import/export.
#----------------------------------------------------------------------------------------
# example      my $Object = LoadObject( $ObjectID );
#               print $Object->class->alias;
#               print $Object->path;
#               $Object->set( { 'LastName' => 'Watson' } );
#               print $Object->get( 'LastName' );
#               $Object->delete;
#========================================================================================
package DE_EPAGES::Object::API::Object::Object;
use base DE_EPAGES::Object::API::Object::DALObject;

use strict;
use feature qw( switch );
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::File qw(
    CopyDirectory
    DeleteDirectory
    ExistsDirectory
    GetTrashRoot
    ConvertPathChars
);
use DE_EPAGES::Core::API::Log qw( LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( GenerateGUID GetCurrentDateTime Compare );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger TestString TestHash );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Constants qw( POSITION_START POSITION_STEP );
use DE_EPAGES::Object::API::Language qw( GetCurrentLanguageID );
use DE_EPAGES::Object::API::Factory qw (
    DeleteObject
    LoadClassByAlias
    LoadObject
    LoadRootObject
    GetRootObjectID
    JoinObjectPath
    SplitObjectPath
    DeleteObjectAtCache
    UpdateObjectAtCache
    SetObjectAtCache
);
use DE_EPAGES::Object::API::ObjectArray qw( ObjectArray );
use DE_EPAGES::Object::API::SavedIncludesHandler;
use DE_EPAGES::Object::API::Table::LastUpdate; # OnDeleteObject
use DE_EPAGES::Object::DAL::LastUpdate qw( DAL_SetObjectDeleted );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use Scalar::Util qw ( blessed weaken );
use URI::Escape qw( uri_escape_utf8 );

use constant MAX_ALIAS_LENGTH => 255;
use constant OBJECTARRAY_LIMIT => 1000; # larger object lists are converted to virtual arrays using ObjectArray()

sub ErrorDefinition {
    return {
    'NoResults_ChildAlias' => 'Object #Path doesn\'t contain a child alias #ChildAlias',
    'NoReferenceObject'    => 'Referenced object not found (Alias=#Alias, Path=#Path, XMLParentPath=#XMLParentPath)',
    'PathWithoutReference' => 'Missing attribute "reference=1" on tag #TagName Path=#Path XMLParentPath=#XMLParentPath',
    'NoParentObject'       => 'Missing parameter Parent (Alias=#Alias)',
    'NoAlias'              => 'Missing parameter Alias',
    'NoClass'              => 'Missing parameter Class (Alias=#Alias)',
    'PATH_NOT_FOUND'       => 'Object not found #Path (Path=#Base)',
    'REF_OBJECT_PATH_NOT_FOUND' => 'Referenced object path not found #Path at object #Alias (Parent=#Parent,AttributeName=#AttributeName)',
    'REF_OBJECT_GUID_NOT_FOUND' => 'Referenced object GUID not found #GUID at object #Alias (Parent=#Parent,AttributeName=#AttributeName)',
    'ATTRIBUTE_NOT_FOUND'  => 'Attribute #Attribute doesn\'t exist in class #Class',
    'DeleteReference'      => 'Don\'t call deleteXML on a reference (Object=#Path).',
    'CantClone'            => 'Not allowed to clone object (Object=#Path).',
    }
}

sub id { shift->{'Attributes'}{'ObjectID'} }
sub primaryKey { 'ObjectID' }

#========================================================================================
# function     alias
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Alias = $Object->alias;
#----------------------------------------------------------------------------------------
# description  Returns the alias of an object
#----------------------------------------------------------------------------------------
# return       $Alias | alias | string
#========================================================================================
sub alias { shift->{'Attributes'}{'Alias'} }

#========================================================================================
# function     guid
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GUID = $Object->guid;
#----------------------------------------------------------------------------------------
# description  Returns the GUID of an object
#----------------------------------------------------------------------------------------
# return       $GUID | GUID | string
#========================================================================================
sub guid { shift->{'Attributes'}{'GUID'} }

#========================================================================================
# function     getSite
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Site = $Object->getSite;
#----------------------------------------------------------------------------------------
# description  Returns the site of an object
#----------------------------------------------------------------------------------------
# return       $Site | site object, see <package DE_EPAGES::Object::API::Object::Site> | object
#========================================================================================
sub getSite {
    my $self = shift;

    my $Site = $self->{'Site'};
    unless (defined $Site) {
        my $SiteID = $self->{'Attributes'}{'SiteID'};
        if( $SiteID == $self->id ) {
            $Site = $self;
        }
        else {
            $Site = LoadObject( $SiteID );
            $self->{'Site'} = $Site;
            weaken $self->{'Site'};
        }
    }
    return $Site;
}

#========================================================================================
# function     requiredChildClass
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RequiredChildClass = $Object->requiredChildClass;
#----------------------------------------------------------------------------------------
# description  Returns the name of the class a new child is required to be an instance
#               of or undef if no class is allowed (no children).
#----------------------------------------------------------------------------------------
# return       $RequiredChildClass | name of a class, i.e. "Object" | String
#========================================================================================
sub requiredChildClass { return 'Object'; }

#========================================================================================
# function     load
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = DE_EPAGES::Object::API::Object->load( $ObjectID );
#               Note: Use <function DE_EPAGES::Object::API::Factory::LoadObject> instead.
#----------------------------------------------------------------------------------------
# description  Loads an existing object.
#----------------------------------------------------------------------------------------
# input        $ObjectID | object id | int
# return       $Object | existing object | object
#========================================================================================
sub load {
    my $class = shift;
    my ($ObjectID) = @_;
    TestInteger( 'ObjectID', $ObjectID );

    my $self = $class->SUPER::load($ObjectID);

    SetObjectAtCache($self);
    return $self;
}

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Object->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'Object' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $Object->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = DE_EPAGES::Object::API::Object::Object->insert($hInfo);
# example      my $Class = LoadClassByAlias('Object');
#               my $Object = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object
#                       <ul>
#                         <li>Class - class object - object</li>
#                         <li>Alias - alias - varchar(255)</li>
#                         <li>Parent - parent object - object</li>
#                         <li>GUID - (optional) general unique id - string</li>
#                         <li>Inherit  - (optional) inherit permissions from parent object; default: 1 - boolean</li>
#                         <li>Position - (optional) sort order number; default: 0 - integer</li>
#                         <li>Site - (optional) Site object; default: $hInfo->{Parent}->getSite() - object</li>
#                         <li>CreationDate - (optional) creation date - DateTime object</li>
#                       </ul> | hash ref
# return       $Object | new object | object
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    Error('NoAlias', $hInfo) unless defined $hInfo->{'Alias'};
    Error('NoClass', $hInfo) unless defined $hInfo->{'Class'};
    Error('NoParentObject', $hInfo) unless exists $hInfo->{'Parent'};

    $hInfo->{'GUID'} = GenerateGUID() unless defined $hInfo->{'GUID'};
    $hInfo->{'CreationDate'} = GetCurrentDateTime() unless defined $hInfo->{'CreationDate'};
    $hInfo->{'Inherit'} = 1 unless defined $hInfo->{'Inherit'};
    $hInfo->{'Position'} = 0 unless defined $hInfo->{'Position'};
    $hInfo->{'Site'} = $hInfo->{'Parent'}->getSite() unless defined $hInfo->{'Site'};
    Error('NoSiteObject', $hInfo) unless defined $hInfo->{'Site'};

    my $self = $class->SUPER::insert($hInfo);

    delete $hInfo->{$_} foreach (qw ( ObjectID Class Parent Alias Position GUID Inherit CreationDate Site FilePathPart )); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     reset
# state        public
# syntax       $Object->reset;
# description  if an attribute was created or removed the class must be reset
#========================================================================================
sub reset {
    my $self = shift;
    UpdateObjectAtCache($self);
    delete $self->{'RecursiveGet'};
    delete $self->{'Parent'};
    $self->SUPER::reset;

    # change the object's package if necessary
    my $NewPackage = $self->class->get('Package');
    bless $self, $NewPackage if $NewPackage ne blessed $self;
    return;
}

#========================================================================================
# function     insertChild
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NewObject = $Object->insertChild( $Class, $hInfo );
# syntax       $NewObject = $Object->insertChild( $ClassAlias, $hInfo );
# example      $NewObject = $Object->insertChild( 'User', { Alias => $Login } );
#----------------------------------------------------------------------------------------
# description  Inserts a new object with the current object as parent.
#----------------------------------------------------------------------------------------
# input        $Class | class | object
# input        $ClassAlias | class alias | string
# input        $hInfo | hash with mandatory attribute values.
#                       <ul>
#                         <li>Alias - alias - varchar(255)</li>
#                         <li>Inherit - (optional) inherit permissions from parent object - boolean</li>
#                       </ul> | hash ref
# return       $NewObject | the new object | object
#========================================================================================
sub insertChild {
    my $self = shift;
    my ($Class, $hInfo) = @_;
    $Class = LoadClassByAlias($Class) unless ref $Class;
    $hInfo->{'Parent'} = $self;
    return $Class->insertObject( $hInfo );
}

#========================================================================================
# function     uniqueChildAlias
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UniqueAlias = $Object->uniqueChildAlias( $String, $AllowedAlias );
# example      $UniqueAlias = $Object->uniqueChildAlias( 'my 5$-bill' );
# example      $UniqueAlias = $Object->uniqueChildAlias( 'my 10$-bill', 'my_10_bill' );
#----------------------------------------------------------------------------------------
# description  Returns a safe (no special chars) alias for a given string and makes it
#               unique by adding a counter, if an element with this alias already exists.
#----------------------------------------------------------------------------------------
# input        $String | string | string
# input        $AllowedString | (optional) if the created alias matches this alias, then
#               use it. | string
# return       $UniqueAlias | the unique alias | string
#========================================================================================
sub uniqueChildAlias {
    my $self = shift;
    my ($String, $AllowedAlias) = @_;
    $String = '' unless defined $String;
    # replace special chars
    $String =~ s/[^0-9a-zA-Z\_\-]+/_/gx;
    # remove leading and trailing _
    $String =~ s/^_*(.*?)_*$/$1/gx;
    $String = 'Alias' if ($String eq '');
    my $UniqueAlias = $String;
    my $Counter = 1;
    while ((!defined($AllowedAlias) || ($UniqueAlias ne $AllowedAlias)) && $self->existsChild($UniqueAlias)) {
        # adds the increment on the right hand side of the alias and ensures that it
        # will not longer than 255 chars...
        $UniqueAlias = substr($String, 0, MAX_ALIAS_LENGTH - length($Counter)) . $Counter++;
    }
    return $UniqueAlias;
}

#========================================================================================
# function     canClone
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->canClone();
#----------------------------------------------------------------------------------------
# description  Check if a object can be cloned.
#----------------------------------------------------------------------------------------
# return       $CanClone | true or false | boolean
#========================================================================================
sub canClone {
    return 1;
}

#========================================================================================
# function     clone
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Clone = $Object->clone( $hInfo );
# syntax       $Clone = $Object->clone( $hInfo, $Recursive );
#----------------------------------------------------------------------------------------
# description  Inserts a new object into the database as copy from old object.
#----------------------------------------------------------------------------------------
# input        $hInfo | hash with attribute values that are different in the clone object.
#                       Note that the clone cannot have the same Parent and the same Alias.
#                    | hash ref
# input        $Recursive | clone child object too (optional, default is false) | boolean
# return       $Clone | the new object | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo, $Recursive) = @_;

    Error('CantClone', { Path => $self->pathString }) unless $self->canClone;

    $Recursive = 0 unless defined $Recursive;

    my $hAttributes = $self->exportableAttributes;
    $hAttributes->{'Parent'} = $self->parent;
    delete $hAttributes->{'GUID'};
    delete $hAttributes->{'FilePathPart'} unless $self->getSite()->get('IsFilePathPartUsed');

    # split old attributes in localizable and non loc.
    my $Class = $self->class;
    my %localAttributes;
    foreach my $Attribute ( keys %$hAttributes ) {
        my $Type = $Class->attribute($Attribute)->type;
        if( $Type =~ /^Localized(String|File)$/ ) {
            foreach my $LanguageID (keys %{$hAttributes->{$Attribute}}) {
                $localAttributes{$LanguageID}{$Attribute} = $hAttributes->{$Attribute}{$LanguageID};
            }

            delete $hAttributes->{$Attribute};
        }
    }

    # split new attributes in localizable and non loc.
    foreach my $Attribute (keys %$hInfo) {
        my $Type = $Class->attribute($Attribute)->type;
        if( $Type =~ /^Localized(String|File)$/ ) {
            foreach my $LanguageID (keys %{$hInfo->{$Attribute}}) {
                $localAttributes{$LanguageID}{$Attribute} = $hInfo->{$Attribute}{$LanguageID};
            }
        }
        else {
            $hAttributes->{$Attribute} = $hInfo->{$Attribute};
        }
    }

    # Site must always correspond to the parent
    $hAttributes->{'Site'} = $hAttributes->{'Parent'}->getSite;

    # create clone
    my $Clone = $self->class->insertObject($hAttributes);
    $Clone->set($localAttributes{$_}, $_) foreach keys %localAttributes; # set localized attributes

    # clone directories (public and private)
    $self->cloneFiles( $Clone );

    $self->class->cloneAttributes($self, $Clone);

    # clone childs
    my $Inherit = defined $hInfo->{'Inherit'} ? $hInfo->{'Inherit'} : 1;
    if ($Recursive) {
        foreach my $Child (@{$self->cloneChildren}) {
            next unless $Child->canClone();
            $Child->clone({'Parent' => $Clone, 'Inherit' => $Inherit}, $Recursive);
        }
    }
    return $Clone;
}

#========================================================================================
# function     cloneChildren
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aChildren = $Object->cloneChildren
#----------------------------------------------------------------------------------------
# example      $Description = $Object->cloneChildren;
#----------------------------------------------------------------------------------------
# description  Returns all children of the object which should be cloned.
#----------------------------------------------------------------------------------------
# return       $aChildren | children of the object | array ref
#========================================================================================
sub cloneChildren {
    my $self = shift;

    return $self->children;
}

#========================================================================================
# function     cloneFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->cloneFiles( $Clone );
#----------------------------------------------------------------------------------------
# description  Copies directories (PublicPath and PrivatePath) from original object to the clone.
#               This function will is called by <function clone>.
#----------------------------------------------------------------------------------------
# input        $Clone | clone object | object
#========================================================================================
sub cloneFiles {
    my $self = shift;
    my ($Clone) = @_;

    CopyDirectory( $self->get('PublicPath'),  $Clone->get('PublicPath'),  undef, { recursive => 0 })
                    if   ExistsDirectory($self->get('PublicPath'))
                     and $self->get('PublicPath')  ne $Clone->get('PublicPath');
    CopyDirectory( $self->get('PrivatePath'), $Clone->get('PrivatePath'), undef, { recursive => 0 })
                    if ExistsDirectory($self->get('PrivatePath'))
                     and $self->get('PrivatePath') ne $Clone->get('PrivatePath');
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object with all attributes and child objects.
#               Triggers the hook 'OBJ_DeleteObject' and any specialized OBJ_Delete* hooks.
#               Calls the methods <function deleteAttributes>, <function deleteChildren>,
#               <function deleteFiles> and <function deleteSelf> in that order.
#               Only <function deleteSelf> is executed in a transaction.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteObject | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ObjectID      - object id            - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    my $ObjectID = $self->id;

    # security check
    die 'can\'t delete system object '.$ObjectID if $ObjectID < 13; ## no critic qw(ProhibitMagicNumbers)

    # delete all dependencies
    TriggerHook('OBJ_DeleteObject', {'Object' => $self, 'ObjectID' => $ObjectID });

    DE_EPAGES::Object::API::Table::LastUpdate::OnDeleteObject({ ObjectID => $ObjectID });

    # mark page cache
    $self->updatePageCache;

    # delete the object itself starting with files
    $self->deleteFiles;

    # delete references to other objects
    $self->deleteAttributes;

    # delete all child objects
    $self->deleteChildren;

    TriggerHook('OBJ_AfterDeleteChildObject', {'Object' => $self, 'ObjectID' => $ObjectID });

    # real delete object, calls deleteSelf()
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     deleteSelf
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->deleteSelf;
#----------------------------------------------------------------------------------------
# description  Deletes the object from database tables. Called by <function delete>.
#========================================================================================
sub deleteSelf {
    my $self = shift;

    # delete the object from cache and from its parent cache
    DeleteObjectAtCache($self);
    $self->SUPER::deleteSelf;
    return;
}

#========================================================================================
# function     deleteFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->deleteFiles;
#----------------------------------------------------------------------------------------
# description  Deletes the directories PublicPath, PrivatePath.
#               If Site.IsFilePathPartUsed and defined FilePathPart the directories will not deleted.
#               Delete the directory StaticPath.
#========================================================================================
sub deleteFiles {
    my $self = shift;

    my $DeleteDirectories = (
        defined($self->get('FilePathPart')) and
        $self->getSite()->get('IsFilePathPartUsed')
    ) ? 0 : 1;
    my $StoreName = LoadRootObject()->get('StoreName');

    DeleteDirectory( $self->get('PublicPath'), {'recursive' => 1, 'move' => 1}, GetTrashRoot($self->get('PublicPath')).'/'.$StoreName) if $DeleteDirectories;
    DeleteDirectory( $self->get('PrivatePath'), {'recursive' => 1, 'move' => 1}, GetTrashRoot($self->get('PrivatePath')).'/'.$StoreName) if $DeleteDirectories;
    DeleteDirectory( $self->get('StaticPath'),  {'recursive' => 1, 'move' => 1}, GetTrashRoot($self->get('StaticPath')) .'/'.$StoreName);

    return;
}

#========================================================================================
# function     deleteChildren
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->deleteChildren;
#----------------------------------------------------------------------------------------
# description  Deletes all child objects of this object.
#========================================================================================
sub deleteChildren {
    my $self = shift;

    # delete objects in reverse order of export
    DeleteObject( $_ ) foreach reverse @{ $self->findChildren };
    return;
}

#========================================================================================
# function     set
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->set($hValues, $LanguageID);
# example      $Object->set({ 'LocaleID' => 'de_DE' });
# example      $Object->set({ 'Name' => 'Fuball' }, GetPKeyLanguageByCode('de'));
# example      $Object->set({ 'Name' => 'Soccer' }, GetPKeyLanguageByCode('en'));
#----------------------------------------------------------------------------------------
# description  Sets the given attribute values on the object.
#               This function triggers hooks 'OBJ_BeforeUpdateObject' and 'OBJ_AfterUpdateObject'
#               and any specialized OBJ_BeforeUpdate* and OBJ_AfterUpdate* hooks.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateObject | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ObjectID      - object id            - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateObject | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ObjectID      - object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID, $hAdditionalOptions) = @_;
    TestHash('Values', $hValues);
    $LanguageID = GetCurrentLanguageID() unless defined $LanguageID;

    # remember if the FilePathPart or ChangedGUID has changed to change FileAlias
    my $ChangedFilePathPart = 0;
    my $ChangedGUID  = 0;

    # update children cache if parent or alias was changed
    if (exists $hValues->{'Parent'}) {
        my $Parent = $self->parent or die 'Can\'t change the parent of the root object';
        die 'Can\'t change the parent to me ' .  $Parent->id if $Parent->id == $self->id;
        die 'Can\'t change the parent to me ' . $hValues->{'Parent'}->id if $hValues->{'Parent'}->isChildOf($self);
        my $ChangedParent = $Parent->id != $hValues->{'Parent'}->id;
        delete $hValues->{'Parent'} unless $ChangedParent;
    } # else no parent change
    if (exists $hValues->{'GUID'}) {
        my $GUID = $self->get('GUID');
        $ChangedGUID = $GUID ne $hValues->{'GUID'};
        delete $hValues->{'GUID'} unless $ChangedGUID;
    } # else no GUID change
    if (exists $hValues->{'FilePathPart'}) {
        my $FilePathPart = $self->get('FilePathPart');
        $ChangedFilePathPart = Compare('String', $FilePathPart, $hValues->{'FilePathPart'});
        delete $hValues->{'FilePathPart'} unless $ChangedFilePathPart;
    } # else no FilePathPart change
    delete $self->{'Site'} if exists $hValues->{'Site'};

    # remember the old directory names, because they will be renamed
    my ($OldStaticPath) = $ChangedFilePathPart || $ChangedGUID ? $self->get('StaticPath') : undef;

    $self->SUPER::set($hValues, $LanguageID, $hAdditionalOptions);

    # update cached class if the class has changed
    if( exists $hValues->{'Class'} or exists $hValues->{'Parent'} ) {
        $self->reset;
    }

    # delete the object directory if the path has changed
    if( $ChangedFilePathPart || $ChangedGUID ) {
        $self->_deleteAttribute('FileAlias'); ##no critic (ProhibitPrivateSubroutineUse)
        DeleteDirectory( $OldStaticPath , {'recursive' => 1});
    }

    UpdateObjectAtCache($self);
    return;
}

#========================================================================================
# function     findChildren
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aChildObjectIDs = $Object->findChildren;
#----------------------------------------------------------------------------------------
# description  Returns a list of all child object ids of an object ordered by Position
#               and Alias.
#----------------------------------------------------------------------------------------
# return       $aChildObjectIDs | list of child object ids | array ref
#========================================================================================
sub findChildren {
    my $self = shift;

    return [
        map  { $_->{'ObjectID'} }
        # sort by Position and if Position is equal, sort by Alias
        sort { $a->{'Position'} <=> $b->{'Position'} || $a->{'Alias'} cmp $b->{'Alias'} }
        @{__PACKAGE__->dal->get('ByParentID', [$self->id])}
    ];
}

#========================================================================================
# function     findChild
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ChildObjectID = $Object->findChild( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns the object id of a child object or undef if the child alias does
#               not exist.
#----------------------------------------------------------------------------------------
# input        $Alias | child object alias | string
# return       $ChildObjectID | child object id | int
#========================================================================================
sub findChild {
    my $self = shift;
    my ($Alias) = @_;

    my $Children = __PACKAGE__->dal->get('ByParentIDAlias', [$self->id, $Alias]);
    return scalar @$Children ? $Children->[0]{'ObjectID'} : undef;
}

#========================================================================================
# function     parent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ParentObject = $Object->parent;
#----------------------------------------------------------------------------------------
# description  Returns the direct parent object, Only the System object will return
#               undef, because it does not have a parent object.
#----------------------------------------------------------------------------------------
# return       $ParentObject | parent object | object
#========================================================================================
sub parent {
    my $self = shift;

    my $Parent = $self->{'Parent'};
    unless( defined $Parent ) {
        my $ParentID = $self->{'Attributes'}{'ParentID'};
        $Parent = LoadObject($ParentID) if defined $ParentID;
        $self->{'Parent'} = $Parent;
        weaken $self->{'Parent'};
    }
    return $Parent;
}

#========================================================================================
# function     findParent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ParentID = $Object->findParent;
#----------------------------------------------------------------------------------------
# description  Returns the direct parent object identifier, Only the System object will return
#               undef, because it does not have a parent object.
#----------------------------------------------------------------------------------------
# return       $ParentID | parent object identifier | integer
#========================================================================================
sub findParent { $_[0]->{'Attributes'}{'ParentID'} } ## no critic qw(RequireArgUnpacking)

#========================================================================================
# function     parentByClass
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $Object->parentByClass($Class);
# syntax       $Object = $Object->parentByClass($ClassAlias);
# example      $Shop = $Object->parentByClass( LoadClassByAlias('Shop') );
#----------------------------------------------------------------------------------------
# description  Returns a parent object which is instance of class.
#----------------------------------------------------------------------------------------
# input        $Class | class object | object
# input        $ClassAlias | class alias | string
# return       $Object | parent object | object
#========================================================================================
sub parentByClass {
    my $self = shift;
    my ($Class) = @_;
    $Class = LoadClassByAlias( $Class ) unless ref $Class;
    my $Parent = $self->parent;
    return undef unless defined $Parent;
    return $Parent if $Parent->instanceOf($Class);
    die 'self eq parent '.$self->id if $Parent->id == $self->id; # no deep recursion
    return $Parent->parentByClass($Class);
}

#========================================================================================
# function     parentPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aParentObjects = $Object->parentPath;
#----------------------------------------------------------------------------------------
# description  Returns the path of objects from the system root to this object.
#               The System object is the first list item and this object is the last
#               item in the list.
#----------------------------------------------------------------------------------------
# return       $aParentObjects | list of parent objects | ref.array.object
#========================================================================================
sub parentPath {
    my $self = shift;

    my @Path;
    my $Object = $self;
    while ( defined $Object ) {
        unshift @Path, $Object;
        $Object = $Object->parent;
        die 'self parent cycle '.$self->id if defined $Object and $Object->id == $self->id; # no deep recursion
    }
    return \@Path;
}

#========================================================================================
# function     findParentPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aParentIDs = $Object->findParentPath;
#----------------------------------------------------------------------------------------
# description  Returns the path of object identifier from the system root to this object.
#               The System object is the first list item and this object is the last
#               item in the list.
#----------------------------------------------------------------------------------------
# return       $aParentIDs | list of parent objects | ref.array.int
#========================================================================================
sub findParentPath {
    my $self = shift;

    my @Path;
    my $Object = $self;
    while ( defined $Object ) {
        unshift @Path, $Object->id;
        $Object = $Object->parent;
    }
    return \@Path;
}

#========================================================================================
# function     pathString
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Path = $Object->pathString;
#----------------------------------------------------------------------------------------
# description  Returns the path of object as a string joined with '/'.
#----------------------------------------------------------------------------------------
# return       $Path | list of parent path | string
#========================================================================================
sub pathString {
    my $self = shift;

    my $aPath = $self->parentPath;
    shift @$aPath; # remove System
    return '/'. JoinObjectPath([ map { $_->alias } @$aPath ]);
}

#========================================================================================
# function     findPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ObjectID = $Object->findPath($Path);
#----------------------------------------------------------------------------------------
# description  find the object via path started from current object. the path can be absolute "/Object1/Object2"
#               or relative "Store/Catalog/Product" to the object.
#----------------------------------------------------------------------------------------
# input        $Path | path | string
# return       $ObjectID | object identifier | integer
#========================================================================================
sub findPath {
    my $self = shift;
    my ($Path) = @_;
    TestString('Path', $Path);

    my $aAliases = SplitObjectPath( $Path );
    die "ObjectPath '$Path' is empty" if 0 == scalar @$aAliases;
    Error('PATH_NOT_FOUND', {'Path'=>$Path,'Base'=>''}) unless defined $aAliases->[0];

    # /fromRoot
    my $Target = $self;
    if ($aAliases->[0] eq '') {
        shift @$aAliases;
        $Target = LoadRootObject();
    }
    foreach my $Alias ( @$aAliases ) {
        next if $Alias eq '' or $Alias eq '.';
        if ($Alias eq '..') {
            $Target = $Target->parent;
        } else {
            return undef unless $Target->existsChild( $Alias );
            $Target = $Target->child( $Alias );
        }
        return undef unless defined $Target;
    }
    return $Target->id;
}

#========================================================================================
# function     path
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PathObject = $Object->path($Path);
# example      $User = $Order->path('../../Users/max_mustermann');
#----------------------------------------------------------------------------------------
# description  find the object via path started from current object. the path can be absolute "/Object1/Object2"
#               or relative "Store/Catalog/Product" to the object.
#----------------------------------------------------------------------------------------
# input        $Path | path | string
# return       $PathObject | object | object
#========================================================================================
sub path {
    my $self = shift;
    my ($Path) = @_;
    my $ObjectID = $self->findPath($Path);
    Error('PATH_NOT_FOUND', {'Path'=> $Path, 'Base'=> $self->pathString}) unless defined $ObjectID;
    return LoadObject( $ObjectID );
}

#========================================================================================
# function     child
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ChildObject = $Object->child( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns the direct child object with the given alias name.
#               Throws an error if no such child object exists.
#----------------------------------------------------------------------------------------
# input        $Alias | child object alias | string
# return       $ChildObject | child object | object
#========================================================================================
sub child {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'Alias', $Alias );

    my $ChildObjectID = $self->findChild( $Alias );
    Error( 'NoResults_ChildAlias', {
        ObjectID => $self->id,
        Alias => $self->alias,
        ChildAlias => $Alias,
        Path => $self->pathString
    } ) unless defined $ChildObjectID;

    return LoadObject( $ChildObjectID );
}

#========================================================================================
# function     hasChildren
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Has = $Object->hasChildren;
#----------------------------------------------------------------------------------------
# description  Returns true if the the object has children.
#----------------------------------------------------------------------------------------
# return       $Has | true if one child exists | boolean
#========================================================================================
sub hasChildren {
    my $self = shift;
    return scalar @{$self->findChildren}? 1 : 0;
}

#========================================================================================
# function     existsChild
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $Object->existsChild( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns true if the direct child object with the given alias name exists.
#----------------------------------------------------------------------------------------
# input        $Alias | child object alias | string
# return       $Exists | true if the child object exists | boolean
#========================================================================================
sub existsChild {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'Alias', $Alias );

    return defined $self->findChild( $Alias );
}

#========================================================================================
# function     children
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aChildObjects = $Object->children;
#----------------------------------------------------------------------------------------
# description  Returns all direct child objects ordered by Position and Alias.
#----------------------------------------------------------------------------------------
# return       $aChildObjects | list of child objects | array refobject
#========================================================================================
sub children {
    my $self = shift;

    my $aChildObjectIDs = $self->findChildren;

    return ( scalar @$aChildObjectIDs < OBJECTARRAY_LIMIT )
        ? [ map { LoadObject($_) } @$aChildObjectIDs ]
        : ObjectArray( $aChildObjectIDs );
}

#========================================================================================
# function     childrenIterator
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ChildObjects = $Object->childrenIterator;
#               while( my $Child = $ChildObjects->next ) {
#                   ...
#               }
# syntax       $ChildObjects = $Object->childrenIterator( $Direction );
#               while( my $Child = <$ChildObjects> ) {
#                   ...
#               }
#----------------------------------------------------------------------------------------
# description  Returns all direct child objects ordered by Position and Alias.
#----------------------------------------------------------------------------------------
# input        $Direction | (optional, default: 'ASC') sort order (ASC|DESC) | string
# return       $ChildObjects | list of child objects | iterator
#========================================================================================
sub childrenIterator {
    my $self = shift;
    my ($Direction) = @_;
    $Direction //= 'ASC';

    # sort by Position and if Position is equal, sort by Alias
    my $aOrderBy = [
        { Column => 'Position', Direction => $Direction },
        { Column => 'Alias', Direction => $Direction }
    ];

    return DE_EPAGES::Core::API::Iterator->wrap(
        Source => __PACKAGE__->dal->iterator('ByParentID', [ $self->id ], $aOrderBy),
        Handle => sub { LoadObject( $_[1] ) },
    );
}

#========================================================================================
# function     childrenByClass
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aChildObjects = $Object->childrenByClass($Class);
# syntax       $aChildObjects = $Object->childrenByClass($ClassAlias);
#----------------------------------------------------------------------------------------
# description  Returns all direct child objects that are instances of the given class.
#----------------------------------------------------------------------------------------
# input        $Class | class object | object
# input        $ClassAlias | class alias | string
# return       $aChildObjects | list of child objects | array refobject
#========================================================================================
sub childrenByClass {
    my $self = shift;
    my ($Class) = @_;
    $Class = LoadClassByAlias( $Class ) unless ref $Class;
    return [ grep { $_->instanceOf($Class) } @{$self->children} ];
}

#========================================================================================
# function     folder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Folder = $Object->folder( $Alias );
# syntax       $Folder = $Object->folder( $Alias, $hInfo );
#----------------------------------------------------------------------------------------
# example      $Orders = $Shop->folder( 'Orders' );
# example      $Orders = $Shop->folder( 'Orders', { 'Position' => 50 } );
#----------------------------------------------------------------------------------------
# description  Returns the child object by alias name. If the child object does not
#               exist, a new one is created with the given alias and class 'Object'.
#----------------------------------------------------------------------------------------
# input        $Alias | child alias | string
# input        $hInfo | additional attribute values for creating the new object
#                           (optional) | ref.hash
# return       $Folder | child object | object
#========================================================================================
sub folder {
    my $self = shift;
    my ($Alias, $hInfo) = @_;
    TestString( 'Alias', $Alias );
    $hInfo = {} unless defined $hInfo;

    my $ChildID = $self->findChild( $Alias );
    return LoadObject($ChildID) if defined $ChildID;
    my $Class = defined $hInfo->{'Class'} ? $hInfo->{'Class'} : LoadClassByAlias('Object');
    return $Class->insertObject({ %$hInfo, 'Alias' => $Alias, 'Parent'=> $self });
}

#========================================================================================
# function     get
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $Object->get( $Name, $LanguageID );
# syntax       $hValues = $Object->get( $aNames, $LanguageID );
#----------------------------------------------------------------------------------------
# example      $Password = $Object->get( 'Password' );
# example      $hCredentials = $Object->get( ['Login', 'Password'] );
#               printf "Login: %s\n", $hCredentials->{'Login'};
#               printf "Password: %s\n", $hCredentials->{'Password'};
#----------------------------------------------------------------------------------------
# description  Returns one or more attribute values.
#----------------------------------------------------------------------------------------
# input        $Name    | attribute name | string
# input        $aNames  | list of attribute names| array ref
# input        $LanguageID | language identifier (optional) | integer
# return       $hValues | name => value pairs | hash ref
# return       $Value   | attribute value | string (or other)
#========================================================================================
sub get {
    my $self = shift;
    my ($aNames, $LanguageID) = @_;

    if (ref $aNames ne 'ARRAY' ) {
        if ($aNames eq 'Alias') {
            return $self->alias;
        }
        if ($aNames eq 'CreationDate') {
            my $date = $self->{'Attributes'}{'CreationDate'};
            if (not ref $date) {
                $date = GetCurrentDBHandle()->unquoteDateTime($date);
                $self->{'Attributes'}{'CreationDate'} = $date;
            }
            return $date;
        }
        return $self->SUPER::get($aNames, $LanguageID);
    } else {
        my $hValues = $self->SUPER::get($aNames, $LanguageID);
        if (defined $hValues->{'CreationDate'} and not ref $hValues->{'CreationDate'}) {
            my $dbi = GetCurrentDBHandle();
            $hValues->{'CreationDate'} = $dbi->unquoteDateTime($hValues->{'CreationDate'});
            $self->{'Attributes'}{'CreationDate'} = $hValues->{'CreationDate'};
        }
        return $hValues;
    }
}

#========================================================================================
# function     exportableAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Object->exportableAttributes;
# syntax       $hValues = $Object->exportableAttributes($Driver, $DriverExportLevel);
#----------------------------------------------------------------------------------------
# example      $Description = $Object->exportableAttributes;
#----------------------------------------------------------------------------------------
# description  Returns all attribute values that have an existing (not-null) value.
#               Unlike $Object->_attributes this function returns localized attribute
#               values as hash reference (language id => value).
#----------------------------------------------------------------------------------------
# input        $Driver | (optional) XML export driver | DE_EPAGES::Object::API::XML::ObjectExportDriver
# input        $DriverExportLevel | (optional) current export level | integer
# return       $hValues | object attributes as key => value pairs | hash ref
#========================================================================================
sub exportableAttributes {
    my $self = shift;
    my ($Driver, $DriverExportLevel) = @_;

    my $hValues = $self->SUPER::exportableAttributes($Driver, $DriverExportLevel);

    if( exists $hValues->{'CreationDate'} and not ref $hValues->{'CreationDate'} ) {
        $hValues->{'CreationDate'} = GetCurrentDBHandle()->unquoteDateTime($hValues->{'CreationDate'});
    }
    return $hValues;
}

#========================================================================================
# function     findXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ObjectID = $Object->findXML( $hAttributes )
#----------------------------------------------------------------------------------------
# example      my $ObjectID = $Object->findXML( { 'GUID' => '12345' } );
#----------------------------------------------------------------------------------------
# description  Returns object identifier if the attributes matches. if no object match
#               the function returns undef
#----------------------------------------------------------------------------------------
# input        $hAttributes | keys
#               <ul>
#                   <li>Alias
#                   <li>Path
#                   <li>GUID
#               </ul>
#               | ref.hash.string
# return       $ObjectID | object identifier (or undef) | integer
#========================================================================================
sub findXML {
    my $self = shift;
    my ($hAttributes) = @_;

    # get object (undef if not exists)

    # GUID
    if (defined $hAttributes->{'GUID'}) {
        my $ahInfo = __PACKAGE__->dal->get('ByGUID', [$hAttributes->{'GUID'}]);
        return $ahInfo->[0]->{'ObjectID'} if scalar @$ahInfo;
    }
    # Path
    return GetRootObjectID() if defined $hAttributes->{'Path'} and $hAttributes->{'Path'} eq '/';
    return $self->findPath($hAttributes->{'Path'}) if defined $hAttributes->{'Path'};

    # Alias and Parent
    if (defined $hAttributes->{'Alias'} and defined $hAttributes->{'Parent'}) {
        if (blessed( $hAttributes->{'Parent'})) {
            return $hAttributes->{'Parent'}->findChild($hAttributes->{'Alias'});
        } elsif ($hAttributes->{'Parent'} =~ /^\//) {
            return $self->path($hAttributes->{'Parent'})->findChild($hAttributes->{'Alias'});
        }
    }
    return $self->findChild($hAttributes->{'Alias'}) if defined $hAttributes->{'Alias'};

    return undef;
}

#========================================================================================
# function     deleteXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = DE_EPAGES::Object::API::Object::Object->deleteXML($Handler, $hElement, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Called by the SAX driver when a start tag for the current object class
#               is encountered. The function identifies the object by the attributes
#               in $hAttributes and deletes the object if it still exists and and the
#               attribute 'delete="1"' is specified.
#----------------------------------------------------------------------------------------
# input        $ObjectPackage | package name of the objects's class | string
# input        $Handler | xml delete handler | object
# input        $hElement | information about the start tag, see SAX function start_element | ref.hash
# input        $hAttributes | attributes of the start tag | ref.hash
# return       $Object | the found object if it still exists | object
#========================================================================================
sub deleteXML {
    my ($ObjectPackage, $Handler, $hElement, $hAttributes) = @_;

    my $XMLParentObject = $Handler->topID('Object');

    my $Parent;
    if (defined $hAttributes->{'Parent'}) {
        $Parent = blessed  $hAttributes->{'Parent'} ? $hAttributes->{'Parent'} : $XMLParentObject->path($hAttributes->{'Parent'});
    } else {
        $Parent = $XMLParentObject;
    }

    # parent is already deleted
    return undef unless defined $Parent;

    my $ObjectID = $Parent->findXML($hAttributes);
    return undef unless defined $ObjectID;

    my $Object = LoadObject($ObjectID);
    return $Object unless defined $hAttributes->{'delete'};

    if ($Handler->topID('IsReference')) {
        Warning('DeleteReference', {
            'Path' => $Object->pathString,
            'Alias' => $Object->alias,
        });
        return $Object;
    }
    $Object->delete;
    return undef;
}

#========================================================================================
# function     importXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $Package->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Imports an object from an XML file. This function is called at the XML
#               start tag. The end tag triggers the <function importXMLEnd> function.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject
#                       | DE_EPAGES::Object::API::XML::ObjectImportHandler
# input        $hElement | XML element structure, see XML::SAX | ref.hash
# input        $Class | class object; the new object will be an instance of this class
#                       | DE_EPAGES::Object::API::Object::Class
# input        $hAttributes | attributes hash | ref.hash
# return       $Object | the imported object | DE_EPAGES::Object::API::Object::Object
#========================================================================================
sub importXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    my $XMLParentObject = $Handler->topID('Object') // LoadRootObject();

    my $Parent = defined $hAttributes->{'Parent'}
        ? blessed $hAttributes->{'Parent'}
            ? $hAttributes->{'Parent'}
            : $XMLParentObject->path($hAttributes->{'Parent'})
        : $XMLParentObject;

    # if reference of object
    my $ObjectID;
    if ($Handler->topID('IsReference')) {
        $ObjectID = $XMLParentObject->findXML($hAttributes);

        # get object (undef if not exists)
        Error( 'NoReferenceObject', {
            TagName => $hElement->{'LocalName'},
            Alias => defined $hAttributes->{'Alias'} ? $hAttributes->{'Alias'} : '',
            Path  => defined $hAttributes->{'Path'} ? $hAttributes->{'Path'} : '',
            XMLParentPath => $XMLParentObject->pathString
        } ) unless defined $ObjectID;

        delete $hAttributes->{'Path'};   # Path is used to identify the object (is no attribute)
    }
    else {
        if( exists $hAttributes->{'Path'} ) {
            Error('PathWithoutReference', {
                TagName => $hElement->{'LocalName'},
                Path => $hAttributes->{'Path'},
                XMLParentPath => $XMLParentObject->pathString
            });
        }
        Error('NoParentObject', $hAttributes) unless defined $Parent;
        Error('NoClass', $hAttributes)        unless defined $Class and blessed $Class;
        $ObjectID = $Parent->findXML($hAttributes);
        $hAttributes->{'Parent'} = $Parent;
    }

    $class->importXMLAttributes( $Handler, $Class, $XMLParentObject, $hAttributes );

    # set attributes
    LogDebug('Attributes',$hAttributes);
    if (defined $ObjectID) {
        my $Object = LoadObject( $ObjectID );
        $Object->set( $hAttributes );
        return $Object;
    }
    else {
        return $Class->insertObject($hAttributes);
    }
}

sub exportXMLDefaultAttributes {
    my $self = shift;
    my ($hAttributes) = @_;
    my $Parent = $self->parent ? $self->parent : $self;
    return $self->class->defaultAttributes($Parent, $hAttributes);
}

#========================================================================================
# function     exportXMLAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes localized attributes (deletes attribues like ParentID, Inherit if default (1), Class if equal TagName).
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    $self->SUPER::exportXMLAttributes($Driver, $hAttributes);

    my $DriverExportLevel = $Driver->exportLevel;
    my $Class = $self->class;
    if ($Class->get('MajorExportLevel') < $DriverExportLevel ) {
        $hAttributes->{'reference'} = 1;
        my $XMLParentObject = $Driver->topID('XMLParentObject');
        if (defined $XMLParentObject) {
            if ($XMLParentObject->id == 1 or $XMLParentObject->id == $self->parent->id) {
                $hAttributes->{'Alias'} = $self->alias;
            } else {
                $hAttributes->{'Path'} = $XMLParentObject->relativePath($self);
            }
        } else {
            $hAttributes->{'Path'} = $self->pathString;
        }
    } else {
        my $ClassName = $Class->alias;
        $hAttributes->{'Class'} = $ClassName if $ClassName ne $Driver->tagName;
        if ($self->parent and $Driver->xmlParentObject and $Driver->xmlParentObject->id != $self->parent->id){
            my $XMLParentObject = $Driver->topID('XMLParentObject');
            $hAttributes->{'Parent'} = defined $XMLParentObject ? $XMLParentObject->relativePath($self->parent) : $self->parent->pathString;
        } else {
            delete $hAttributes->{'Parent'};
        }
    }

    return $hAttributes;
}

#========================================================================================
# function     planXMLExport
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->planXMLExport( $Driver );
#----------------------------------------------------------------------------------------
# description  Add this object to driver (possible for nextLevel).
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
#========================================================================================
sub planXMLExport {
    my $self = shift;
    my ($Driver) = @_;

    # has children or max attribute major export level >= current export level
    my $Class = $self->class;
    my $ClassMajorExportLevel = $Class->get('MajorExportLevel');
    return unless defined $ClassMajorExportLevel;

    # eg customer order has level 2 or any attribute has higher level like references
    my $DriverExportLevel = $Driver->exportLevel;
    if ( $DriverExportLevel < $ClassMajorExportLevel) {
        $Driver->addNextLevelItem($self, $ClassMajorExportLevel);
    } else {
        if ($DriverExportLevel == $ClassMajorExportLevel) {
            $Driver->addObject($self->tagNameXML, $self);
        }
        if ($DriverExportLevel < $Class->get('MaxMajorExportLevel') and
            $ClassMajorExportLevel <= $DriverExportLevel ) {
            my $hAttributes = $self->exportableAttributes($Driver, $DriverExportLevel + 1);
            $Driver->addNextLevelItem($self, $DriverExportLevel + 1) if scalar keys %$hAttributes;
        }
    }
    return;
}

#========================================================================================
# function     exportXMLChildElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->exportXMLChildElements( $Driver, $hAttributes );
#----------------------------------------------------------------------------------------
# description  Add xml elements for children (asks each with planXMLExport to add to driver).
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLChildElements {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    # do only export child objects if ExportRecursive
    if($Driver->{'ExportRecursive'}) {
        $_->planXMLExport($Driver) foreach sort {
            $a->get('Position')     <=> $b->get('Position') ||
            $a->class->alias cmp $b->class->alias ||
            $a->alias        cmp $b->alias
        } @{$self->exportableChildren};
    }
    return;
}

#========================================================================================
# function     exportXMLElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->exportXMLElements($Driver, $hAllAttributes);
#----------------------------------------------------------------------------------------
# description  Exports multilinguagal attribute value of this object to the xml SAX driver.
#----------------------------------------------------------------------------------------
# input        $Driver | export driver | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    # export language tags
    $self->exportXMLLanguageElements($Driver, $hAttributes);

    # export children
    $self->exportXMLChildElements($Driver, $hAttributes);

    # add additional attributes from attribute packages
    $self->class->exportXMLElementsPackages($Driver, $self, $hAttributes);
    return;
}

#========================================================================================
# function     nextAlias
#----------------------------------------------------------------------------------------
# syntax       $Alias = $Object->nextAlias;
# syntax       $Alias = $Object->nextAlias( $Format );
# example      $Alias = $Object->nextAlias( "P_%s" );
#----------------------------------------------------------------------------------------
# description  Returns the an the next unused alias for a new object using a format
#               pattern.
#----------------------------------------------------------------------------------------
# input        $Format | sprintf-style format string (default "Object_%4d") | string
# return       $Alias | next free alias string  | string
#========================================================================================
sub nextAlias {
    my $self = shift;
    my ($format) = @_;

    $format = 'Object_%4d' unless defined $format;
    Error('FALSE_Format', { 'Format' => $format, 'Correct'=> 'Object_%4d' }) unless $format =~ /%\d*(s|d|f)/;
    # find new alias
    my $StartCounter = 1 + scalar @{$self->findChildren};
    my $Alias = sprintf $format, $StartCounter;
    while( defined $self->findChild($Alias) ) {
        my $NewAlias = sprintf $format, ++$StartCounter;
        if( $NewAlias eq $Alias ) {
            die sprintf 'Can\'t create a unique alias with format \'%s\' and counter %d for object %s',
                $format, $StartCounter, $self->get('Path');
        }
        $Alias = $NewAlias;
    }
    return $Alias;
}

#========================================================================================
# function     renumberChildren
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->renumberChildren;
#----------------------------------------------------------------------------------------
# description  Updates the Position attribute of all child objects with a distance of 10
#               between subsequent objects. The new position number will be 10, 20, ....
#========================================================================================
sub renumberChildren {
    my $self = shift;

    my $aChildren = $self->children;
    my $Position = POSITION_START;
    foreach my $Object ( @$aChildren ) {
        $Object->set( { Position => $Position } );
        $Position += POSITION_STEP;
    }
}

#========================================================================================
# function     nextPosition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Position = $Object->nextPosition($LastPosition);
#----------------------------------------------------------------------------------------
# description  Returns the next (top) position for a new child. Such as, if the last
#               child has Position = 150 the method returns 160 (if PositionStep is 10).
#----------------------------------------------------------------------------------------
# input        $LastPosition | last inserted position (optional) | integer
# return       $Position | new (next) position | integer
#========================================================================================
sub nextPosition {
    my $self = shift;
    my ($LastPosition) = @_;
    return $LastPosition + POSITION_STEP if defined $LastPosition;

    my $aChildIDs = $self->findChildren;
    my $cChildren = scalar @$aChildIDs;
    return POSITION_START unless $cChildren;
    return LoadObject($aChildIDs->[$cChildren - 1])->get('Position') + POSITION_STEP;
}

#========================================================================================
# function     filePath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Path = $Object->filePath;
#----------------------------------------------------------------------------------------
# description  Joins FilePath of site and FileAlias of this object.
#----------------------------------------------------------------------------------------
# return       $Path | relative file path | string
#========================================================================================
sub filePath {
    my $self = shift;

    my $FilePathPart = $self->get('FilePathPart');
    if (defined $FilePathPart and $FilePathPart eq 'rebuild') {
        my $aPath = $self->get('PathFromSite');
        my $Path = join '/', map { ConvertPathChars($_->alias) } @$aPath;
        my $Site = $self->getSite();
        if (ExistsDirectory($Site->get('PublicPath') . '/' . $Path)
            or ExistsDirectory($Site->get('PrivatePath') . '/' . $Path)
        ) {
            $FilePathPart = $Path;
        } else {
            DeleteDirectory($Site->get('StaticPath') . '/' . $Path, {'recursive' => 1});
            $FilePathPart = undef;
        }

        # store the combined path into the FilePathPart database field.
        # don't use $self->set({'FilePathPart'=>$FilePathPart}) because that
        # would cause an infinite loop
        $self->dal->update( $self->id, { FilePathPart => $FilePathPart } );
        $self->_attribute( 'FilePathPart', $FilePathPart );##no critic (ProhibitPrivateSubroutineUse)
    }

    my $SiteFilePath = $self->getSite()->get('FilePath');
    return $SiteFilePath eq ''
        ? $self->get('FileAlias')
        : $SiteFilePath .'/'. $self->get('FileAlias');
}

#========================================================================================
# function     relativePath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $relativePath = $Object->relativePath($Object2);
# example      $Object1 = LoadObjectByPath('/Shops/DemoShop/Products/be_40401/SubProducts/be_4040104005');
#               $Object2 = LoadObjectByPath('/Shops/DemoShop/Customers');
#               ok( $Object1->relativePath($Object2) eq '../../../../Customers' );
#               ok( $Object2->relativePath($Object1) eq '../Products/be_40401/SubProducts/be_4040104005' );
#----------------------------------------------------------------------------------------
# description  Returns the relative path of an object to another object.
#----------------------------------------------------------------------------------------
# input        $Object2 | target object | object
# return       $Path | relative object path | string
#========================================================================================
sub relativePath {
    my $self = shift;
    my ($Object) = @_;

    return '.' if $self->id == $Object->id;
    return $Object->pathString if $self->getSite()->id != $Object->getSite()->id;

    my @SelfPath   = map { $_->alias } @{$self->parentPath()};
    my @ObjectPath = map { $_->alias } @{$Object->parentPath()};

    my $equal = 0;
    while (defined($SelfPath[$equal])   &&
           defined($ObjectPath[$equal]) &&
           $SelfPath[$equal] eq $ObjectPath[$equal])
    {
        $equal++;
    }

    my @RelativePath;
    push @RelativePath, '..' for 1 .. scalar(@SelfPath) - $equal;
    push @RelativePath, $ObjectPath[$_] for $equal .. scalar(@ObjectPath)-1;

    return JoinObjectPath(\@RelativePath);
}

#========================================================================================
# function     pageCacheDuration
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageCache = $Object->pageCacheDuration;
#----------------------------------------------------------------------------------------
# description  How long a page can be cached.
#----------------------------------------------------------------------------------------
# return       $PageCache | maximal cache age (minutes) | integer
#========================================================================================
sub pageCacheDuration {
    my $self = shift;
    my $Site = $self->getSite();
    return 0 unless $Site->get('IsPageCacheUsed');

    my $CachedPages = $Site->get('CachedPages');
    my $Class = $self->class;
    my $PageCache = undef;
    while (not defined $PageCache and defined $Class) {
        $PageCache = $CachedPages->{$Class->alias};
        $Class = $Class->baseClass;
    }
    $PageCache = 0 unless defined $PageCache;
    return $PageCache;
}

#========================================================================================
# function     updatePageCache
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->updatePageCache;
#----------------------------------------------------------------------------------------
# description  Updates page cache date of the object's site if the object
#               is an instance of one of the pre-defined classes (this classes overwrites
#               canInvalidatePageCache method). These classes influence the content which
#               is shown on storefront pages.
#========================================================================================
sub updatePageCache {
    my $self = shift;
    if ($self->canInvalidatePageCache) {
        my $Site = $self->getSite();
        $Site->invalidatePageCache;
    }
}

#========================================================================================
# function     canInvalidatePageCache
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->canInvalidatePageCache;
#----------------------------------------------------------------------------------------
# description  Return true if page cache can be updated. Default is false. If page cache
#               should set to invalid the object has to overwrite this method.
#----------------------------------------------------------------------------------------
# return       $canInvalidate | true if page cache can be updated | boolean
#========================================================================================
sub canInvalidatePageCache {
    return 0;
}

#========================================================================================
# function     clearSavedPages
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Shop->clearSavedPages();
#----------------------------------------------------------------------------------------
# description  Clears the saved pages of object.
#               See also: <function DE_EPAGES::Presentation::API::TLE::IncludeTemplateHandler::SavedInclude>
#========================================================================================
sub clearSavedPages {
    my $self = shift;

    my $CacheHandler = DE_EPAGES::Object::API::SavedIncludesHandler->new;

    $CacheHandler->clearObjectCache($self);

    return;
}

#========================================================================================
# function     savePage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Content = $Shop->savePage($CacheKey, $FallBackSub);
#----------------------------------------------------------------------------------------
# example      $Content = $Shop->savePage('MBO.Menu_de',
#               sub{
#                   return join '', @{$Processor->replaceTLEs($cTemplate)};
#               });
#----------------------------------------------------------------------------------------
# description  Save cache of actual object in File. Use static path of actual object.
#               To save cache file the function will need a unique cachekey, this key is
#               integrated in filename. Also a function to compute the content is needed.
#               Content is computed if no cachefile exist.
#========================================================================================
# input        $CacheKey | name or key must be unique for an object | string
# input        $FallBackSub | function that generate and returns actual content | sub
# return       $Content | Content from cached file | content
#========================================================================================
sub savePage {
    my $self = shift;
    my ($CacheKey, $FallBackSub) = @_;

    my $CacheHandler = DE_EPAGES::Object::API::SavedIncludesHandler->new;
    return $CacheHandler->savePage( $self, $CacheKey, $FallBackSub );
}

#========================================================================================
# function     pageCacheMaxAge
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageCache = $Object->pageCacheMaxAge;
#----------------------------------------------------------------------------------------
# description  How long a page can be cached on client.
#----------------------------------------------------------------------------------------
# return       $PageCache | maximal cache age (seconds) | integer
#========================================================================================
sub pageCacheMaxAge {
    my $self = shift;
    my $Site = $self->getSite();
    return 0 unless $Site->get('IsPageCacheUsed');

    my $CachedPages = $Site->get('MaxAgePages');
    my $Class = $self->class;
    my $PageCache = undef;
    while (not defined $PageCache and defined $Class) {
        $PageCache = $CachedPages->{$Class->alias};
        $Class = $Class->baseClass;
    }
    $PageCache = 0 unless defined $PageCache;
    return $PageCache;
}

#========================================================================================
# function     isChildOf
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isChild = $Object->isChildOf($PossibleParent);
#----------------------------------------------------------------------------------------
# description  Returns whether an Object is a child (direct or indirect) of a
#               PossibleParent.
#----------------------------------------------------------------------------------------
# input        $PossibleParent | possible parent object | object
# return       $isChild | is a child | boolean
#========================================================================================
sub isChildOf($) {
    my $self = shift;
    my ($PossibleParent) = @_;

    my $CompareParent = $self->get('Parent');
    return 0 unless $CompareParent;

    do {
        #print $PossibleParent->id, ", ", $CompareParent->id, "\n";
        return 1 if ($PossibleParent->id == $CompareParent->id);
    } while ($CompareParent = $CompareParent->get('Parent'));
    return 0;
}

#========================================================================================
# function     isParentOf
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isParent = $Object->isParentOf($PossibleChild);
#----------------------------------------------------------------------------------------
# description  Returns true if an Object is a parent (direct or indirect) of another
#               object.
#----------------------------------------------------------------------------------------
# input        $PossibleChild | possible child object | object
# return       $isParent | is a parent | boolean
#========================================================================================
sub isParentOf($) {
    my $self = shift;
    my ($PossibleChild) = @_;
    return $PossibleChild->isChildOf($self);
}

#========================================================================================
# function     isAllowedChild
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isAllowed = $Object->isAllowedChild($PossibleChild);
#----------------------------------------------------------------------------------------
# description  Returns true if an Object could be the parent of a PossibleChild.
#----------------------------------------------------------------------------------------
# input        $PossibleChild | possible child object | boolean
# return       $isAllowed | is a allowed child type | boolean
#========================================================================================
sub isAllowedChild($) {
    my $self = shift;
    my ($PossibleChild) = @_;
    return 0 unless $PossibleChild->get('Parent');
    return 1 if $PossibleChild->instanceOf('Object');
    return 0;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Object::API::Object::Object->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'Position' => 0,
        'Inherit'=> 1,
        'Site'=> $Parent->getSite(),
    };
}

#========================================================================================
# function     exportableChildren
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aChildren = $Object->exportableChildren
#----------------------------------------------------------------------------------------
# example      $Description = $Object->exportableChildren;
#----------------------------------------------------------------------------------------
# description  Returns all children of the object which should be exported.
#----------------------------------------------------------------------------------------
# return       $aChildren | children of the object | ref.array.object or DE_EPAGES::Core::API::Iterator
#========================================================================================
sub exportableChildren {
    my $self = shift;

    return $self->children;
}

#========================================================================================
# function     classTemplate
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Name = $Object->classTemplate($Name, $ObjectPageType);
#----------------------------------------------------------------------------------------
# description  This function is used to overwrite the given template for the purpose that
#               each content object can defined there own template (visualisation).
#----------------------------------------------------------------------------------------
# input        $Name | include name | string
# input        $ObjectPageType | object related pagetype | DE_EPAGES::Presentation::API::Object::ObjectPageType
# return       $Name | include name | string
#========================================================================================
sub classTemplate {
    my $self = shift;
    my ($TemplateName, $ObjectPageType) = @_;

    my $ExistsTemplate = 0;
    my $Class = $self->class;
    my $PageType = $ObjectPageType->get('PageType');
    while (not $ExistsTemplate and defined $Class) {
        my $ClassAlias = $Class->alias;
        if ($PageType->existsTemplate($TemplateName.$ClassAlias)) {
            $ExistsTemplate = 1;
            $TemplateName.=$ClassAlias;
        }
        $Class = $Class->baseClass;
    }
    return $TemplateName;
}

#========================================================================================
# function     templatePath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aObjects = $Object->templatePath;
#----------------------------------------------------------------------------------------
# description  This function return a list of objects which can define the used templates.
#----------------------------------------------------------------------------------------
# return       $aObjects | list of objects, classes | ref.array.object
#========================================================================================
sub templatePath {
    my $self = shift;
    return [ LoadRootObject(), @{$self->class->baseClasses}, $self];
}

#========================================================================================
# function     useFilePathPart
# state        public
#----------------------------------------------------------------------------------------
# syntax       $useFilePathPart = $Object->useFilePathPart;
#----------------------------------------------------------------------------------------
# description  This function return 0 as default for all objects. Mainly used for MediaGallery.
#----------------------------------------------------------------------------------------
# return       $useFilePathPart | false if not use filepathpart | boolean
#========================================================================================
sub useFilePathPart {
    return 0;
}

#========================================================================================
# function     setDeleted
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->setDeleted( $ClassName );
#----------------------------------------------------------------------------------------
# description  Records the deletion of this object in the "objectdelete" table with
#               the current time, GUID, object path and class.
#               This method should be called in the same transaction that actually
#               deletes the object.
#----------------------------------------------------------------------------------------
# input        $Class | class name | string
#========================================================================================
sub setDeleted {
    my $self = shift;
    my ($ClassName) = @_;
    TestString('ClassName', $ClassName);

    DAL_SetObjectDeleted({
        GUID => $self->guid,
        ClassID => LoadClassByAlias($ClassName)->id,
        SiteID => $self->getSite->id,
        PathFromSite => $self->get('RelativePath'),
    });

    return;
}

#========================================================================================
# function     setUpdated
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->setUpdated( $Profile );
#----------------------------------------------------------------------------------------
# description  Records the update of this object in the "lastupdate" table with
#               the current time related to a LastUpdateProfile.
#               This method should be called in the same transaction that actually
#               updates the object regarding that profile.
#----------------------------------------------------------------------------------------
# input        $Profile | instance of LastUpdateProfile | DE_EPAGES::Object::API::Object::LastUpdateProfile
#========================================================================================
sub setUpdated {
    my $self = shift;
    my ($Profile) = @_;

    $Profile->setLastUpdate( $self );
    return;
}

#========================================================================================
# function     countChildrenDepth
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->countChildrenDepth;
#----------------------------------------------------------------------------------------
# description  Counts the maximum number of levels of existing child objects
#               of the calling object
#----------------------------------------------------------------------------------------
# return       $CurrentMaxDepth | biggest existing children depth | int
#========================================================================================
sub countChildrenDepth {
    my $self = shift;
    my $CurrentMaxDepth = 0;

    foreach my $Child (@{$self->children}) {
        my $CurrentDepth = $Child->countChildrenDepth + 1;
        if ($CurrentDepth > $CurrentMaxDepth) {
            $CurrentMaxDepth = $CurrentDepth;
        }
    }
    return $CurrentMaxDepth;
}

#========================================================================================
# function     getWebUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $WebUrl = $Object->getWebUrl($hParams);
#----------------------------------------------------------------------------------------
# description  Construct the web url of a given object dependend of the following parameters:
#----------------------------------------------------------------------------------------
# input        $hParams | attributes of new object
#                       <ul>
#                         <li>SSL - SSL web url - boolean</li>
#                         <li>Section - Storefront | Mobile | Admin - string</li>
#                       </ul> | hash ref
# return       $WebUrl | web url of the object | string
#========================================================================================
sub getWebUrl {
    my $self = shift;
    my ($hParams) = @_;
    my $Site = $self->getSite();
    my $WebUrl;

    if ($hParams->{'SSL'}) {
        # SSL url
        my $WebServerScriptUrl;
        given ($hParams->{'Section'}) {
            when ('Storefront') {
                $WebServerScriptUrl = $Site->get('WebServerScriptUrlSSL');
            }
            when ('Mobile') {
                $WebServerScriptUrl = $Site->get('WebServerScriptUrlMobileSSL');
            }
            when ('Admin') {
                $WebServerScriptUrl = $Site->get('WebServerScriptAdminUrlSSL');
            }
        }

        if ($Site->id == $self->id and (
                defined $Site->get('DomainName') and $Site->get('HasSSLCertificate')
                or
                defined $Site->get('WebServerScriptNamePart')
                )
            ) {
            $WebUrl = $WebServerScriptUrl;
        } else {
            $WebUrl = $self->_addObjectPath($WebServerScriptUrl);
        }
    } else {
        # non SSL url
        my $WebServerScriptUrl;
        given ($hParams->{'Section'}) {
            when ('Storefront') {
                $WebServerScriptUrl = $Site->get('WebServerScriptUrl');
            }
            when ('Mobile') {
                $WebServerScriptUrl = $Site->get('WebServerScriptUrlMobile');
            }
            when ('Admin') {
                $WebServerScriptUrl = $Site->get('WebServerScriptAdminUrl');
            }
        }

        if ($Site->id == $self->id and (
            defined $Site->get('WebServerScriptNamePart') or
            defined $Site->get('DomainName')
            )
        ) {
            $WebUrl = $WebServerScriptUrl;
        } else {
            $WebUrl = $self->_addObjectPath($WebServerScriptUrl);
        }
    }
    return $WebUrl;
}

sub _addObjectPath {
    my $self = shift;
    my ($WebServerScriptUrl) = @_;

    my $Path = uri_escape_utf8( $self->get('Path') );
    $Path =~ s/%2F/\//ig;
    return $WebServerScriptUrl .'/?ObjectPath=' . $Path;
}

#========================================================================================
# function     getWebUrlUnity
# state        public
#----------------------------------------------------------------------------------------
# syntax       $WebUrl = $Object->getWebUrlUnity($hParams);
#----------------------------------------------------------------------------------------
# description  Construct the web url for Unity of a given object
#               dependend of the following parameters:
#----------------------------------------------------------------------------------------
# input        $hParams | attributes of new object
#                       <ul>
#                         <li>SSL - SSL web url - boolean</li>
#                         <li>Section - Storefront | Mobile | Admin - string</li>
#                       </ul> | hash ref
# return       $WebUrlUnity | web url of the object | string
#========================================================================================
sub getWebUrlUnity {
    my $self = shift;
    my ($hParams) = @_;
    my $WebUrlUnity;

    if ($hParams->{'Section'} eq 'Storefront' || $hParams->{'Section'} eq 'Mobile') {
        if ($hParams->{'SSL'}) {
            $WebUrlUnity = $self->get('WebUrlUnitySSL');
        } else {
            $WebUrlUnity = $self->get('WebUrlUnity');
        }
    } else {
        # use method of this exact package
        $WebUrlUnity = getWebUrl($self, $hParams);
    }
    return $WebUrlUnity;
}

1;
