#========================================================================================
# §package      DE_EPAGES::Order::API::TLE::BasketOrder
# §description  TLE functions for Basket and Order
# §state        public
#========================================================================================
package DE_EPAGES::Order::API::TLE::BasketOrder;

use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );


#========================================================================================
# §function     RegisterHandlerProc
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# §description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'BasketOrder.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# §function     new
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler = new DE_EPAGES::Object::API::TLE::BasketOrder;
# §example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::BasketOrder;
#----------------------------------------------------------------------------------------
# §description  create a new handler
#----------------------------------------------------------------------------------------
# §return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# §function     register
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# §description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# §input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'BasketOrder.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'IsInMultiStepCheckout');
    return;
}

#========================================================================================
# §function     IsInMultiStepCheckout
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #IsInMultiStepCheckout(#BasketOrderProcess)
#----------------------------------------------------------------------------------------
# §description  TLE to determine if a user is on a multistep checkout page. This method
#               wraps the actual logic and provides TLE specific error handling.
#----------------------------------------------------------------------------------------
# §input        $hBasketOrderProcess | hash provided by the #BasketOrderProcess TLE | ref.hash
#----------------------------------------------------------------------------------------
# return        1 or 0 | 1, if customer is in the multistep checkout, 0 otherwise | int
#========================================================================================
sub IsInMultiStepCheckout {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($hBasketOrderProcess) = @$aParams;

    my $Result = eval {
        $self->_isInMultiStepCheckout($hBasketOrderProcess);
    };
    if ( ExistsError() ) {
        LogError(GetError());
        $Result = 0;
    }

    return $Result;
}

#========================================================================================
# §function     _isInMultiStepCheckout
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->_isInMultiStepCheckout($hBasketOrderProcess)
#----------------------------------------------------------------------------------------
# §description  Method to determine if a user is on a multistep checkout page.
#----------------------------------------------------------------------------------------
# §input        $hBasketOrderProcess | has containing by the #BasketOrderProcess TLE | ref.hash
#----------------------------------------------------------------------------------------
# return        true or false | true, if customer is in the multistep checkout | boolean
#========================================================================================
sub _isInMultiStepCheckout {
    my $self = shift;
    my ($hBasketOrderProcess) = @_;

    my $hCurrentStep = $hBasketOrderProcess->{'CurrentStep'};

    # check if customer is on basket or checkout step
    return 0 unless defined $hBasketOrderProcess;
    # check if one step in the checkout process is active
    return 0 unless defined $hCurrentStep;
    # check if customer is on the basket step (basket is not part of the checkout)
    return 0 if $hCurrentStep->{'Alias'} eq 'Basket';
    # returns 1 only if customer is not on the basket step but in another checkout step
    return 1;
}

1;
