#==============================================================================
# package      DE_EPAGES::Paynet::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::Paynet::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use strict;
use Digest;
use Encode qw ( encode );
use DE_EPAGES::Core::API::Error          qw (GetError Error ExistsError);
use DE_EPAGES::Core::API::PerlTools      qw (fcmp GetCurrentDateTime isValid);
use DE_EPAGES::ExternalPayment::API::Log qw (LogPayment);
use DE_EPAGES::Object::API::Factory      qw (ExistsObjectByGUID LoadObjectByGUID);
use DE_EPAGES::Order::UI::Basket;

#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  return GoogleCheckout as string. Name of PaymentType for search/view
#                               transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName { 'Paynet' }

#==============================================================================
# This callback is sent to the success link. We do not get the hash to verify
# the correctness. Therefore we will not create the order here. This must be
# done in the callback function below.
#==============================================================================
sub PaymentBacklinkPaynet {
    my $self = shift;
    my ($Servlet) = @_;

    my $hFormValues = $Servlet->params();
    LogPayment('Paynet', 'Paynet backlink parameters', $hFormValues);

    my $PaymentLineItemGUID = $hFormValues->{'kunden_var_5'};
    $PaymentLineItemGUID = $hFormValues->{'MGUID'} unless defined $hFormValues->{'kunden_var_5'};

    # reset cached language id etc.
    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);

    my $Shop = $Servlet->object();

    my $hOptions = {
        'Name'       => 'M_PaymentGUID',
        'Reason'     => 'BadCallback_NoPaymentGUID',
        'ViewAction' => 'ViewPaymentPaynetResult',
    };

    unless (isValid($PaymentLineItemGUID) && ExistsObjectByGUID($PaymentLineItemGUID)) {

        # Not handling this case in Unity, because is a generic error and we are highly depending on parameters sending by Sofort
        # If the url is configured correctly in Sorfort as showed in our MBO should work.
        $self->_handleFailInEpages($Servlet, $hOptions);
        return;
    }

    my $PaymentLineItem = LoadObjectByGUID($PaymentLineItemGUID);

    # wait 3 seconds for the callback below to finish
    for (1..3) {
        my $PaymentObject = $PaymentLineItem->container->parent;
        my $Status = $PaymentLineItem->get('TransStatus') // '';

        return $PaymentObject if ($PaymentObject->instanceOf('CustomerOrder') && $Status eq 'Y'); # view order confirmation page
        sleep(1);
    }

    # no order - show error page
    if($Shop->get('IsUnityRegistered')) {

        $self->_handleFailInUnity($Servlet, $PaymentLineItem);

        return;

    } else {

        $hOptions = {
            'Name'       => 'Declined',
            'Reason'     => 'Declined',
            'ViewAction' => 'ViewPaymentPaynetResult'
        };

        $self->_handleFailInEpages($Servlet, $hOptions);

        return;
    }
    return;
}

#==============================================================================
# This callback is used as NotificationLink that is configured in Sofort Backoffice
sub PaymentCallbackPaynet {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $IsUnityRegistered = $Shop->get('IsUnityRegistered');

    my $hFormValues = $Servlet->params();
    LogPayment('Paynet', 'Paynet callback parameters', $hFormValues);

    # test the parameters of Paynet, show errors if something is wrong
    my $hValues = $self->_testPaynetParameters($Servlet, $hFormValues);

    # update the paymentlineitem
    my $PaymentLineItem = $hValues->{'PaymentLineItem'};
    delete $hValues->{'PaymentLineItem'};
    $PaymentLineItem->set($hValues);

    my $ErrorViewAction;
    my $Object = $PaymentLineItem->container->parent;

    # used for debugging
    my $ObjectPath = $Object->pathString . ' (ObjectID ' . $Object->id .')';
    LogPayment('Paynet', "Paynet transaction for $ObjectPath; Params", $hFormValues);


    if ( $Object->instanceOf('Basket') or $Object->instanceOf('CustomerOrder') ) {

        if ($hValues->{'TransStatus'} eq 'Y') {
            my $Order = $Object->instanceOf('CustomerOrder') ? $Object : undef;
            if ($Object->instanceOf('Basket')) {
                $Order = DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $Object);
            }
            if ($PaymentLineItem->get('TransStatus') eq 'Y') {
                eval { $Order->set({'PaidOn' => GetCurrentDateTime()}); };
                if (ExistsError()) {
                    my $Error = GetError();
                    if ($Error->code eq 'SEND_MAIL_FAILED') {
                        $Servlet->vars('SEND_MAIL_FAILED', 1);
                    } else {
                        Error($Error);
                    }
                }
            }
        }
        my $Response = $Servlet->response;
        $Response->content_type('text/plain');
        $Response->content('OK');
    } else {
        # already ordered

        if($IsUnityRegistered){

            $self->_handleFailInUnity($Servlet, $PaymentLineItem);
        } else {
            my $hOptions = {
                'Name'       => 'OrderID',
                'Reason'     => 'BadCallback_OrderAlreadyExists',
                'ViewAction' => $ErrorViewAction,
                'ObjectID'   => $Object->id,
                'ObjectClass'=> $Object->class->alias,
            };

            $self->_handleFailInEpages($Servlet, $hOptions);
        }
    }

    return;
}

#==============================================================================
sub _testPaynetParameters {
    my $self = shift;
    my ($Servlet, $hFormValues) = @_;

    my $Shop = $Servlet->object;
    my $PaymentLineItemGUID;

    my $IsUnityRegistered = $Shop->get('IsUnityRegistered');

    $PaymentLineItemGUID = $hFormValues->{'kunden_var_5'};
    $PaymentLineItemGUID = $hFormValues->{'MGUID'} unless defined $hFormValues->{'kunden_var_5'};

    my $hErrorOptions = {
        'Name'       => 'M_PaymentGUID',
        'Reason'     => 'BadCallback_NoPaymentGUID',
        'ViewAction' => 'ViewPaymentPaynetResult',
    };

    unless (isValid($PaymentLineItemGUID) && ExistsObjectByGUID($PaymentLineItemGUID)) {
        $self->_handleFailInEpages($Servlet, $hErrorOptions);
    }

    my $PaymentLineItem = LoadObjectByGUID($PaymentLineItemGUID);
    my $ErrorViewAction = 'ViewPaymentPaynetResult';

    $hErrorOptions = {
        'Name'       => 'MGUID',
        'Reason'     => 'BadCallback_ShopIDMisMatch',
        'ViewAction' => $ErrorViewAction
    };

    if ($PaymentLineItem->getSite()->id != $Shop->id ) {

        if($IsUnityRegistered){

            $self->_handleFailInUnity($Servlet, $PaymentLineItem);
        } else {

            $self->_handleFailInEpages($Servlet, $hErrorOptions);
        }
    }

    # check hash
    my @tmp = (
        $hFormValues->{'transaction'},
        $hFormValues->{'user_id'},
        $hFormValues->{'project_id'},
        $hFormValues->{'sender_holder'},
        $hFormValues->{'sender_account_number'},
        $hFormValues->{'sender_bank_code'},
        $hFormValues->{'sender_bank_name'},
        $hFormValues->{'sender_bank_bic'},
        $hFormValues->{'sender_iban'},
        $hFormValues->{'sender_country_id'},
        $hFormValues->{'recipient_holder'},
        $hFormValues->{'recipient_account_number'},
        $hFormValues->{'recipient_bank_code'},
        $hFormValues->{'recipient_bank_name'},
        $hFormValues->{'recipient_bank_bic'},
        $hFormValues->{'recipient_iban'},
        $hFormValues->{'recipient_country_id'},
        $hFormValues->{'international_transaction'},
        $hFormValues->{'amount'},
        $hFormValues->{'currency_id'},
        $hFormValues->{'reason_1'},
        $hFormValues->{'reason_2'},
        $hFormValues->{'security_criteria'},
        $hFormValues->{'user_variable_0'},
        $hFormValues->{'user_variable_1'},
        $hFormValues->{'user_variable_2'},
        $hFormValues->{'user_variable_3'},
        $hFormValues->{'user_variable_4'},
        $hFormValues->{'user_variable_5'},
        $hFormValues->{'created'},
    );

    my $PassWord = $PaymentLineItem->get('PaymentMethod')->get('NotificationPW')
                // $PaymentLineItem->get('PaymentMethod')->get('ProjectPW');
    my $key = join('|', map {$_ // ''} (@tmp, $PassWord));
    my $keyOctets = encode('utf8', $key );
    my $hash = Digest->new('SHA-256')->add($keyOctets)->hexdigest;

    if ($hash ne $hFormValues->{'hash'}) {
        LogPayment('Paynet', "Paynet transaction for payment lineitem $PaymentLineItemGUID: hash mismatch", {'calculated' => $hash, 'expected' => $hFormValues->{'hash'}});

        if($IsUnityRegistered){

            $self->_handleFailInUnity($Servlet, $PaymentLineItem);
        } else {

            $hErrorOptions = {
                'Name'       => 'MGUID',
                'Reason'     => 'BadCallback_HashMismatch',
                'ViewAction' => $ErrorViewAction
            };
            $self->_handleFailInEpages($Servlet, $hErrorOptions);
        }
    }
    my %Values;
    $Values{'PaymentLineItem'} = $PaymentLineItem;

    if (defined $hFormValues->{'trans_id'}) {
        $Values{'TransID'} = $hFormValues->{'trans_id'};
    }

    if (defined $hFormValues->{'bank_code'} and defined $hFormValues->{'country_id'}) {
        if ($hFormValues->{'country_id'} eq 'DE') {
            $Values{'IsTestTransaction'} = 1 if $hFormValues->{'bank_code'} eq '88888888';
        }
        elsif ($hFormValues->{'country_id'} eq 'AT') {
            $Values{'IsTestTransaction'} = 1 if $hFormValues->{'bank_code'} eq '00000';
        }
    }

    if (defined $hFormValues->{'betrag'}) {
        $Values{'Amount'} = $hFormValues->{'betrag'};
    }

    $Values{'TransTime'} = GetCurrentDateTime();
    $Values{'TransStatus'} = fcmp($PaymentLineItem->get('Amount'),$Values{'Amount'}) == 0  ? 'Y' : 'M';

    LogPayment('Paynet', "Paynet transaction Params for $PaymentLineItemGUID", \%Values);

    return \%Values;
}


sub _handleFailInUnity {
    my $self = shift;
    my ($Servlet, $PaymentLineItem) = @_;

    my $Order =  $PaymentLineItem->container()->parent();

    $PaymentLineItem->set({'PaymentErrorCode' => 'Failed'});

    $Servlet->redirect($Order->get('WebUrlUnitySSL'));

    return;
}

sub _handleFailInEpages {
    my $self = shift;
    my ($Servlet, $Options) = @_;

    my $Form = $Servlet->form();

    $Form->executeFormError({
        'Name'       => $Options->{'Name'},
        'Reason'     => $Options->{'Reason'},
        'ViewAction' => $Options->{'ViewAction'},
        'ObjectID'   => $Options->{'ObjectID'},
        'ObjectClass'=> $Options->{'ObjectClass'}
    });

    return;
}
1;


