#========================================================================================
# package      DE_EPAGES::Presentation::API::JSSyntaxCheck
# state        public
#----------------------------------------------------------------------------------------
# description  API functions for checking JavaScript syntax
#========================================================================================
package DE_EPAGES::Presentation::API::JSSyntaxCheck;
use base Exporter;

use strict;
use Config;
use DE_EPAGES::Core::API::File qw ( CorrectPathSlashes DeleteFile );
use DE_EPAGES::Core::API::PerlTools qw ( GenerateShortGUID );
use DE_EPAGES::Presentation::API::YUICompressor qw ( GetYUIJar );
use DE_EPAGES::Core::API::Program::java;

our @EXPORT_OK = qw (
    CheckImpliedGlobals
    CheckObjectCTORCalls
    CheckCommentedEvils
    CheckJQueryEvents
    CheckAttributeModifiers
    CheckForLoops
    CheckWithAlert
    CheckConsoleDebugLog
    CheckStrictCompareUndef
    CheckCallPrivateFunctions
    CheckTranslationList
    CheckInnerHtmlEmpty
    CheckJSLint
    CheckYUICompressor
);

#========================================================================================
# function     CheckImpliedGlobals
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckImpliedGlobals($Sourcecode);
# example      if(!$CheckImpliedGlobals($Line)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  Looks for implied and possible hidden creation of global variables. For
#               example when doing:
#                   var a = b = 0;
#               the variable b becomes global. So always declare variables one by one or
#               separated by coma:
#                   var a = 0,
#                       b = 0;
#               See the book "JavaScript Patterns" by Stoyan Stefanov, page 10+.
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code string | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckImpliedGlobals {
    my ($Content) = @_;
    if($Content =~ /var\s+(?:\w+\s*=\s){2,}/xg) {
        return 0;
    }
    return 1;
}

#========================================================================================
# function     CheckObjectCTORCalls
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckObjectCTORCalls($Sourcecode);
# example      if(!$CheckObjectCTORCalls($Line)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  Looks for creation of object instances via class constructor calls like:
#                   var x = new Object();
#               Do not use that pattern, instead use:
#                   var x = {};
#               See book "JavaScript Patterns" by Stoyan Stefanov, page 39+.
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code string | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckObjectCTORCalls {
    my ($Content) = @_;

    if($Content =~ /=\s*new\s+(?:Object|Array|Number|Boolean|String|Error|Function)\(/xg) { ## no critic qw(ProhibitComplexRegexes)
        return 0;
    }
    return 1;
}

#========================================================================================
# function     CheckCommentedEvils
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckCommentedEvils($Line, $Linebefore);
# example      if(!$CheckObjectCTORCalls($Line, $Linebefore)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  The usage of the JavaScript methods "setInterval", "setTimeout" and "eval"
#               are prone to errors, can degrade performance and may complicate debugging.
#               Either do not use this methods or add a comment in the source code at the
#               line above the function call to explain what the function does and why it
#               has to be used there. Hence the source code should then look like:
#                   // this interval is needed because ...
#                   this._interval = window.setInterval(function(){ ... },10);
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code line | string
# input        $ContentLineBefore | source code line before $Content | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckCommentedEvils {
    my ($Content, $ContentLineBefore) = @_;
    if($Content =~ /(?:setInterval|setTimeout|eval)/xg) {
        if(!$ContentLineBefore || $ContentLineBefore !~ /^\s*(?:\/\/|\/\*)/xg) {
            return 0;
        }
    }
    return 1;
}

#========================================================================================
# function     CheckJQueryEvents
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckJQueryEvents($Line);
# example      if(!$CheckJQueryEvents($Line)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  Applies to use of the jQuery API. In jQuery the call of a ".click" handler
#               on a node is just a wrapper for the ".bind" method. Therefore we want to
#               save the extra function call and work with ".bind" directly. The following
#               event methods should not be called directly, they should be wrapped in
#               ".bind()" or ".trigger()":
#                   blur, focus, focusin, focusout, load, resize, scroll, unload, click,
#                   dblclick, mousedown, mouseup, mousemove, mouseover, mouseout, mouseenter,
#                   mouseleave, change, select, submit, keydown, keypress, keyup, error
#               For example:
#                   $('#target').click(function() { ... });
#               should be:
#                   $('#target').bind('click', function() { ... });
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code line | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckJQueryEvents {
    my ($Content) = @_;
    # EPG-26068 Javascript: Syntax check need instructions to ignore conditions in special cases (backbone)
    # events .select, .change removed from list
    if($Content =~ /\.(?:blur|focus|focusin|focusout|load|resize|scroll|unload|click|
                       dblclick|mousedown|mouseup|mousemove|mouseover|mouseout|
                       mouseenter|mouseleave|submit|keydown|keypress|
                       keyup)\([^\)]/xg) { ## no critic qw(ProhibitComplexRegexes)
        return 0;
    }
    return 1;
}

#========================================================================================
# function     CheckAttributeModifiers
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckAttributeModifiers($Line);
# example      if(!$CheckAttributeModifiers($Line)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  jQuery supplies methods for modifying DOM node attributes, therefore do
#               not use the native JavaScript methods to do this. Instead of:
#                   var domNode = document.getElementById('mynode');
#                   domNode.setAttribute('name', 'myname');
#               use:
#                   $('#mynode').attr('name', 'myname');
#               The same applies to getAttribute. For removeAttribute use $().removeAttr.
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code line | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckAttributeModifiers {
    my ($Content) = @_;
    if($Content =~ /\.(set|remove|get)Attribute\(/xg) {
        return 0;
    }
    return 1;
}

#========================================================================================
# function     CheckForLoops
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckForLoops($Line);
# example      if(!$CheckForLoops($Line)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  Then looping through collections, save the limit/size of the collection
#               in a variable instead of accessing the collection's ".length" attribute
#               on every iteration. Given an array of DOM nodes in a variable "nodes",
#               instead of:
#                   for(var i=0; i<nodes.length; i++) { ... }
#               use:
#                   for(var i=0, iLength=nodes.length; i < iLength; i++) { ... }
#               This prevents lookups in the DOM tree for each iteration.
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code line | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckForLoops {
    my ($Content) = @_;
    if($Content =~ /for\(.*;.*\.length.*;.*/xg) {
        return 0;
    }
    return 1;
}

#========================================================================================
# function     CheckWithAlert
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckWithAlert($Line);
# example      if(!$CheckWithAlert($Line)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  Alert boxes should not remain in productive source code. The with
#               statement can severely slow down the code runtime and should never be used.
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code line | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckWithAlert {
    my ($Content) = @_;

    if($Content =~ /[^\.](?:with|alert)\(/xg && $Content !~ /(?:\/\/|\/\*).*(?:with|alert)\(/x) {
        return 0;
    }
    return 1;
}

#========================================================================================
# function     CheckConsoleDebugLog
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckConsoleDebugLog($Line);
# example      if(!$CheckConsoleDebugLog($Line)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  The statements "console.debug" and "console.log" should not appear in
#               productive source code.
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code line | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckConsoleDebugLog {
    my ($Content) = @_;
    if($Content =~ /console\.(?:debug|log)\([^\n]+\);?/xg) {
        return 0;
    }
    return 1;
}

#========================================================================================
# function     CheckStrictCompareUndef
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckStrictCompareUndef($Line);
# example      if(!$CheckStrictCompareUndef($Line)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  Comparisons to "undefined" should always be done with the strict
#               operators !== or === .
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code line | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckStrictCompareUndef {
    my ($Content) = @_;
    if($Content =~ /[^!=](!=\s|==\s)\s*['"]undefined['"]/xg) {  #"
        return 0;
    }
    return 1;
}

#========================================================================================
# function     CheckCallPrivateFunctions
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckCallPrivateFunctions($Line);
# example      if(!$CheckCallPrivateFunctions($Line)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  Do not call private methods of other objects.
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code line | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckCallPrivateFunctions {
    my ($Content) = @_;
    if($Content =~ /(?<!prototype)(?<!self)(?<!me)(?<!this)\._\w+/xg) {
        return 0;
    }
    return 1;
}

#========================================================================================
# function     CheckTranslationList
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckTranslationList($rContent);
# example      if(!$CheckTranslationList($rFileContent)) { print "error ..."; }
#----------------------------------------------------------------------------------------
# description  Applies to the ePages JavaScript API, based on jQuery. If a JS module
#               uses a function call to our dictionary like:
#                   this.dict.translate(...)
#               the module MUST contain a declaration of the used language tags like:
#                   /*
#                    * @dictionary  ep.dict
#                    *
#                    * @translation
#                    *  {Today}
#                    *  {Done}
#                    */
#----------------------------------------------------------------------------------------
# input        $rContent | javascript source code file content | ref.string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckTranslationList {
    my ($rContent) = @_; # entire file content, hash to string

    if($rContent =~ /dict\.translate\(/xg) {
        if($rContent !~ /\@dictionary/x || $rContent !~ /\@translation/x) {
            return 0;
        }
    }
    return 1;
}

#========================================================================================
# function     CheckInnerHtmlEmpty
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckInnerHtmlEmpty($Line);
# example      if(!$CheckInnerHtmlEmpty($Line)) { print "error in $LineNum ..."; }
#----------------------------------------------------------------------------------------
# description  Don't use ".innerHTML" to check for empty nodes.
#               Instead of:
#                   if(element.innerHTML == '') ...
#               use:
#                   if(element.childNodes.length == 0) ...
#----------------------------------------------------------------------------------------
# input        $Content | javascript source code line | string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckInnerHtmlEmpty {
    my ($Content) = @_;
    if($Content =~ /\.innerHTML\s*[!=]={1,2}\s*["']{2}/xg) {  #"
        return 0;
    }
    return 1;
}

#========================================================================================
# function     CheckJSLint
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckJSLint($File, $LogWarning);
# example      my ($JSLintOK, $Output) = CheckJSLint($JSFile, $LogWarnings);
#----------------------------------------------------------------------------------------
# description  Call external application "jsl" to check JavaScript syntax.
#----------------------------------------------------------------------------------------
# input        $JSFile | javascript full file path | string
# input        $LogWarnings | show only errors or warnings as well? | boolean (optional)
# input        $JSLintConfig | Use this jsl configuration file | string (optional)
# return       (0|1, $Output) | Array with two values: 1 or 0 whether jsl found any
#               errors/warnings and the jsl error output string | ref.array
#========================================================================================
sub CheckJSLint {
    my ($JSFile, $LogWarnings, $JSLintConfig) = @_;

    my $JSLintApp = CorrectPathSlashes($ENV{'EPAGES_PERL'} . '/bin/jsl');

    # JSLint
    my @Params = (
       ($JSLintConfig ? '-conf "'.$JSLintConfig.'"' : ''),
       '-process "'.CorrectPathSlashes($JSFile).'"',
       '2>&1',
    );
    my $CmdLine = _JoinCommandLine([$JSLintApp, @Params]);

    my $Output = "FILE:$JSFile\n" . `$CmdLine` . "\n";
    my $ReturnCode = $?>>8;
    #0 - Success
    #1 - JavaScript warning
    #2 - Usage or configura
    #3 - JavaScript error
    #4 - File error
    if($ReturnCode =~ /34/ or ($ReturnCode == 1 and defined $LogWarnings)) {
        return (0, $Output);
    }
    return (1, undef);
}

#========================================================================================
# function     CheckYUICompressor
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckYUICompressor($File);
# example      my ($JSLintOK, $Output) = CheckYUICompressor($JSFile);
#----------------------------------------------------------------------------------------
# description  Call external YUI compressor application to check JavaScript syntax.
#----------------------------------------------------------------------------------------
# input        $JSFile | javascript full file path | string
# return       (0|1, $Output) | Array with two values: 1 or 0 whether YUI found any
#               errors/warnings and the YUI error output string | ref.array
#========================================================================================
sub CheckYUICompressor {
    my ($JSFile) = @_;

    my $TmpFileName = $JSFile . '_' . GenerateShortGUID();
    my @Params = (
        '-jar',
        GetYUIJar(),
        '-o', CorrectPathSlashes($TmpFileName),
        qw(--type js --charset utf8),
        CorrectPathSlashes($JSFile),
        '2>&1',
    );
    my $Java = DE_EPAGES::Core::API::Program::java->new;
    my $CmdLine = _JoinCommandLine(['"' . $Java->command() . '"', @Params]);
    my $Output = "FILE:$JSFile\n" . `$CmdLine` . "\n";

    DeleteFile($TmpFileName);

    my $ReturnCode = $?>>8;
    if($ReturnCode != 0) {
        return (0, $Output);
    } else {
        return (1, undef);
    }
}


#========================================================================================
# function     _JoinCommandLine
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $CmdLine = _JoinCommandLine([$Exe, @Params]);
#----------------------------------------------------------------------------------------
# description  Combine the supplied array into a command line string
#----------------------------------------------------------------------------------------
# input        $CommandLine | parameters | ref.array
# return       $CommandLine | command line string | ref.string
#========================================================================================
sub _JoinCommandLine {
    my ($CommandLine) = @_;
    return ref $CommandLine ? join(' ', @$CommandLine) : $CommandLine
}

1;
