#========================================================================================
# package      DE_EPAGES::Product::API::Attributes::Product_Ext
# state        private
# base         DE_EPAGES::Object::API::BaseAttribute
# description  Attribute interface for Product.
#========================================================================================
package DE_EPAGES::Product::API::Attributes::Product_Ext;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use feature qw (switch);
use DateTime;
use DE_EPAGES::Core::API::CSV qw ( ParseCSV );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( ExistsDirectory ExistsFile fsplit GetInfoDirectory );
use DE_EPAGES::Core::API::Html qw ( RemoveScriptTags );
use DE_EPAGES::Core::API::Image qw ( ScaleImage TestImageFormat );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid trim );
use DE_EPAGES::Core::API::Url qw ( StringIsURL );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject LoadRootObject );
use DE_EPAGES::Presentation::API::SiteTLEProcessor qw ( ReplaceTLEs );
use DE_EPAGES::Shop::API::Constants qw ( STOCK_IS_ORDERABLE STOCK_NO_ADDTOBASKET );
use DE_EPAGES::Product::API::Product qw ( TestUPCEAN UsedVariationValues );
use DE_EPAGES::Product::API::Shop qw ( GetProductImageSizes );
use DE_EPAGES::Product::API::Table::ProductManufacturer qw (
    DeleteProductManufacturer
    ExistsProductManufacturer
    ExistsProductManufacturerBySiteID
    GetInfoProductManufacturer
    GetInfoProductManufacturerBySiteID
    InsertProductManufacturer
);
use DE_EPAGES::Product::API::Table::ProductPrice qw ( GetInfoProductPricesByProductID );
use DE_EPAGES::Product::API::Table::ProductShippingMethod qw (
    DeleteProductShippingMethod
    GetInfoProductShippingMethodsByProductID
    InsertProductShippingMethod
);
use DE_EPAGES::Product::API::Table::SelectedVariation qw (
    DeleteSelectedVariation
    GetInfoSelectedVariationsByProductID
    InsertSelectedVariation
);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'PreDefAttributeUsedBySubProduct' => 'PreDefAttribute (PreDefAttributeID=#PreDefAttributeID) is still used by a subproduct of product ( ProductID=#ProductID )',
        'MISSING_PUBLIC_PATH'             => 'Directory #Directory for object #ObjectPath does not exist. SlideShowString (#SlideShowString) should be set to undef',
        'BundleLoop'                      => 'Product #ProductID is reverse bundled with product #BundledProductID',
   };
}
#ERRORDEFINITION END

#========================================================================================
# function     getAttributes
# state        private
#========================================================================================
# description  Getter for attributes.
# syntax       my $hInfo = getAttributes($Object, $aNames, $LanguageID);
#========================================================================================
# input        $Object | owner of attributes | object
# input        $aNames | list attribute names | array.ref.
# input        $LanguageID | (optional) language id for localized values | int
# return       $hInfo | values for requested attributes | hash.ref.
#========================================================================================
sub getAttributes {
    shift;
    my ($Object, $aNames, $LanguageID) = @_;
    my $hInfo = {};

    foreach my $Name (@$aNames) {
        given ($Name) {
            when ('ListPrices') {
                $hInfo->{$Name} = _GetPrice($Object, 'ListPrice');
            } when ('ManufacturerPrices') {
                $hInfo->{$Name} = _GetPrice($Object, 'ManufacturerPrice');
            } when ('EcoParticipations') {
                $hInfo->{$Name} = _GetPrice($Object, 'EcoParticipation');
            } when ('Deposits') {
                $hInfo->{$Name} = _GetPrice($Object, 'Deposit');
            } when ('ImagesSlideShow') {
                my $ahSlideShow = _ImagesSlideShow( $Object );
                if( (scalar(@{$ahSlideShow} > 0)) or $Object->getSite()->get('IsUnityRegistered') ) {
                    $hInfo->{$Name} = $ahSlideShow;
                }
                else {
                    # no slideshow on product, maybe it's a subproduct and the superproduct has one?
                    my $SuperProduct = $Object->superProduct;
                    $hInfo->{$Name} = defined $SuperProduct
                        ? $SuperProduct->get($Name, $LanguageID)
                        : [];
                }
            } when ('ImageThumbnail') {
                $hInfo->{$Name} = _GetImageThumbnail($Object);
            } when ('SubProducts') {
                $hInfo->{$Name} = $Object->subProducts;
            } when ('NameOrAlias') {
                $hInfo->{$Name} = _NameOrAlias( $Object, $LanguageID );
            } when ('UserDefinedAttributeValues') {
                $hInfo->{$Name} = _UserDefinedAttributeValues( $Object, $LanguageID, 'UserDefinedAttributes' );
            } when ('OwnUserDefinedAttributeValues') {
                $hInfo->{$Name} = _UserDefinedAttributeValues( $Object, $LanguageID, 'OwnUserDefinedAttributes' );
            } when ('InheritedUserDefinedAttributeValues') {
                $hInfo->{$Name} = _UserDefinedAttributeValues( $Object, $LanguageID, 'InheritedUserDefinedAttributes' );
            } when ('VariationTypeValues') {
                $hInfo->{$Name} = _VariationTypeValues( $Object );
            } when ('SelectedVariations') {
                $hInfo->{$Name} =  [
                    map { {'SelectedID' => $_->{'PreDefAttributeID'} } }
                    @{GetInfoSelectedVariationsByProductID($Object->id)}
                ];
            } when ('IsMaster') {
                $hInfo->{$Name} = $Object->isMaster;
            } when ('CanAddToBasket') {
                $hInfo->{$Name} = $Object->canAddToBasket();
            } when ('IsDeliveryTimeLonger') {
                $hInfo->{$Name} = $Object->isDeliveryTimeLonger();
            } when ('StockAlert') {
                $hInfo->{$Name} = $Object->stockAlert();
            } when ('RemoteFiles') {
                $hInfo->{$Name} = _RemoteFiles( $Object );
            } when ('IsUPCEANValid') {
                $hInfo->{$Name} = _IsUPCEANValid( $Object );
            } when ('BusinessHoursID') {
                my $BusinessHours = $Object->existsChild('BusinessHours') ? $Object->child('BusinessHours') : undef;
                if (not defined $BusinessHours) {
                    my $Shop = $Object->getSite();
                    $BusinessHours = $Shop->child('BusinessHours') if $Shop->existsChild('BusinessHours');
                }
                $hInfo->{$Name} = defined $BusinessHours ? $BusinessHours->id : undef;
            } when ('ShippingMethods') {
                $hInfo->{$Name} = [
                    map { LoadObject($_->{'ShippingMethodID'} ) }
                    @{GetInfoProductShippingMethodsByProductID($Object->id)}
                ];
            } when ('StockLevel') {
                if ($Object->get('IsBundleProduct')) {
                    $hInfo->{$Name} = _BundleStockLevel($Object);
                } else {
                    $hInfo->{$Name} = $Object->get('_StockLevel');
                }
            } when ('DisplayDeliveryWeight') {
                $hInfo->{$Name} = _DisplayDeliveryWeight($Object);
            } when('SeoCheckRatings') {
                my $hRatings = {
                    'Title'             => 0.3,
                    'Url'               => 0.2,
                    'Description'       => 0.2,
                    'AdvDescription'    => 0.2,
                    'ProductImages'     => 0.1,
                };
                TriggerHook('ProductSEOCheckRatings', {'Object' => $Object, 'Ratings' => $hRatings });
                $hInfo->{$Name} = $hRatings;
            } when('CanDoSEOCheck') {
                $hInfo->{$Name} = $Object->isSubProduct ? 0 : 1;
            } when('Manufacturer') {
                $hInfo->{$Name} = _Manufacturer($Object);
            } when('DisplayTitle') {
                $hInfo->{$Name} = _DisplayTitle($Object, $LanguageID);
            } when('UseSlideshow') {
                $hInfo->{$Name} = _UseSlideshow($Object);
            } when('AvailabilityText') {
                $hInfo->{$Name} = _AvailabilityText($Object, $LanguageID);
            } when('AvailabilityIcon') {
                $hInfo->{$Name} = _AvailabilityIcon($Object);
            }
            when('EnergyLabels') {
                my $aParsed = ParseCSV($Object->getWithParent('EnergyLabelsString'));
                $aParsed = [] unless $Object->get('EnergyLabelSourceFile');
                my $aLabels = [];
                foreach my $Value (@$aParsed){
                    push @$aLabels, {'Value' => $Value} if $Value =~ /\S/;
                }
                $hInfo->{$Name} = $aLabels;
            }
            when('EnergyLabelsAverage') {
                if (not scalar(@{$Object->get('EnergyLabels')})) {
                    $hInfo->{$Name} = {};
                }
                else {
                    my @Labels = @{$Object->get('EnergyLabels')};

                    # get system labels
                    my $ahSystemlabels = LoadRootObject->get('EnergyLabels');

                    #overwrite start position here by attribute
                    my $HighestLabel = $Object->getWithParent('EnergyLabelHighestEfficiencyClass');

                    my $StartValue = $Labels[0]->{'Value'};
                    my $EndValue = $Labels[-1]->{'Value'};
                    my ($StartPos,$EndPos);
                    my $cnt = 1;
                    my $HighValPos = 0;
                    # found start and end at wich position of ahSystemlabels?
                    foreach my $hLabel (@{$ahSystemlabels}) {
                        $HighValPos = $cnt if $hLabel->{'Value'} eq $HighestLabel;
                        $StartPos = $cnt-1 if $hLabel->{'Value'} eq $StartValue;
                        $EndPos = $cnt-1 if $hLabel->{'Value'} eq $EndValue;
                        $cnt++;
                    }

                    # count of elements
                    my $count = 1 + $EndPos - $StartPos;
                    my $idx = $StartPos + int($count / 2);
                    $hInfo->{$Name} = {'Value' => $ahSystemlabels->[$idx]->{'Value'},
                                       'CSSValue' => _prepareCSS($idx + 1 - $HighValPos+1 ),
                                      };
                }
            }
            when('EnergyLabelFromTo') {
                if (not defined $Object->get('EnergyLabels')) {
                    $hInfo->{$Name} = undef;
                }else {
                    my @Labels = @{$Object->get('EnergyLabels')};
                    my $Start = $Labels[0]->{'Value'};
                    my $End = $Labels[-1]->{'Value'};
                    $hInfo->{$Name} = scalar @Labels == 1 ? $Start : $Start." - ".$End;
                }
            } when('EnergyLabelSourceFile') {
                $hInfo->{$Name} = _testAttributeFile($Object,'_EnergyLabelSourceFile');
            } when('ProductDataSheet') {
                $hInfo->{$Name} = _testAttributeFile($Object,'_ProductDataSheet',$LanguageID);
            }
        }
    }
    return $hInfo;
}

# test if the file for the given attribute name exists in the public path
# if not, set the attribute to undef and return undef (the merchant can also delete files in the slideshow!)
# if yes, return the attribute value
sub _testAttributeFile {
    my ($Object,$AttributeName,$LanguageID) = @_;

    my $SuperProduct = $Object->get('SuperProduct');
    my $SubAttributeValue = $Object->get($AttributeName,$LanguageID);

    my $AttributeValue = undef;
    $AttributeValue = $SubAttributeValue if $SubAttributeValue;
    $AttributeValue = $SuperProduct->get($AttributeName,$LanguageID) if ((defined $SuperProduct) and (not defined $AttributeValue));

    return undef unless $AttributeValue;

    my $PublicPath = $Object->get('PublicPath');
    my $FileName = $PublicPath.'/'.$AttributeValue;

    #set attribute to undef because we have no file
    if (not ExistsFile($FileName)) {
        #$Object->set({$AttributeName => undef },$LanguageID);
        return undef;
    }
    return $AttributeValue;
}


sub _prepareCSS {
    my ($Value) = @_;

    $Value = 7 if $Value >= 7;
    return 'class-'.$Value;

}

sub _AvailabilityAttribute {
    my ($Object) = @_;

    return if ($Object->get('IsAppointment'));

    my $TLEAttribute;
    my $StockAlert = $Object->get('StockAlert');

    if ($StockAlert eq 'ON_STOCK') {
        $TLEAttribute = 'ProductStockAvailableText';
    } elsif ($StockAlert eq 'WARN_STOCK') {
        $TLEAttribute = 'ProductStockCriticalText';
    } elsif ($StockAlert eq 'OUT_STOCK') {
        my $LowOnStockAction = $Object->getSite->get('ProductLowOnStockAction');
        # TODO: Shouldn't this be STOCK_HAS_LONGER_DELIVERYTIME?
        if($LowOnStockAction == STOCK_IS_ORDERABLE) {
           $TLEAttribute = 'ProductLongerDeliveryTimeText';
        } elsif ($LowOnStockAction == STOCK_NO_ADDTOBASKET) {
            $TLEAttribute = 'ProductNoAddToBasketText';
        }
    } elsif ($StockAlert eq 'NO_STOCK') {
        $TLEAttribute = 'ProductNoStockDataText';
    }
    return $TLEAttribute;
}

sub _AvailabilityIcon {
    my ($Object) = @_;

    my $TLEAttribute = _AvailabilityAttribute($Object);
    return unless (isValid($TLEAttribute));
    return unless ($Object->getSite->get('ShowAvailabilityIcons'));

    given($TLEAttribute) {
        when('ProductStockAvailableText') {
            return 'ProductOnStockIcon';
        } when ('ProductStockCriticalText') {
            return 'ProductWarnStockIcon';
        } when ('ProductLongerDeliveryTimeText') {
            return 'ProductOutStockIcon';
        } when ('ProductNoAddToBasketText') {
            return 'ProductOutStockIcon';
        } when ('ProductNoStockDataText') {
            return;
        }
        return;
    }
}

sub _AvailabilityText {
    my ($Object, $LanguageID) = @_;

    my $TLEAttribute = _AvailabilityAttribute($Object);
    return unless (isValid($TLEAttribute));

    my $Site = $Object->getSite();
    my $TLEText = RemoveScriptTags($Site->get($TLEAttribute, $LanguageID));

    my $hVarsObject = $Object->tleHash;
    $hVarsObject->{'INPUT'} = {
        'ObjectID' => $Object->id,
        'LanguageID' => $LanguageID,
    };

    return ReplaceTLEs($TLEText, $hVarsObject);
}

sub _GetPrice {
    my ($Object, $Attribute) = @_;
    return [
        sort { $a->{'CurrencyID'} cmp $b->{'CurrencyID'} }
        grep { defined $_->{$Attribute} }
        @{GetInfoProductPricesByProductID($Object->id)}
    ];
}

sub _GetImageThumbnail {
    my ($Object) = @_;
    return undef unless defined $Object->get('ImageSmall');

    my $PublicPath = $Object->get('PublicPath');
    my $ImageSmall = $Object->get('ImageSmall');
    my $ImageSmallFile = $PublicPath.'/'.$ImageSmall;
    return undef unless ExistsFile($ImageSmallFile);

    eval {
        TestImageFormat('ImageSmall', $ImageSmallFile);
    };
    if(ExistsError()) {
        GetLog->debug("UNSUPPORTED_IMAGE_FORMAT on file $ImageSmallFile");
        return $ImageSmall; # thumb-generating not possible, use same image for thumbnail
    }

    my $ImageThumbnail = $ImageSmall;
    $ImageThumbnail =~ s/(.*)_s(\..*?)$/$1_xs$2/x;
    my $ImageThumbnailFile = $PublicPath.'/'.$ImageThumbnail;

    # scale thumbnail from small image
    return $ImageThumbnail if ExistsFile($ImageThumbnailFile);

    my ($Width, $Height) = @{GetProductImageSizes($Object->getSite())->{'xs'}};
    eval {
        ScaleImage($ImageSmallFile, $ImageThumbnailFile, $Width, $Height);
    };
    if( ExistsError() ) {
        GetLog->debug("can't create thumbnail image $ImageThumbnailFile from $ImageSmallFile: ".
            GetError()->message);
        return $ImageSmall; # thumb-generating not possible, use same image for thumbnail
    }

    return $ImageThumbnail;
}

#========================================================================================
# function     setAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Package->setAttributes( $Object, $hValues );
# example      # never call setAttributes directly!
#               $Object->set( { $Name => $Value } );
#----------------------------------------------------------------------------------------
# description  Sets attribute values of an object.
#               Used by <function DE_EPAGES::Object::Object::Object::set>
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $hUpdate |  attribute name => value hash | ref.hash
# input        $LanguageID | (optional) language id for localized values | int
#========================================================================================
sub setAttributes {
    shift;
    my ($Object, $hUpdate, $LanguageID, $hAdditionalOptions) = @_;
    if ( exists $hUpdate->{'SelectedVariations'} ) {
        _UpdateSelectedVariations($Object, $hUpdate->{'SelectedVariations'} );
    }
    if ( exists $hUpdate->{'ShippingMethods'} ) {
        _UpdateShippingMethods($Object, $hUpdate->{'ShippingMethods'} );
    }
    if ( exists $hUpdate->{'StockLevel'} ) {
        # If we have a relative value, than we substract an amount of the stocklevel...
        if (defined ($hAdditionalOptions) && defined ($hAdditionalOptions->{'RelativeStockLevel'})) {
            $Object->setRelativeStockLevel($Object, $hAdditionalOptions->{'RelativeStockLevel'});
        } else {
            # ... else we set the stocklevel to a fix value.
            $Object->set({'_StockLevel' => $hUpdate->{'StockLevel'}}) unless $Object->get('IsBundleProduct');
        }
    }
    if ( exists $hUpdate->{'Manufacturer'} ) {
        _UpdateManufacturer($Object, $hUpdate->{'Manufacturer'} );
    }
    if (exists $hUpdate->{'EnergyLabels'}) {
        my $aParsed = ParseCSV(LoadRootObject->get('EnergyLabelsString'));
        my @SetLabels = grep { $_ ~~ @$aParsed } @{$hUpdate->{'EnergyLabels'}};
        $Object->set({'EnergyLabelsString' => join(',', @SetLabels)});
    }
    if ( exists $hUpdate->{'EnergyLabelSourceFile'} ) {
        $Object->set({'_EnergyLabelSourceFile' => $hUpdate->{'EnergyLabelSourceFile'}});
    }
    if ( exists $hUpdate->{'ProductDataSheet'} ) {
        $Object->set({'_ProductDataSheet' => $hUpdate->{'ProductDataSheet'}},$LanguageID);
    }

    return;
}

sub _UpdateManufacturer {
    my ($Product,$ManufacturerName) = @_;

    my $OldManufacturerName = $Product->get('Manufacturer');

    #no changes needed if names are the same
    $OldManufacturerName = $OldManufacturerName // '';
    $ManufacturerName = $ManufacturerName // '';

    return if $OldManufacturerName eq $ManufacturerName;

    my $SiteID = $Product->getSite->id;
    my $OldManufacturerID = $Product->get('ManufacturerID');
    my $ManufacturerID;


    #we have no new name
    if(not $ManufacturerName) {
        $Product->set({'ManufacturerID' => undef});
        # test if we can delete the old name from table
        _CleanUpManufacturer($SiteID,$OldManufacturerID);
        return;
    }

    # test if an entry for the new name already exists
    if (ExistsProductManufacturerBySiteID($SiteID, $ManufacturerName)) {
        $ManufacturerID = GetInfoProductManufacturerBySiteID($SiteID, $ManufacturerName)->{'ManufacturerID'};
    } else {
        $ManufacturerID = InsertProductManufacturer({'SiteID' => $SiteID, 'Name' => $ManufacturerName });
    }
    $Product->set({'ManufacturerID' => $ManufacturerID});

    #test if we can delete the old id from table
    _CleanUpManufacturer($SiteID,$OldManufacturerID);

    return;
}

# test if $OldManufacturerID is used at least with one product
# otherwise entry is deleted
sub _CleanUpManufacturer {
    my ($SiteID,$OldManufacturerID) = @_;
    return unless $OldManufacturerID;

    my $aProducts = LoadClassByAlias('Product')->findObjects('ByManufacturerID', [$OldManufacturerID]);

    DeleteProductManufacturer($OldManufacturerID) unless scalar @$aProducts;

    return;

}


sub _Manufacturer {
    my ($Product) = @_;

    # get ManufacturerID from product and read Manucaturer from productmanufacturer table
    my $ManufacturerID = $Product->get('ManufacturerID');

    return unless $ManufacturerID;
    return  GetInfoProductManufacturer($ManufacturerID)->{'Name'} if ExistsProductManufacturer($ManufacturerID);
    return ;
}

sub _UpdateShippingMethods{

    my ($Product,$aShippingMethods) = @_;

    my $OldShippingMethods = $Product->get('ShippingMethods');

    my (%SMTD, %SMTA);
    $SMTD{$_->id} = 1 foreach @$OldShippingMethods;
    foreach my $id (map {$_->id} @$aShippingMethods) {
        delete $SMTD{$id};
        $SMTA{$id} = 1;
    }
    delete $SMTA{$_->id} foreach @$OldShippingMethods;

    my $ProductID = $Product->id;
    DeleteProductShippingMethod($ProductID, $_) foreach keys %SMTD;
    InsertProductShippingMethod({'ProductID' => $ProductID,'ShippingMethodID' => $_}) foreach keys %SMTA;

    return;
}

sub _UpdateSelectedVariations {
    my ( $Product, $sSelectedVariations) = @_;

    #not for sub products
    return if $Product->isSubProduct;

    my $ahExistingValues = GetInfoSelectedVariationsByProductID($Product->id);
    my @VariationValues2Set = map { $_->{'SelectedID'} } @$sSelectedVariations;

    #add not existing values
    my %ExistingPreDefAttributeIDs = map { $_->{'PreDefAttributeID'} => undef } @$ahExistingValues;
    foreach my $ValueID ( @VariationValues2Set ) {
        # do nothing if already exists
        next if exists $ExistingPreDefAttributeIDs{$ValueID};

        my $PreDefValue = LoadObject($ValueID);
        my $Attribute = $PreDefValue->get('Attribute');
        InsertSelectedVariation({
            'ProductID'         => $Product->id,
            'AttributeID'       => $Attribute->id,
            'PreDefAttributeID' => $ValueID,
        });
    }

    #delete values which should not exist anymore
    my %UsedVariationValues = map { $_ => undef } @{ UsedVariationValues($Product) };
    my %VariationValues2Set = map { $_ => undef } @VariationValues2Set;
    foreach my $hExistingValue ( @$ahExistingValues ) {
        # do nothing if it should be set
        next if exists $VariationValues2Set{$hExistingValue->{'PreDefAttributeID'}};

        # warn if it is used by a subproduct
        if( exists $UsedVariationValues{$hExistingValue->{'PreDefAttributeID'}} ) {
            Warning('PreDefAttributeUsedBySubProduct', {
                'ProductID' => $Product->id,
                'PreDefAttributeID' => $hExistingValue->{'PreDefAttributeID'},
            });
        } else {
            DeleteSelectedVariation($hExistingValue->{'SelectedVariationID'} );
        }
    }

    # get used variation values again and set IsMaster to the correct value
    my $ahNewExistingValues = GetInfoSelectedVariationsByProductID($Product->id);
    $Product->set({ _IsMaster => 1 }) if (scalar(@{$ahNewExistingValues}) > 0) && !$Product->isMaster;
    $Product->set({ _IsMaster => 0 }) if (scalar(@{$ahNewExistingValues}) == 0) && $Product->isMaster;

    return;
}

# returns a loop of all attributes of an object
sub _UserDefinedAttributeValues {
    my ($Object, $LanguageID, $ClassAttributeName) = @_;

    return [
        map {
            {
                Attribute => $_,
                Value     => $Object->getWithParent($_->alias, $LanguageID),
            }
        }
        @{ $Object->class->get($ClassAttributeName) }
    ]
}

# returns the attribute value of 'Name' or the Alias if no name exists.
sub _NameOrAlias {
    my ($Object, $LanguageID) = @_;

    my $Name = $Object->getWithParent('Name', $LanguageID);
    return $Name if defined $Name && $Name ne '';
    return $Object->alias;
}

# returns a array of hashs with following keys
# Attribute => Object of Attribute order by position
# Values => array of PreDefAttributes which are in table PreDefAttributeLoop saved for this product order by position
sub _VariationTypeValues {
    my ($Product) = @_;

    my $rExistingValues = GetInfoSelectedVariationsByProductID($Product->id);

    my %Attributes;
    foreach my $PreDefAttributeValue ( @$rExistingValues ) {
        push @{$Attributes{$PreDefAttributeValue->{'AttributeID'}}{'Values'}}
            , LoadObject( $PreDefAttributeValue->{'PreDefAttributeID'} ) ;
    }
    #sort values by position
    foreach my $AttributeID (keys   %Attributes) {
        my $rValues = $Attributes{$AttributeID}{'Values'};
        $Attributes{$AttributeID}{'Values'} = [sort { $a->get('Position')<=> $b->get('Position') } @$rValues];
    }

    return [
        map {
            {
                Attribute => $_,
                Values    => $Attributes{$_->id}{'Values'},
            }
        }
        sort{ $a->get('Position') <=> $b->get('Position') }
        map { LoadObject($_) }
        keys %Attributes
    ];
}

#========================================================================================
# function     exportXMLElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->exportXMLElements($Driver, $Object, $aNames, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Adds sub elements of object for xml export.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver | object
# input        $Object | object to export | object
# input        $aNames | attribute names | ref.array.string
# input        $hAttributes | imported attributes | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $Object, $aNames, $hAttributes) = @_;
    if ($Object->class->get('ExportLevel') == $Driver->exportLevel) {
        $Driver->addElement( 'ProductPrice', { 'ProductPrice' => $_ } ) foreach @{GetInfoProductPricesByProductID($Object->id)};
    }
    if ($Object->class->attribute('ShippingMethods')->get('ExportLevel') == $Driver->exportLevel) {
        $Driver->addElement( 'ProductShippingMethod', { 'ProductShippingMethod' => $_ } ) foreach @{$Object->get('ShippingMethods')};
    }
    return;
}

sub _RemoteFiles {
    my ($Product) = @_;
    my $aAttributes = $Product->class->attributes;
    my $PublicPath = $Product->get('PublicPath');
    my %RemoteFiles;
    foreach my $Attribute (grep { $_->get('Type') eq 'File' } @$aAttributes) {
        my $AttributeAlias = $Attribute->alias;

        my $AttributeValue = $Product->get($AttributeAlias);
        my @Values;
        if (ref ($AttributeValue) ne 'ARRAY') {
            @Values = ($AttributeValue) if defined $AttributeValue;
        } else {
            foreach my $Value (@$AttributeValue) {
                # is ImagesSlideShow
                if (ref $Value eq 'HASH' and exists $Value->{'ImageName'}) {
                    push @Values, $Value->{'ImageName'};
                # other file loops
                } elsif (ref $Value eq 'HASH' and exists $Value->{'FileName'}) {
                    push @Values, $Value->{'FileName'};
                }
            }
        }
        foreach my $Value (@Values) {
            my $LocalFileName = $PublicPath.'/'.$Value;
            next if ExistsFile($LocalFileName);
            my (undef, $FName, undef) = fsplit($Value);
            my $UrlFileName = "$PublicPath/$FName.url";
            next if ExistsFile($UrlFileName);
            $RemoteFiles{$Value} = undef;
        }
    }
    return [ map { {'FileName' => $_} } keys %RemoteFiles];
}

sub _IsUPCEANValid {
    my ($Product) = @_;
    my $UPCEAN = $Product->getWithParent('UPCEAN');

    return 0 unless $UPCEAN;

    eval { TestUPCEAN('UPCEAN',$UPCEAN); };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) if $Error->code ne 'FORMAT_NOT_UPCEAN';
        return 0;
    }

    return 1;
}

#========================================================================================
# function     _ImagesSlideShow
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aResult = _ImagesSlideShow($Object);
# example      if( $Name eq 'ImagesSlideShow' ) {
#                   $hInfo->{$Name} = _ImagesSlideShow( $Object );
#               }
#----------------------------------------------------------------------------------------
# description  Parses the ImagesSlideShowString attribute of the product object and
#               returns the image file names of the corresponding image file variations
#               for the different sizes.
#----------------------------------------------------------------------------------------
# input        $Object | product object | object
# return       $ahResult or empty array | array of hashes with the image names and
#               different sizes (thumbnail, small, large, etc.) | ref.array of hashes
#========================================================================================
sub _ImagesSlideShow {
    my ($Object) = @_;

    my $ImagesSlideShowString = $Object->get('ImagesSlideShowString');
    return [] unless defined $ImagesSlideShowString;

    # list of all containing images with info, if image is from url or from public path
    my $NeedPublicPath = 0;
    my @ImagesInString = trim( split /;/, $ImagesSlideShowString );

    my $hAllImages;
    my $PositionCounter = 0;
    foreach my $Image (@ImagesInString) {
        # check if each image is a URL?
        if(StringIsURL($Image)) {
            $hAllImages->{$Image}->{'Type'} = 'URL';
        } else {
            # at least one of the images needs the product's public path
            $NeedPublicPath = 1;
            $hAllImages->{$Image}->{'Type'} = 'File';
        }
        $hAllImages->{$Image}->{'Position'} = $PositionCounter;
        $PositionCounter+=10; ## no critic qw(ProhibitMagicNumbers)
    }

    # empty if no public path
    my $hInfoDirectory;

    if ($NeedPublicPath) {
        if(not ExistsDirectory($Object->get('PublicPath'))){
            Warning(
                'MISSING_PUBLIC_PATH', {
                    'ObjectPath' => $Object->get('Path'),
                    'Directory' => $Object->get('PublicPath'),
                    'SlideShowString' => $ImagesSlideShowString
                }
            );
            #$Object->set({ 'ImagesSlideShowString' => undef });
            #$Object->set({ 'ImagesSlideShow' => () });
            return [];
        }
        $hInfoDirectory = GetInfoDirectory( $Object->get('PublicPath'), { recursive => 0, relative => 1 } );
    }

    my $DefaultImage = $Object->get('ImageLarge') // '';
    my $hSizes = {
        'Thumbnail' => '_xs',
        'Small' => '_s',
        'Medium' => '_m',
        'MediumSmall' => '_ms',
        'MediumLarge' => '_ml',
        'Large' => '',
        'HotDeal' => '_h',
    };

    my $ahResult = [];
    foreach my $Image (keys %$hAllImages) {
        if ($Image) {
            my $hResult = {
                'ImageName' => $Image,
                'Sizes' => {},
            };

            # Sizes info only if file and not for URL
            if($hAllImages->{$Image}->{'Type'} eq 'File') {
                # the default image of a product can have files that don't
                # follow the naming conventions
                if ($Image eq $DefaultImage) {
                    $hResult->{'Sizes'} = {
                        'Thumbnail' => $Object->get('ImageThumbnail'),
                        'Small' => $Object->get('ImageSmall'),
                        'Medium' => $Object->get('ImageMedium'),
                        'MediumSmall' => $Object->get('ImageMediumSmall'),
                        'MediumLarge' => $Object->get('ImageMediumLarge'),
                        'Large' => $DefaultImage,
                        'HotDeal' => $Object->get('ImageHotDeal'),
                    };
                }
                else {
                    # try to find the corresponding files by naming convention, e.g. "*_s*"
                    my ($FileName, $Extension) = ($Image =~ /(.*)(\.[^.]+)$/x);
                    foreach my $Size (keys %$hSizes) {
                        my $File = $FileName . $hSizes->{$Size} . $Extension;
                        $hResult->{'Sizes'}->{$Size} = ElementOf($File, @{$hInfoDirectory->{'Files'}}) ? $File : undef;
                    }
                    $hResult->{'Sizes'}->{'Thumbnail'} = $hResult->{'Sizes'}->{'Small'}
                        unless defined $hResult->{'Sizes'}->{'Thumbnail'};
                }
            } else {
                # map all sizes to same image
                foreach my $Size (keys %$hSizes) {
                    $hResult->{'Sizes'}->{$Size} = $Image;
                }
            }
            $hResult->{'Position'} = $hAllImages->{$Image}->{'Position'};
            push @$ahResult, $hResult;
        }
    }

    my @SortedResult = sort { $a->{'Position'} <=> $b->{'Position'} } @{$ahResult};
    return \@SortedResult;
}

#========================================================================================
# function     _BundleStockLevel
# state        private
#----------------------------------------------------------------------------------------
# syntax       $StockLevel = _BundleStockLevel($Object);
#----------------------------------------------------------------------------------------
# description  Returns the lowest stocklevel.
#               Returns undef if stock of all bundledproducts is undef.
#----------------------------------------------------------------------------------------
# input        $Object | product object | DE_EPAGES::Product::API::Object::Product
# return       $StockLevel | min stock level | float
#========================================================================================
sub _BundleStockLevel {
    my ($Object) = @_;

    my %Quantities;
    foreach my $BundledProductMap ( @{$Object->get('BundleProductMaps')} ) {
        my $BundledQuantity = $BundledProductMap->get('BundledQuantity');
        my $BundledProduct = $BundledProductMap->get('BundledProduct');
        if( $BundledProduct->get('IsBundleProduct') ) {
            Error('BundleLoop', {
                ProductID => $Object->id,
                BundledProductID => $BundledProduct->id,
            });
        }
        my $StockLevel = $BundledProduct->get('StockLevel');
        next unless defined $StockLevel;
        $Quantities{$BundledProduct->id} = int( $StockLevel / $BundledQuantity );
    }

    my @Quantities = values %Quantities;
    return undef if 0 == scalar @Quantities;

    my $StockLevel = shift @Quantities;
    do { $StockLevel = $_ if $_ < $StockLevel } foreach @Quantities;

    return $StockLevel;
}

#========================================================================================
# function     _DisplayDeliveryWeight
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DisplayDeliveryWeight = _DisplayDeliveryWeight($Object);
#----------------------------------------------------------------------------------------
# description  Returns the true if product has value for weight and shop has visible
#               shippingmethod of type ShippingMethodWeightOfBasket.
#----------------------------------------------------------------------------------------
# input        $Object | product object | DE_EPAGES::Product::API::Object::Product
# return       $DisplayDeliveryWeight | display delivery weight | boolean
#========================================================================================
sub _DisplayDeliveryWeight {
    my ($Object) = @_;

    return 0 unless $Object->getWithParent('Weight');
    my $Shop = $Object->getSite();
    return $Shop->get('HideProductWeigt') ? 0 : 1;
}

#========================================================================================
# function     _DisplayTitle
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DisplayTitle = _DisplayTitle($Product, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Returns the page title of the product detail page.
#----------------------------------------------------------------------------------------
# input        $Product | product object | DE_EPAGES::Product::API::Object::Product
# input        $LanguageID | language id | int
# return       $DisplayTitle | display title | string
#========================================================================================
sub _DisplayTitle {
    my ($Product, $LanguageID) = @_;

    my $Title = $Product->getWithParent('Title', $LanguageID);
    return $Title if defined $Title;

    my $ShopName = $Product->getSite->get('Name', $LanguageID);
    my $Name = $Product->getWithParent('NameOrAlias', $LanguageID);

    return $Name unless defined $ShopName;
    return $Name if $ShopName eq $Name;
    return "$Name - $ShopName";
}

#========================================================================================
# function     _UseSlideshow
# state        private
#----------------------------------------------------------------------------------------
# syntax       $UseSlideshow = _UseSlideshow($Product);
#----------------------------------------------------------------------------------------
# description  Returns true if the product displays a slidehow on the product detail page.
#               Note: variation products with custom images, but no custom slide show
#               do not use the slide show from the master product. (see EPG-31426)
#----------------------------------------------------------------------------------------
# input        $Product | product object | DE_EPAGES::Product::API::Object::Product
# return       $UseSlideshow | product uses a slideshow | boolean
#========================================================================================
sub _UseSlideshow {
    my ($Product) = @_;

    # get the sliseshow images that are directly assigned to this product or variation
    my $aImages = _ImagesSlideShow($Product);
    return 1 if scalar @$aImages > 0;

    my $SuperProduct = $Product->superProduct;
    return 0 unless defined $SuperProduct;

    # this is a sub product without a custom slideshow.
    # check if it has a custom detail image
    my $hImages = $Product->get([qw(ImageLarge ImageMedium)]); # get custom images
    return 0 if( defined $hImages->{'ImageLarge'} || defined $hImages->{'ImageMedium'} );

    # use the slideshow from the master product (if it has one)
    $aImages = _ImagesSlideShow($SuperProduct);
    return scalar @$aImages > 0 ? 1 : 0;
}

1;
