#========================================================================================
# package      DE_EPAGES::Product::DAL::SQL::Product
# state        private
# description  Generic SQL database interface for tables:
#               <ul>
#                   <li>productmanufacturer 
#                   <li>productavailabilityemail
#                   <li>productprepayment
#                   <li>productprice
#                   <li>productshippingmethod
#               </ul>
#========================================================================================
package DE_EPAGES::Product::DAL::SQL::Product;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Product::API::Constants qw( SMART_PRODUCT_SEARCH_RESULT_COUNT );

## no critic qw(ProhibitLongLines)

my %TABLE_DEF = (
    'ProductManufacturer' => {
        'ManufacturerID' => ['manufacturerid','Integer'],
        'SiteID' => ['siteid','Integer'],
        'Name' => ['name','String'],
    },
    'ProductAvailabilityEmail' => {
        'ProductID' => ['productid','Integer'],
        'EMail' => ['email','String'],
        'LocaleID' => ['localeid','String'],
    },
    'ProductPrepayment' => {
        'ProductID' => ['productid','Integer'],
        'CurrencyID' => ['currencyid','String'],
        'Value' => ['value','Money'],
    },
    'ProductPrice' => {
        'ProductID' => ['productid','Integer'],
        'CurrencyID' => ['currencyid','String'],
        'ListPrice' => ['listprice','Money'],
        'ManufacturerPrice' => ['manufacturerprice','Money'],
        'EcoParticipation' => ['ecoparticipation','Money'],
        'Deposit' => ['deposit','Money'],
    },
    'ProductShippingMethod' => {
        'ProductID' => ['productid','Integer'],
        'ShippingMethodID' => ['shippingmethodid','Integer'],
    },
);

sub ErrorDefinition {
    return {
        'TableNotExists'  => 'table #Table does not exist at package DE_EPAGES::Product::DAL::SQL::Product',
        'ColumnNotExists' => 'column #Column does not exist at table #Table at package DE_EPAGES::Product::DAL::SQL::Product',
    };
}

sub TableDef {
    my $self = shift;
    my ($Table, $Column) = @_;
    Error('TableNotExists',  {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table};
    Error('ColumnNotExists', {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table}{$Column};
    return $TABLE_DEF{$Table}{$Column}[0];
}

sub TableType {
    my $self = shift;
    my ($Table, $Column) = @_;
    Error('TableNotExists',  {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table};
    Error('ColumnNotExists', {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table}{$Column};
    return $TABLE_DEF{$Table}{$Column}[1];
}

#========================================================================================
# function     InsertProductManufacturer
#----------------------------------------------------------------------------------------
# syntax       $ManufacturerID = $Package->InsertProductManufacturer($dbi, $hParameter);
#----------------------------------------------------------------------------------------
# description  Inserts an entry into table productmanufacturer.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $hParameter | with following keys:
#               <ul>
#                  <li>ManufacturerID - manufacturer identifier (primary key)(optional)- int
#                  <li>SiteID        - site id              - int
#                  <li>Name          - manufacturer name    - varchar(100)
#               </ul> | reference to hash
# return       $ManufacturerID | manufacturer identifier (primary key) | int
#========================================================================================
sub InsertProductManufacturer {
    my $self = shift;
    my ($dbi, $hParameter) = @_;
    # auto-generated key
    my $ManufacturerID;
    $dbi->transaction( sub {
        $dbi->do('INSERT INTO productmanufacturer (manufacturerid, siteid, name)'.
            ' SELECT 1 + IFNULL(MAX(manufacturerid), 0), ?, ?'.
            ' FROM productmanufacturer',
            [ $hParameter->{'SiteID'}, $hParameter->{'Name'} ]
        );

        $ManufacturerID = $dbi->execute('SELECT MAX(manufacturerid) FROM productmanufacturer')->[0]->[0];
        die 'Failed to create primary key for productmanufacturer' unless defined $ManufacturerID;
    } );
    return $ManufacturerID;
}

#========================================================================================
# function     UpdateProductManufacturer
#----------------------------------------------------------------------------------------
# syntax       UpdateProductManufacturer($dbi, $ManufacturerID, $hParameter);
#----------------------------------------------------------------------------------------
# description  Updates an entry of table productmanufacturer.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ManufacturerID | manufacturer identifier (primary key) | int
# input        $hParameter | with following keys:
#               <ul>
#                   <li>SiteID        - site id              - int
#                   <li>Name          - manufacturer name    - varchar(100)
#               </ul>|reference to hash
#========================================================================================
sub UpdateProductManufacturer {
    my $self = shift;
    my ($dbi, $ManufacturerID, $hParameter) = @_;
    if (scalar keys %$hParameter) {
        my $sql = 'UPDATE productmanufacturer SET ';
        $sql .= join(', ', map { $self->TableDef('ProductManufacturer',$_).'=?' } sort keys %$hParameter);
        $sql .= ' WHERE manufacturerid=?';
        $dbi->do($sql, [(map { $hParameter->{$_} } sort keys %$hParameter), $ManufacturerID]);
    }
    return;
}

#========================================================================================
# function     DeleteProductManufacturer
#----------------------------------------------------------------------------------------
# syntax       $Package->DeleteProductManufacturer($dbi, $ManufacturerID);
#----------------------------------------------------------------------------------------
# description  Removes an entry of table productmanufacturer.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ManufacturerID | manufacturer identifier (primary key) | int
#========================================================================================
sub DeleteProductManufacturer {
    my $self = shift;
    my ($dbi, $ManufacturerID) = @_;
    $dbi->do('DELETE FROM productmanufacturer WHERE manufacturerid=?', [$ManufacturerID]);
    return;
}

#========================================================================================
# function     GetAllPKeyProductManufacturers
#----------------------------------------------------------------------------------------
# syntax       $aProductManufacturerIDs = $Package->GetAllPKeyProductManufacturers($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of table productmanufacturer.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $aProductManufacturerIDs | manufacturer identifier | reference to array of int
#========================================================================================
sub GetAllPKeyProductManufacturers {
    my $self = shift;
    my ($dbi) = @_;
    my $dbResult = $dbi->execute('SELECT manufacturerid FROM productmanufacturer', [], ['Integer']);
    return [ map { $_->[0] } @$dbResult ];
}

#========================================================================================
# function     GetAllInfoProductManufacturers
#----------------------------------------------------------------------------------------
# syntax       $ahProductManufacturerIDs = $Package->GetAllInfoProductManufacturers($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all rows of table productmanufacturer.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahProductManufacturerIDs | with following keys:
#               <ul>
#                   <li>ManufacturerID - manufacturer identifier (primary key)- int
#                   <li>SiteID        - site id              - int
#                   <li>Name          - manufacturer name    - varchar(100)
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoProductManufacturers {
    my $self = shift;
    my ($dbi) = @_;
    return $dbi->execute('SELECT manufacturerid, siteid, name FROM productmanufacturer', [],
        ['Integer', 'Integer', 'String'],
        [qw(ManufacturerID SiteID Name)]
    );
}

#========================================================================================
# function     GetInfoProductManufacturer
#----------------------------------------------------------------------------------------
# syntax       $hResult = $Package->GetInfoProductManufacturer($dbi, $ManufacturerID);
#----------------------------------------------------------------------------------------
# description  Returns a single row by primary key.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ManufacturerID | manufacturer identifier (primary key) | int
# return       $hResult | with following keys:
#               <ul>
#                   <li>ManufacturerID - manufacturer identifier (primary key)- int
#                   <li>SiteID        - site id              - int
#                   <li>Name          - manufacturer name    - varchar(100)
#               </ul>| reference to hash
#========================================================================================
sub GetInfoProductManufacturer {
    my $self = shift;
    my ($dbi, $ManufacturerID) = @_;
    return $dbi->execute('SELECT manufacturerid, siteid, name FROM productmanufacturer WHERE manufacturerid=?',
        [$ManufacturerID],
        ['Integer', 'Integer', 'String'],
        [qw(ManufacturerID SiteID Name)]
    )->[0];
}

#========================================================================================
# function     GetPKeyProductManufacturersBySiteID
#----------------------------------------------------------------------------------------
# syntax       $aResult = $Package->GetPKeyProductManufacturersBySiteID($dbi, $SiteID, $Name);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table productmanufacturer by index i_siteid_name.
#               Uses the index i_siteid_name.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $SiteID | site id | int
# input        $Name | manufacturer name (optional) | varchar(100)
# return       $aManufacturerIDs | manufacturer identifier (primary key) | reference to array of int
#========================================================================================
sub GetPKeyProductManufacturersBySiteID {
    my $self = shift;
    my ($dbi, $SiteID, $Name) = @_;
    my (@WhereClauses, @Parameter);
    if (defined $Name) {
        unshift @WhereClauses, 'name = ?';
        unshift @Parameter, $Name;
    }
    else {
        @WhereClauses = ();
        @Parameter = ()
    }
    unshift @WhereClauses, 'siteid = ?';
    unshift @Parameter, $SiteID;

    my $sql = 'SELECT manufacturerid FROM productmanufacturer WHERE ' . join(' AND ', @WhereClauses);
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer']);
    return [ map { $_->[0] } @$dbResult ];
}

#========================================================================================
# function     GetInfoProductManufacturersBySiteID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoProductManufacturersBySiteID($dbi, $SiteID, $Name);
#----------------------------------------------------------------------------------------
# description  get full row of table productmanufacturer by index i_siteid_name
#               Uses the index i_siteid_name.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $SiteID | site id | int
# input        $Name | manufacturer name (optional) | varchar(100)
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ManufacturerID - manufacturer identifier (primary key)- int
#                   <li>SiteID        - site id              - int
#                   <li>Name          - manufacturer name    - varchar(100)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoProductManufacturersBySiteID {
    my $self = shift;
    my ($dbi, $SiteID, $Name) = @_;
    my (@WhereClauses, @Parameter);
    if (defined $Name) {
        unshift @WhereClauses, 'name = ?';
        unshift @Parameter, $Name;
    }
    else {
        @WhereClauses = ();
        @Parameter = ()
    }
    unshift @WhereClauses, 'siteid = ?';
    unshift @Parameter, $SiteID;

    my $sql = 'SELECT manufacturerid, siteid, name FROM productmanufacturer WHERE ' . join(' AND ', @WhereClauses);
    return $dbi->execute($sql, \@Parameter,
        ['Integer', 'Integer', 'String'],
        [qw(ManufacturerID SiteID Name)]
    );
}

#========================================================================================
# function     InsertProductAvailabilityEmail
#----------------------------------------------------------------------------------------
# syntax       $Package->InsertProductAvailabilityEmail($dbi, $hParameter);
#----------------------------------------------------------------------------------------
# description  Inserts an entry into table productavailabilityemail.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $hParameter | with following keys:
#               <ul>
#                  <li>ProductID     - product identifier   (part of primary key)- int
#                  <li>EMail         - email to send availablity information to (part of primary key)- varchar(150)
#                  <li>LocaleID      - locale id, example en_GB - char(5)
#               </ul> | reference to hash
#========================================================================================
sub InsertProductAvailabilityEmail {
    my $self = shift;
    my ($dbi, $hParameter) = @_;
    $dbi->do('INSERT INTO productavailabilityemail (productid, email, localeid) VALUES (?, ?, ?)',
        [ @{$hParameter}{qw( ProductID EMail LocaleID )} ]
    );
}

#========================================================================================
# function     UpdateProductAvailabilityEmail
#----------------------------------------------------------------------------------------
# syntax       UpdateProductAvailabilityEmail($dbi, $ProductID, $EMail, $hParameter);
#----------------------------------------------------------------------------------------
# description  Updates an entry of table productavailabilityemail.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $EMail         | email to send availablity information to (part of primary key) | varchar(150)
# input        $hParameter | with following keys:
#               <ul>
#                   <li>LocaleID      - locale id, example en_GB - char(5)
#               </ul>|reference to hash
#========================================================================================
sub UpdateProductAvailabilityEmail {
    my $self = shift;
    my ($dbi, $ProductID, $EMail, $hParameter) = @_;
    if (scalar keys %$hParameter) {
        my $sql = 'UPDATE productavailabilityemail SET ';
        $sql .= join(', ', map { $self->TableDef('ProductAvailabilityEmail',$_).'=?' } sort keys %$hParameter);
        $sql .= ' WHERE productid=? AND email=?';
        $dbi->do($sql, [(map { $hParameter->{$_} } sort keys %$hParameter), $ProductID, $EMail]);
    }
    return;
}

#========================================================================================
# function     UpdateProductAvailabilityEmailLocaleID
#----------------------------------------------------------------------------------------
# syntax       UpdateProductAvailabilityEmailLocaleID($dbh, $LocaleIDOld, $LocaleIDNew);
#----------------------------------------------------------------------------------------
# description  Updates the locale id of table productavailabilityemail.
#----------------------------------------------------------------------------------------
# input        $dbh | database handle | object
# input        $LocaleIDOld     | old local id | string
# input        $LocaleIDNew     | new local id | string
#========================================================================================
sub UpdateProductAvailabilityEmailLocaleID {
    my $self = shift;
    my ($dbi, $LocaleIDOld, $LocaleIDNew) = @_;

    my $sql = 'UPDATE productavailabilityemail SET ';
    $sql .= 'localeid=? WHERE localeid=?';
    $dbi->do($sql,[$LocaleIDOld, $LocaleIDNew], ['String', 'String']);
    return;
}

#========================================================================================
# function     DeleteProductAvailabilityEmail
#----------------------------------------------------------------------------------------
# syntax       $Package->DeleteProductAvailabilityEmail($dbi, $ProductID, $EMail);
#----------------------------------------------------------------------------------------
# description  Removes an entry of table productavailabilityemail.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $EMail         | email to send availablity information to (part of primary key) | varchar(150)
#========================================================================================
sub DeleteProductAvailabilityEmail {
    my $self = shift;
    my ($dbi, $ProductID, $EMail) = @_;
    $dbi->do('DELETE FROM productavailabilityemail WHERE productid=? AND email=?', [$ProductID, $EMail]);
    return;
}

#========================================================================================
# function     GetAllPKeyProductAvailabilityEmails
#----------------------------------------------------------------------------------------
# syntax       $ahProductAvailabilityEmailIDs = $Package->GetAllPKeyProductAvailabilityEmails($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of table productavailabilityemail.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahProductAvailabilityEmailIDs | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>EMail         - email to send availablity information to (part of primary key)- varchar(150)
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllPKeyProductAvailabilityEmails {
    my $self = shift;
    my ($dbi) = @_;
    my $dbResult = $dbi->execute('SELECT productid, email FROM productavailabilityemail', [], ['Integer', 'String'], [qw(ProductID EMail)]);
    return $dbResult;
}

#========================================================================================
# function     GetAllInfoProductAvailabilityEmails
#----------------------------------------------------------------------------------------
# syntax       $ahProductAvailabilityEmailIDs = $Package->GetAllInfoProductAvailabilityEmails($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all rows of table productavailabilityemail.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahProductAvailabilityEmailIDs | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>EMail         - email to send availablity information to (part of primary key)- varchar(150)
#                   <li>LocaleID      - locale id, example en_GB - char(5)
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoProductAvailabilityEmails {
    my $self = shift;
    my ($dbi) = @_;
    return $dbi->execute('SELECT productid, email, localeid FROM productavailabilityemail', [],
        ['Integer', 'String', 'String'],
        [qw(ProductID EMail LocaleID)]
    );
}

#========================================================================================
# function     GetInfoProductAvailabilityEmail
#----------------------------------------------------------------------------------------
# syntax       $hResult = $Package->GetInfoProductAvailabilityEmail($dbi, $ProductID, $EMail);
#----------------------------------------------------------------------------------------
# description  Returns a single row by primary key.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $EMail         | email to send availablity information to (part of primary key) | varchar(150)
# return       $hResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>EMail         - email to send availablity information to (part of primary key)- varchar(150)
#                   <li>LocaleID      - locale id, example en_GB - char(5)
#               </ul>| reference to hash
#========================================================================================
sub GetInfoProductAvailabilityEmail {
    my $self = shift;
    my ($dbi, $ProductID, $EMail) = @_;
    return $dbi->execute('SELECT productid, email, localeid FROM productavailabilityemail WHERE productid=? AND email=?',
        [$ProductID, $EMail],
        ['Integer', 'String', 'String'],
        [qw(ProductID EMail LocaleID)]
    )->[0];
}

#========================================================================================
# function     GetPKeyProductAvailabilityEmailsByProductID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyProductAvailabilityEmailsByProductID($dbi, $ProductID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table productavailabilityemail by primary key.
#               Uses the index pk_productavailabilityemail.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID | product identifier | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>EMail         - email to send availablity information to (part of primary key)- varchar(150)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyProductAvailabilityEmailsByProductID {
    my $self = shift;
    my ($dbi, $ProductID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'productid = ?';
    unshift @Parameter, $ProductID;

    my $sql = 'SELECT productid, email FROM productavailabilityemail WHERE ' . join(' AND ', @WhereClauses);
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'String'], [qw(ProductID EMail)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoProductAvailabilityEmailsByProductID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoProductAvailabilityEmailsByProductID($dbi, $ProductID);
#----------------------------------------------------------------------------------------
# description  Returns full rows of table productavailabilityemail by primary key.
#               Uses the index pk_productavailabilityemail.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID | product identifier | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>EMail         - email to send availablity information to (part of primary key)- varchar(150)
#                   <li>LocaleID      - locale id, example en_GB - char(5)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoProductAvailabilityEmailsByProductID {
    my $self = shift;
    my ($dbi, $ProductID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'productid = ?';
    unshift @Parameter, $ProductID;

    my $sql = 'SELECT productid, email, localeid FROM productavailabilityemail WHERE ' . join(' AND ', @WhereClauses);
    return $dbi->execute($sql, \@Parameter,
        ['Integer', 'String', 'String'],
        [qw(ProductID EMail LocaleID)]
    );
}

#========================================================================================
# function     GetPKeyProductAvailabilityEmailsByEMail
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyProductAvailabilityEmailsByEMail($dbi, $EMail);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table productavailabilityemail by index i_productavailability.
#               Uses the index i_productavailability.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $EMail | email to send availablity information to | varchar(150)
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>EMail         - email to send availablity information to (part of primary key)- varchar(150)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyProductAvailabilityEmailsByEMail {
    my $self = shift;
    my ($dbi, $EMail) = @_;
    my @Parameter = ($EMail);
    my $sql = 'SELECT productid, email FROM productavailabilityemail WHERE email = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'String'], [qw(ProductID EMail)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoProductAvailabilityEmailsByEMail
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoProductAvailabilityEmailsByEMail($dbi, $EMail);
#----------------------------------------------------------------------------------------
# description  get full row of table productavailabilityemail by index i_productavailability
#               Uses the index i_productavailability.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $EMail | email to send availablity information to | varchar(150)
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>EMail         - email to send availablity information to (part of primary key)- varchar(150)
#                   <li>LocaleID      - locale id, example en_GB - char(5)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoProductAvailabilityEmailsByEMail {
    my $self = shift;
    my ($dbi, $EMail) = @_;
    return $dbi->execute('SELECT productid, email, localeid FROM productavailabilityemail WHERE email = ?',
        [$EMail],
        ['Integer', 'String', 'String'],
        [qw(ProductID EMail LocaleID)]
    );
}

#========================================================================================
# function     GetPKeyProductAvailabilityEmailsByLocaleID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyProductAvailabilityEmailsByLocaleID($dbi, $LocaleID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table productavailabilityemail by index i_productavailabilityemail_locale.
#               Uses the index i_productavailabilityemail_locale.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $LocaleID | locale id, example en_GB | char(5)
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>EMail         - email to send availablity information to (part of primary key)- varchar(150)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyProductAvailabilityEmailsByLocaleID {
    my $self = shift;
    my ($dbi, $LocaleID) = @_;
    my @Parameter = ($LocaleID);
    my $sql = 'SELECT productid, email FROM productavailabilityemail WHERE localeid = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'String'], [qw(ProductID EMail)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoProductAvailabilityEmailsByLocaleID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoProductAvailabilityEmailsByLocaleID($dbi, $LocaleID);
#----------------------------------------------------------------------------------------
# description  get full row of table productavailabilityemail by index i_productavailabilityemail_locale
#               Uses the index i_productavailabilityemail_locale.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $LocaleID | locale id, example en_GB | char(5)
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>EMail         - email to send availablity information to (part of primary key)- varchar(150)
#                   <li>LocaleID      - locale id, example en_GB - char(5)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoProductAvailabilityEmailsByLocaleID {
    my $self = shift;
    my ($dbi, $LocaleID) = @_;
    return $dbi->execute('SELECT productid, email, localeid FROM productavailabilityemail WHERE localeid = ?',
        [$LocaleID],
        ['Integer', 'String', 'String'],
        [qw(ProductID EMail LocaleID)]
    );
}

#========================================================================================
# function     GetPKeyProductsByAlias
#----------------------------------------------------------------------------------------
# syntax       $aResult = $Package->GetPKeyProductsByAlias($dbh, $ShopID, $Alias);
#----------------------------------------------------------------------------------------
# description  Get list of ProductIDs by product alias for a specified shop.
#----------------------------------------------------------------------------------------
# input        $dbh | database handle | object
# input        $Shop | shop | object
# input        $Alias | product alias | string
# return       $aResult | list of product ids | array
#========================================================================================
sub GetPKeyProductsByAlias {
    my $self = shift;
    my ($dbi, $ShopID, $Alias) = @_;

    my $sql = 'SELECT productid FROM product p inner join object o on (o.objectid = p.productid) ' .
        'WHERE p.shopid = ? and o.alias = ? and o.siteid = ?';
    my $dbResult = $dbi->execute($sql, [$ShopID, $Alias, $ShopID], ['Integer']);
    return [ map { $_->[0] } @$dbResult];
}

#========================================================================================
# function     InsertProductPrepayment
#----------------------------------------------------------------------------------------
# syntax       $Package->InsertProductPrepayment($dbi, $hParameter);
#----------------------------------------------------------------------------------------
# description  Inserts an entry into table productprepayment.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $hParameter | with following keys:
#               <ul>
#                  <li>ProductID     - product identifier   (part of primary key)- int
#                  <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#                  <li>Value         -                      - money
#               </ul> | reference to hash
#========================================================================================
sub InsertProductPrepayment {
    my $self = shift;
    my ($dbi, $hParameter) = @_;
    $dbi->do('INSERT INTO productprepayment (productid, currencyid, value) VALUES (?, ?, ?)',
        [ @{$hParameter}{qw( ProductID CurrencyID Value )} ]
    );
}

#========================================================================================
# function     UpdateProductPrepayment
#----------------------------------------------------------------------------------------
# syntax       UpdateProductPrepayment($dbi, $ProductID, $CurrencyID, $hParameter);
#----------------------------------------------------------------------------------------
# description  Updates an entry of table productprepayment.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $CurrencyID    | alpha currency code (ISO 4217) (part of primary key) | char(3)
# input        $hParameter | with following keys:
#               <ul>
#                   <li>Value         -                      - money
#               </ul>|reference to hash
#========================================================================================
sub UpdateProductPrepayment {
    my $self = shift;
    my ($dbi, $ProductID, $CurrencyID, $hParameter) = @_;
    if (scalar keys %$hParameter) {
        my $sql = 'UPDATE productprepayment SET ';
        $sql .= join(', ', map { $self->TableDef('ProductPrepayment',$_).'=?' } sort keys %$hParameter);
        $sql .= ' WHERE productid=? AND currencyid=?';
        $dbi->do($sql, [(map { $hParameter->{$_} } sort keys %$hParameter), $ProductID, $CurrencyID]);
    }
    return;
}

#========================================================================================
# function     DeleteProductPrepayment
#----------------------------------------------------------------------------------------
# syntax       $Package->DeleteProductPrepayment($dbi, $ProductID, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  Removes an entry of table productprepayment.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $CurrencyID    | alpha currency code (ISO 4217) (part of primary key) | char(3)
#========================================================================================
sub DeleteProductPrepayment {
    my $self = shift;
    my ($dbi, $ProductID, $CurrencyID) = @_;
    $dbi->do('DELETE FROM productprepayment WHERE productid=? AND currencyid=?', [$ProductID, $CurrencyID]);
    return;
}

#========================================================================================
# function     GetAllPKeyProductPrepayments
#----------------------------------------------------------------------------------------
# syntax       $ahProductPrepaymentIDs = $Package->GetAllPKeyProductPrepayments($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of table productprepayment.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahProductPrepaymentIDs | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllPKeyProductPrepayments {
    my $self = shift;
    my ($dbi) = @_;
    my $dbResult = $dbi->execute('SELECT productid, currencyid FROM productprepayment', [], ['Integer', 'String'], [qw(ProductID CurrencyID)]);
    return $dbResult;
}

#========================================================================================
# function     GetAllInfoProductPrepayments
#----------------------------------------------------------------------------------------
# syntax       $ahProductPrepaymentIDs = $Package->GetAllInfoProductPrepayments($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all rows of table productprepayment.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahProductPrepaymentIDs | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#                   <li>Value         -                      - money
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoProductPrepayments {
    my $self = shift;
    my ($dbi) = @_;
    return $dbi->execute('SELECT productid, currencyid, value FROM productprepayment', [],
        ['Integer', 'String', 'Money'],
        [qw(ProductID CurrencyID Value)]
    );
}

#========================================================================================
# function     GetInfoProductPrepayment
#----------------------------------------------------------------------------------------
# syntax       $hResult = $Package->GetInfoProductPrepayment($dbi, $ProductID, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  Returns a single row by primary key.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $CurrencyID    | alpha currency code (ISO 4217) (part of primary key) | char(3)
# return       $hResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#                   <li>Value         -                      - money
#               </ul>| reference to hash
#========================================================================================
sub GetInfoProductPrepayment {
    my $self = shift;
    my ($dbi, $ProductID, $CurrencyID) = @_;
    return $dbi->execute('SELECT productid, currencyid, value FROM productprepayment WHERE productid=? AND currencyid=?',
        [$ProductID, $CurrencyID],
        ['Integer', 'String', 'Money'],
        [qw(ProductID CurrencyID Value)]
    )->[0];
}

#========================================================================================
# function     GetPKeyProductPrepaymentsByProductID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyProductPrepaymentsByProductID($dbi, $ProductID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table productprepayment by primary key.
#               Uses the index pk_productprepayment.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID | product identifier | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyProductPrepaymentsByProductID {
    my $self = shift;
    my ($dbi, $ProductID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'productid = ?';
    unshift @Parameter, $ProductID;

    my $sql = 'SELECT productid, currencyid FROM productprepayment WHERE ' . join(' AND ', @WhereClauses);
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'String'], [qw(ProductID CurrencyID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoProductPrepaymentsByProductID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoProductPrepaymentsByProductID($dbi, $ProductID);
#----------------------------------------------------------------------------------------
# description  Returns full rows of table productprepayment by primary key.
#               Uses the index pk_productprepayment.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID | product identifier | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#                   <li>Value         -                      - money
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoProductPrepaymentsByProductID {
    my $self = shift;
    my ($dbi, $ProductID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'productid = ?';
    unshift @Parameter, $ProductID;

    my $sql = 'SELECT productid, currencyid, value FROM productprepayment WHERE ' . join(' AND ', @WhereClauses);
    return $dbi->execute($sql, \@Parameter,
        ['Integer', 'String', 'Money'],
        [qw(ProductID CurrencyID Value)]
    );
}

#========================================================================================
# function     GetPKeyProductPrepaymentsByCurrencyID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyProductPrepaymentsByCurrencyID($dbi, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table productprepayment by index i_productprepayment_currency.
#               Uses the index i_productprepayment_currency.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $CurrencyID | alpha currency code (ISO 4217) | char(3)
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyProductPrepaymentsByCurrencyID {
    my $self = shift;
    my ($dbi, $CurrencyID) = @_;
    my @Parameter = ($CurrencyID);
    my $sql = 'SELECT productid, currencyid FROM productprepayment WHERE currencyid = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'String'], [qw(ProductID CurrencyID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoProductPrepaymentsByCurrencyID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoProductPrepaymentsByCurrencyID($dbi, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  get full row of table productprepayment by index i_productprepayment_currency
#               Uses the index i_productprepayment_currency.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $CurrencyID | alpha currency code (ISO 4217) | char(3)
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#                   <li>Value         -                      - money
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoProductPrepaymentsByCurrencyID {
    my $self = shift;
    my ($dbi, $CurrencyID) = @_;
    return $dbi->execute('SELECT productid, currencyid, value FROM productprepayment WHERE currencyid = ?',
        [$CurrencyID],
        ['Integer', 'String', 'Money'],
        [qw(ProductID CurrencyID Value)]
    );
}

#========================================================================================
# function     InsertProductPrice
#----------------------------------------------------------------------------------------
# syntax       $Package->InsertProductPrice($dbi, $hParameter);
#----------------------------------------------------------------------------------------
# description  Inserts an entry into table productprice.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $hParameter | with following keys:
#               <ul>
#                  <li>ProductID     - product identifier   (part of primary key)- int
#                  <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#                  <li>ListPrice     - price amount         (optional)- money
#                  <li>ManufacturerPrice -                      (optional)- money
#                  <li>EcoParticipation - Additional fee for electronic products in France (optional)- money
#                  <li>Deposit       - Deposit (e.g. for bottles) (optional)- money
#               </ul> | reference to hash
#========================================================================================
sub InsertProductPrice {
    my $self = shift;
    my ($dbi, $hParameter) = @_;

    foreach my $key (keys %$hParameter){
        $hParameter->{$key} = undef if $hParameter->{$key} eq '';
    }

    $dbi->do('INSERT INTO productprice (productid, currencyid, listprice, manufacturerprice, ecoparticipation, deposit) VALUES (?, ?, ?, ?, ?, ?)',
        [ @{$hParameter}{qw( ProductID CurrencyID ListPrice ManufacturerPrice EcoParticipation Deposit )} ]
    );
}

#========================================================================================
# function     UpdateProductPrice
#----------------------------------------------------------------------------------------
# syntax       UpdateProductPrice($dbi, $ProductID, $CurrencyID, $hParameter);
#----------------------------------------------------------------------------------------
# description  Updates an entry of table productprice.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $CurrencyID    | alpha currency code (ISO 4217) (part of primary key) | char(3)
# input        $hParameter | with following keys:
#               <ul>
#                   <li>ListPrice     - price amount         (optional)- money
#                   <li>ManufacturerPrice -                      (optional)- money
#                   <li>EcoParticipation - Additional fee for electronic products in France (optional)- money
#                   <li>Deposit       - Deposit (e.g. for bottles) (optional)- money
#               </ul>|reference to hash
#========================================================================================
sub UpdateProductPrice {
    my $self = shift;
    my ($dbi, $ProductID, $CurrencyID, $hParameter) = @_;
    if (scalar keys %$hParameter) {
        my $sql = 'UPDATE productprice SET ';
        $sql .= join(', ', map { $self->TableDef('ProductPrice',$_).'=?' } sort keys %$hParameter);
        $sql .= ' WHERE productid=? AND currencyid=?';
        $dbi->do($sql, [(map { $hParameter->{$_} ne '' ? $hParameter->{$_} : undef } sort keys %$hParameter), $ProductID, $CurrencyID]);
    }
    return;
}

#========================================================================================
# function     DeleteProductPrice
#----------------------------------------------------------------------------------------
# syntax       $Package->DeleteProductPrice($dbi, $ProductID, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  Removes an entry of table productprice.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $CurrencyID    | alpha currency code (ISO 4217) (part of primary key) | char(3)
#========================================================================================
sub DeleteProductPrice {
    my $self = shift;
    my ($dbi, $ProductID, $CurrencyID) = @_;
    $dbi->do('DELETE FROM productprice WHERE productid=? AND currencyid=?', [$ProductID, $CurrencyID]);
    return;
}

#========================================================================================
# function     GetAllPKeyProductPrices
#----------------------------------------------------------------------------------------
# syntax       $ahProductPriceIDs = $Package->GetAllPKeyProductPrices($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of table productprice.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahProductPriceIDs | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllPKeyProductPrices {
    my $self = shift;
    my ($dbi) = @_;
    my $dbResult = $dbi->execute('SELECT productid, currencyid FROM productprice', [], ['Integer', 'String'], [qw(ProductID CurrencyID)]);
    return $dbResult;
}

#========================================================================================
# function     GetAllInfoProductPrices
#----------------------------------------------------------------------------------------
# syntax       $ahProductPriceIDs = $Package->GetAllInfoProductPrices($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all rows of table productprice.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahProductPriceIDs | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#                   <li>ListPrice     - price amount         (optional)- money
#                   <li>ManufacturerPrice -                      (optional)- money
#                   <li>EcoParticipation - Additional fee for electronic products in France (optional)- money
#                   <li>Deposit       - Deposit (e.g. for bottles) (optional)- money
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoProductPrices {
    my $self = shift;
    my ($dbi) = @_;
    return $dbi->execute('SELECT productid, currencyid, listprice, manufacturerprice, ecoparticipation, deposit FROM productprice', [],
        ['Integer', 'String', 'Money', 'Money', 'Money', 'Money'],
        [qw(ProductID CurrencyID ListPrice ManufacturerPrice EcoParticipation Deposit)]
    );
}

#========================================================================================
# function     GetInfoProductPrice
#----------------------------------------------------------------------------------------
# syntax       $hResult = $Package->GetInfoProductPrice($dbi, $ProductID, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  Returns a single row by primary key.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $CurrencyID    | alpha currency code (ISO 4217) (part of primary key) | char(3)
# return       $hResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#                   <li>ListPrice     - price amount         (optional)- money
#                   <li>ManufacturerPrice -                      (optional)- money
#                   <li>EcoParticipation - Additional fee for electronic products in France (optional)- money
#                   <li>Deposit       - Deposit (e.g. for bottles) (optional)- money
#               </ul>| reference to hash
#========================================================================================
sub GetInfoProductPrice {
    my $self = shift;
    my ($dbi, $ProductID, $CurrencyID) = @_;
    return $dbi->execute('SELECT productid, currencyid, listprice, manufacturerprice, ecoparticipation, deposit FROM productprice WHERE productid=? AND currencyid=?',
        [$ProductID, $CurrencyID],
        ['Integer', 'String', 'Money', 'Money', 'Money', 'Money'],
        [qw(ProductID CurrencyID ListPrice ManufacturerPrice EcoParticipation Deposit)]
    )->[0];
}

#========================================================================================
# function     GetPKeyProductPricesByProductID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyProductPricesByProductID($dbi, $ProductID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table productprice by primary key.
#               Uses the index pk_productprice.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID | product identifier | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyProductPricesByProductID {
    my $self = shift;
    my ($dbi, $ProductID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'productid = ?';
    unshift @Parameter, $ProductID;

    my $sql = 'SELECT productid, currencyid FROM productprice WHERE ' . join(' AND ', @WhereClauses);
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'String'], [qw(ProductID CurrencyID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoProductPricesByProductID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoProductPricesByProductID($dbi, $ProductID);
#----------------------------------------------------------------------------------------
# description  Returns full rows of table productprice by primary key.
#               Uses the index pk_productprice.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID | product identifier | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#                   <li>ListPrice     - price amount         (optional)- money
#                   <li>ManufacturerPrice -                      (optional)- money
#                   <li>EcoParticipation - Additional fee for electronic products in France (optional)- money
#                   <li>Deposit       - Deposit (e.g. for bottles) (optional)- money
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoProductPricesByProductID {
    my $self = shift;
    my ($dbi, $ProductID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'productid = ?';
    unshift @Parameter, $ProductID;

    my $sql = 'SELECT productid, currencyid, listprice, manufacturerprice, ecoparticipation, deposit FROM productprice WHERE ' . join(' AND ', @WhereClauses);
    return $dbi->execute($sql, \@Parameter,
        ['Integer', 'String', 'Money', 'Money', 'Money', 'Money'],
        [qw(ProductID CurrencyID ListPrice ManufacturerPrice EcoParticipation Deposit)]
    );
}

#========================================================================================
# function     GetPKeyProductPricesByCurrencyID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyProductPricesByCurrencyID($dbi, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table productprice by index i_productprice_currency.
#               Uses the index i_productprice_currency.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $CurrencyID | alpha currency code (ISO 4217) | char(3)
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyProductPricesByCurrencyID {
    my $self = shift;
    my ($dbi, $CurrencyID) = @_;
    my @Parameter = ($CurrencyID);
    my $sql = 'SELECT productid, currencyid FROM productprice WHERE currencyid = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'String'], [qw(ProductID CurrencyID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoProductPricesByCurrencyID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoProductPricesByCurrencyID($dbi, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  get full row of table productprice by index i_productprice_currency
#               Uses the index i_productprice_currency.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $CurrencyID | alpha currency code (ISO 4217) | char(3)
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (part of primary key)- char(3)
#                   <li>ListPrice     - price amount         (optional)- money
#                   <li>ManufacturerPrice -                      (optional)- money
#                   <li>EcoParticipation - Additional fee for electronic products in France (optional)- money
#                   <li>Deposit       - Deposit (e.g. for bottles) (optional)- money
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoProductPricesByCurrencyID {
    my $self = shift;
    my ($dbi, $CurrencyID) = @_;
    return $dbi->execute('SELECT productid, currencyid, listprice, manufacturerprice, ecoparticipation, deposit FROM productprice WHERE currencyid = ?',
        [$CurrencyID],
        ['Integer', 'String', 'Money', 'Money', 'Money', 'Money'],
        [qw(ProductID CurrencyID ListPrice ManufacturerPrice EcoParticipation Deposit)]
    );
}

#========================================================================================
# function     InsertProductShippingMethod
#----------------------------------------------------------------------------------------
# syntax       $Package->InsertProductShippingMethod($dbi, $hParameter);
#----------------------------------------------------------------------------------------
# description  Inserts an entry into table productshippingmethod.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $hParameter | with following keys:
#               <ul>
#                  <li>ProductID     - product identifier   (part of primary key)- int
#                  <li>ShippingMethodID - shipping method id   (part of primary key)- int
#               </ul> | reference to hash
#========================================================================================
sub InsertProductShippingMethod {
    my $self = shift;
    my ($dbi, $hParameter) = @_;
    $dbi->do('INSERT INTO productshippingmethod (productid, shippingmethodid) VALUES (?, ?)',
        [ @{$hParameter}{qw( ProductID ShippingMethodID )} ]
    );
}

#========================================================================================
# function     DeleteProductShippingMethod
#----------------------------------------------------------------------------------------
# syntax       $Package->DeleteProductShippingMethod($dbi, $ProductID, $ShippingMethodID);
#----------------------------------------------------------------------------------------
# description  Removes an entry of table productshippingmethod.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $ShippingMethodID | shipping method id   (part of primary key) | int
#========================================================================================
sub DeleteProductShippingMethod {
    my $self = shift;
    my ($dbi, $ProductID, $ShippingMethodID) = @_;
    $dbi->do('DELETE FROM productshippingmethod WHERE productid=? AND shippingmethodid=?', [$ProductID, $ShippingMethodID]);
    return;
}

#========================================================================================
# function     GetAllPKeyProductShippingMethods
#----------------------------------------------------------------------------------------
# syntax       $ahProductShippingMethodIDs = $Package->GetAllPKeyProductShippingMethods($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of table productshippingmethod.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahProductShippingMethodIDs | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>ShippingMethodID - shipping method id   (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllPKeyProductShippingMethods {
    my $self = shift;
    my ($dbi) = @_;
    my $dbResult = $dbi->execute('SELECT productid, shippingmethodid FROM productshippingmethod', [], ['Integer', 'Integer'], [qw(ProductID ShippingMethodID)]);
    return $dbResult;
}

#========================================================================================
# function     GetAllInfoProductShippingMethods
#----------------------------------------------------------------------------------------
# syntax       $ahProductShippingMethodIDs = $Package->GetAllInfoProductShippingMethods($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all rows of table productshippingmethod.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahProductShippingMethodIDs | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>ShippingMethodID - shipping method id   (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoProductShippingMethods {
    my $self = shift;
    my ($dbi) = @_;
    return $dbi->execute('SELECT productid, shippingmethodid FROM productshippingmethod', [],
        ['Integer', 'Integer'],
        [qw(ProductID ShippingMethodID)]
    );
}

#========================================================================================
# function     GetInfoProductShippingMethod
#----------------------------------------------------------------------------------------
# syntax       $hResult = $Package->GetInfoProductShippingMethod($dbi, $ProductID, $ShippingMethodID);
#----------------------------------------------------------------------------------------
# description  Returns a single row by primary key.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID     | product identifier   (part of primary key) | int
# input        $ShippingMethodID | shipping method id   (part of primary key) | int
# return       $hResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>ShippingMethodID - shipping method id   (part of primary key)- int
#               </ul>| reference to hash
#========================================================================================
sub GetInfoProductShippingMethod {
    my $self = shift;
    my ($dbi, $ProductID, $ShippingMethodID) = @_;
    return $dbi->execute('SELECT productid, shippingmethodid FROM productshippingmethod WHERE productid=? AND shippingmethodid=?',
        [$ProductID, $ShippingMethodID],
        ['Integer', 'Integer'],
        [qw(ProductID ShippingMethodID)]
    )->[0];
}

#========================================================================================
# function     GetPKeyProductShippingMethodsByProductID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyProductShippingMethodsByProductID($dbi, $ProductID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table productshippingmethod by primary key.
#               Uses the index pk_productshippingmethod.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID | product identifier | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>ShippingMethodID - shipping method id   (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyProductShippingMethodsByProductID {
    my $self = shift;
    my ($dbi, $ProductID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'productid = ?';
    unshift @Parameter, $ProductID;

    my $sql = 'SELECT productid, shippingmethodid FROM productshippingmethod WHERE ' . join(' AND ', @WhereClauses);
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'Integer'], [qw(ProductID ShippingMethodID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoProductShippingMethodsByProductID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoProductShippingMethodsByProductID($dbi, $ProductID);
#----------------------------------------------------------------------------------------
# description  Returns full rows of table productshippingmethod by primary key.
#               Uses the index pk_productshippingmethod.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ProductID | product identifier | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>ShippingMethodID - shipping method id   (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoProductShippingMethodsByProductID {
    my $self = shift;
    my ($dbi, $ProductID) = @_;
    my (@WhereClauses, @Parameter);
    unshift @WhereClauses, 'productid = ?';
    unshift @Parameter, $ProductID;

    my $sql = 'SELECT productid, shippingmethodid FROM productshippingmethod WHERE ' . join(' AND ', @WhereClauses);
    return $dbi->execute($sql, \@Parameter,
        ['Integer', 'Integer'],
        [qw(ProductID ShippingMethodID)]
    );
}

#========================================================================================
# function     GetPKeyProductShippingMethodsByShippingMethodID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyProductShippingMethodsByShippingMethodID($dbi, $ShippingMethodID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table productshippingmethod by index i_pshippmethod_smethod.
#               Uses the index i_pshippmethod_smethod.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ShippingMethodID | shipping method id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>ShippingMethodID - shipping method id   (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyProductShippingMethodsByShippingMethodID {
    my $self = shift;
    my ($dbi, $ShippingMethodID) = @_;
    my @Parameter = ($ShippingMethodID);
    my $sql = 'SELECT productid, shippingmethodid FROM productshippingmethod WHERE shippingmethodid = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer', 'Integer'], [qw(ProductID ShippingMethodID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoProductShippingMethodsByShippingMethodID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoProductShippingMethodsByShippingMethodID($dbi, $ShippingMethodID);
#----------------------------------------------------------------------------------------
# description  get full row of table productshippingmethod by index i_pshippmethod_smethod
#               Uses the index i_pshippmethod_smethod.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ShippingMethodID | shipping method id | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>ProductID     - product identifier   (part of primary key)- int
#                   <li>ShippingMethodID - shipping method id   (part of primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoProductShippingMethodsByShippingMethodID {
    my $self = shift;
    my ($dbi, $ShippingMethodID) = @_;
    return $dbi->execute('SELECT productid, shippingmethodid FROM productshippingmethod WHERE shippingmethodid = ?',
        [$ShippingMethodID],
        ['Integer', 'Integer'],
        [qw(ProductID ShippingMethodID)]
    );
}

sub CountFeatureProducts {
    my $self = shift;
    my ($dbi, $ShopID) = @_;
    my $sql = 'SELECT count(productid) FROM product WHERE superproductid IS NULL AND shopid=?';
    my $dbResult = $dbi->execute($sql, [$ShopID], ['Integer']);
    return $dbResult->[0][0];
}

#========================================================================================
# function     GetSmartSearchProductIDs
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $Package->GetSmartSearchProductIDs($dbh, $ProductFolderID, $SearchString);
#----------------------------------------------------------------------------------------
# description  Search for a product alias in a specified product folder of the shop and
#               return a list of product ids.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | object
# input        $ProductFolderID | id of the product folder of the shop | integer
# input        $SearchString | search string | string
# return       $aProductIDs | list of product ids | ref.array.integer
#========================================================================================
sub GetSmartSearchProductIDs {
    my $self = shift;
    my ($dbi, $ProductFolderID, $SearchString) = @_;

    my $sql = 'SELECT objectid FROM object WHERE parentid = ? AND alias LIKE ?';
    my $aRows = $dbi->execute($sql, [$ProductFolderID, $SearchString], ['Integer']);

    return [ map { $_->[0] } splice(@$aRows, SMART_PRODUCT_SEARCH_RESULT_COUNT)];
}

#========================================================================================
# function     GetSubProductIDsBySuperProductID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSubProductIDs = GetSubProductIDsBySuperProductID($SuperProductID, $aVariations);
#----------------------------------------------------------------------------------------
# description  Returns the sub product ids of a super product having the given set of
#               variation values. In general, one can expect that only a single
#               sub product will match the given variation combination. If no such
#               sub product exists, the return value equals a reference to an empty array.
#
#               It should be noted, however, that the following example might be valid:
#               Given a super product with various variations (e.g. color, size, weight).
#               A user might only be interested in all sub products having the color
#               'red'. In this case, the result set is likely to be greater than 1.
#
#               Hence, this subroutine returns a reference to an array of sub product ids.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | object
# input        $SuperProductID | super product identifier | int
# input        $aVariations | array of PreDefAttribute object ids of the attribute values | ref.array.int
# return       $aSubProductIDs | array of SubProductIDs | ref.array.int
#========================================================================================
sub GetSubProductIDsBySuperProductID {
    my $self = shift;
    my ($dbi, $SuperProductID, $aVariations) = @_;

    # SQL statement template (below: two variation values)
    #
    # SELECT p.ProductID
    # FROM Product p
    # INNER JOIN SelectPreDefAttribute spda1
    #       ON spda1.ObjectID = p.ProductID
    # INNER JOIN SelectPreDefAttribute spda2
    #       ON spda2.ObjectID = p.ProductID
    # WHERE p.SuperProductID = ?
    #       AND spda1.PreDefAttributeID = ?
    #       AND spda2.PreDefAttributeID = ?;
    my @InnerJoinClauses;
    my @ConditionalClauses;
    for (my $i = 0; $i < scalar @{ $aVariations }; ++$i) {
        push @InnerJoinClauses, "INNER JOIN selectpredefattribute spda$i ON spda$i.objectid = p.productid";
        push @ConditionalClauses, "spda$i.predefattributeid = ?";
    }

    my $sql = join(' ', 'SELECT p.productid FROM product p', @InnerJoinClauses);
    $sql .= ' WHERE p.superproductid = ? AND ';
    $sql .= join(' AND ', @ConditionalClauses);

    my $aaResult = $dbi->execute($sql, [ $SuperProductID, @{ $aVariations } ], [ 'Integer' ]);
    return [ map { $_->[0] } @{ $aaResult } ];
}

#========================================================================================
# function     GetVariationAttributesBySuperProductID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aaResult = GetVariationAttributesBySuperProductID(
#                   $SuperProductID,
#                   $aVariationAttributeIDs
#               );
#----------------------------------------------------------------------------------------
# description  Returns all sub product ids and all respective predef attribute id's
#               for the given super product id.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | object
# input        $SuperProductID | super product identifier | int
# input        $aVariationAttributeIDs | array of all variation attribute id's of the
#                   product | ref.array.int
# return       $aaResult | array of arrays containing sub product id and variation
#                   attribute value id's | ref.array.array.int
#========================================================================================
sub GetVariationAttributesBySuperProductID {
    my $self = shift;
    my ($dbi, $SuperProductID, $aVariationAttributeIDs) = @_;

    # example for two variation attributes, one already selected

    # select p.`productid`, spda1.`predefattributeid`, spda2.`predefattributeid`
    # from `product` p
    # inner join `selectpredefattribute` spda1
    # on spda1.`objectid` = p.`productid`
    # inner join `selectpredefattribute` spda2
    # on spda2.`objectid` = p.`productid`

    # -- $SuperProductID
    # where p.`superproductid` = 17131

    # -- all variation attribute id's
    # and spda1.`attributeid` = 17056
    # and spda2.`attributeid` = 17062


    my @SelectClauses;
    my @InnerJoinClauses;
    my @ConditionalClauses;

    foreach my $i (1 .. scalar @$aVariationAttributeIDs) {
        push @SelectClauses, "spda$i.predefattributeid";
        push @InnerJoinClauses, "INNER JOIN selectpredefattribute spda$i ON spda$i.objectid = p.productid";
        push @ConditionalClauses, "spda$i.attributeid = ?";
    }

    my $sql = join(',', 'SELECT p.productid', @SelectClauses);
    $sql .= join(' ', ' FROM product p', @InnerJoinClauses);
    $sql .= ' WHERE p.superproductid = ?';
    $sql .= ' AND ' if scalar @ConditionalClauses;
    $sql .= join(' AND ', @ConditionalClauses);

    my $aaResult = $dbi->execute($sql, [$SuperProductID, @$aVariationAttributeIDs]);
    return $aaResult;
}

#========================================================================================
# function     GetAllProductIDsAliasOrderedByShopID
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $Package->GetAllProductIDsAliasOrderedByShopID($dbh, $ShopID);
#----------------------------------------------------------------------------------------
# description  Returns an iterator over the IDs of all shop products ordered by their alias
#               with the restriction that super products have to be in front of their subs.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | object
# input        $ShopID | shop id of the products | integer
# return       $Iterator | object iterator | DE_EPAGES::Core::API::Iterator
#========================================================================================
sub GetAllProductIDsAliasOrderedByShopID {
    my $self = shift;
    my ($dbi, $ShopID) = @_;

    my $Iterator = $dbi->iterator(<<END_SQL , [ $ShopID, $ShopID ]);
SELECT productid FROM
(
    (SELECT sub.productid, subo.alias, po.alias AS orderBase, 20 AS orderPos
     FROM product p
     INNER JOIN object po ON po.objectid = p.productid
     INNER JOIN product sub ON sub.superproductid = p.productid
     INNER JOIN object subo ON subo.objectid = sub.productid
     WHERE p.superproductid IS NULL
      AND p.shopid = ?
      AND p.isappointment = 0
    )
    UNION
    (SELECT p.productid, po.alias, po.alias AS orderBase, 10 AS orderPos
     FROM product p
     INNER JOIN object po ON po.objectid = p.productid
     WHERE p.superproductid IS NULL
      AND p.shopid = ?
      AND p.isappointment = 0
    )
) p
ORDER BY orderBase, orderPos, alias
END_SQL

    return $Iterator;
}

#========================================================================================
# function     GetSubProductsOrderedByPrice
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $Package->GetSubProductsOrderedByPrice($dbh, $SuperProductID, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  Returns an iterator of product ids for all sub produsts ordered by price.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | object
# input        $SuperProductID | id of the super product | integer
# input        $CurrencyID | currency id, e.g. 'USD' | string
# return       $Iterator | iterator of product ids | DE_EPAGES::Core::API::Iterator
#========================================================================================
sub GetSubProductsOrderedByPrice {
    my $self = shift;
    my ($dbi, $SuperProductID) = @_;

    die 'not implemented for this database';
}

#========================================================================================
# function     GetUpdatedProducts
#----------------------------------------------------------------------------------------
# syntax       my @ProductIDs = GetUpdatedProducts($dbi, $AttributeSet, $UpdatedAfter);
#----------------------------------------------------------------------------------------
# description  Returns an array of arrays of productID and lastChangeDate.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | object
# input        $ShopId | object id of the shop | int
# input        $AttributeSet | name of the LastUpdateProfile | string
# input        $UpdatedAfter | time after which the update happened | datetime
# return       array of arrays of productID and lastChangeDate | array ref
#========================================================================================
sub GetUpdatedProducts {
    my $self = shift;
    my ($dbi, $ShopId, $AttributeSet, $UpdatedAfter) = @_;

    my $Query = <<'END_SQL';
        SELECT DISTINCT COALESCE(p.superproductid,p.productid) AS masterproductid, MAX(l.lastupdatedon) AS lastchange
        FROM product p
        JOIN lastupdate l ON p.productid = l.objectid
        JOIN lastupdateprofile lup ON l.profileid = lup.profileid
        JOIN object o ON lup.profileid = o.objectid
        WHERE p.shopid = ?
        AND o.alias = ?
        AND l.lastupdatedon > ?
        GROUP BY masterproductid;
END_SQL

    my $aResult = $dbi->execute($Query, [ $ShopId, $AttributeSet, $UpdatedAfter ], ['Integer']);

    return $aResult;
}

#========================================================================================
# function     GetUpdatedProducts
#----------------------------------------------------------------------------------------
# syntax       my @ProductIDs = GetUpdatedProducts($dbi, $AttributeSet, $UpdatedAfter);
#----------------------------------------------------------------------------------------
# description  Returns an array of arrays of productID and number of watchers.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | object
# input        $ShopId | object id of the shop | int
# return       array of arrays of productID and number of watchers | array ref
#========================================================================================
sub GetWatchedProducts {
    my $self = shift;
    my ($dbi, $ShopId) = @_;

    my $Query = <<'END_SQL';
        SELECT p.productid, count(p.productid)
        FROM productavailabilityemail p
        JOIN object o ON o.objectid = p.productid
        WHERE o.siteid= ?
        GROUP BY productid;
END_SQL

    my $aResult = $dbi->execute($Query, [ $ShopId ], ['Integer']);

    return $aResult;
}

#========================================================================================
# function     UpdateStockLevel
#----------------------------------------------------------------------------------------
# syntax       UpdateStockLevel($dbi, $ObjectID, $Amount);
#----------------------------------------------------------------------------------------
# description  Update the stocklevel of a product with a relative value. In difference
#               to the classic set method, the value will be added (or substracted, if
#               the amount is negative)
#----------------------------------------------------------------------------------------
# input        $dbi      | database handle             | object
# input        $ObjectID | object id of the product    | int
# input        $Amount   | amount of stock to increase | int
#========================================================================================
sub UpdateStockLevel {
    my $self = shift;
    my ($dbi, $ObjectID, $Amount) = @_;

    my $StockLevel;

    $dbi->transaction( sub {
        $dbi->do('UPDATE product SET stocklevel = stocklevel + ? WHERE productid = ?',
            [ $Amount, $ObjectID ], ['Integer']);

        $StockLevel = $dbi->execute('SELECT stocklevel FROM product WHERE productid = ?', [ $ObjectID ])->[0]->[0];
    } );

    return $StockLevel;
}

1;
