use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Product::API::Cartridge;
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         InitializeProductNotifications
# state            public
#----------------------------------------------------------------------------------------
# syntax           InitializeProductNotifications();
#----------------------------------------------------------------------------------------
# description      Activates the product notification product for all Unity shops
#                   that have products out of stock and/or low on stock.
#========================================================================================
sub InitializeProductNotifications {

    print "Reading all Unity shops\n";
    my $aShopIDs = GetUnityShopIDs();

    my $Cartridge = DE_EPAGES::Product::API::Cartridge->new;
    foreach my $ShopID (@$aShopIDs) {
        my $Shop = LoadObject($ShopID);
        # import notifications
        ImportObjects($Cartridge->xmlDirectory().'/ShopNotifications.xml', $Shop->pathString);
        my $aProducts = $Shop->get('Products');
        my $ProductOutOfStockNotification = $Shop->folder('ShopNotifications')->child('ProductOutOfStock');
        my $ProductLowOnStockNotification = $Shop->folder('ShopNotifications')->child('ProductLowOnStock');
        foreach my $Product (@$aProducts) {
            _checkStockLevelNotifications($Product, $ProductOutOfStockNotification, $ProductLowOnStockNotification);
            if ($Product->countSubProducts > 0) {
                foreach my $SubProduct (@{$Product->subProducts}) {
                    _checkStockLevelNotifications($SubProduct, $ProductOutOfStockNotification, $ProductLowOnStockNotification);
                }
            }
        }
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub _checkStockLevelNotifications {
    my ($Product, $ProductOutOfStockNotification, $ProductLowOnStockNotification) = @_;

    my $hProductInfo = $Product->get(['StockLevel', 'StockLevelAlert']);
    my $StockLevel = $hProductInfo->{'StockLevel'};
    my $MinStockLevel = $hProductInfo->{'StockLevelAlert'};
    # product out of stock
    if (defined $StockLevel && $StockLevel <= 0) {
        $ProductOutOfStockNotification->addObject($Product);
    }
    # product low on stock
    if (defined $MinStockLevel && defined $StockLevel && $StockLevel < $MinStockLevel) {
        $ProductLowOnStockNotification->addObject($Product);
    }
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { InitializeProductNotifications(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Activates the product notification product for all Unity shops
    that have products out of stock and/or low on stock.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
