package DE_EPAGES::Product::t::ProductStockLevel;
use base DE_EPAGES::Database::API::Test;

use strict;

use threads;

use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use DE_EPAGES::DataCache::API::Cache qw (UpdateCache);
use Test::More;
use Data::Dumper;

use constant ROOTPATH       => 'Test';
use constant SHOP_ALIAS     => 'TestShop';
use constant PRODUCT_ALIAS  => 'TestProduct';
use constant PATH           => '/' . ROOTPATH . '/' . SHOP_ALIAS . '/Products/' . PRODUCT_ALIAS;

sub test {
    my $self = shift;

    $self->test_synchronous_stocklevel_decrease();
    $self->test_set_negative_stocklevel();
    $self->test_negative_stocklevel_decrease();
    done_testing();

    return;
}

#========================================================================================
# function     test_synchronous_stocklevel_decrease
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test_synchronous_stocklevel_decrease()
#----------------------------------------------------------------------------------------
# description  Test case to verify, that the stocklevel is correct after two nearly
#               synchonous reductions of the stocklevel for the same product.
#========================================================================================
sub test_synchronous_stocklevel_decrease {
    my $self = shift;

    my $Store = GetTestSite();
    my $Product = LoadObjectByPath(PATH);

    # Set stocklevel to 100
    LoadObjectByPath(PATH)->set({'StockLevel'=> 100 });

    # Reduce stocklevel
    my $ReduceStock1 = $self->_runThreadOnStore(
        $Store,
        sub {
            $Product->substractFromStock(25);
        }
    );

    # Reduce stocklevel
    my $ReduceStock2 = $self->_runThreadOnStore(
        $Store,
        sub {
           $Product->substractFromStock(50);
        }
    );

    my $ResultThread1 = $ReduceStock1->join();
    my $ResultThread2 = $ReduceStock2->join();

    # Remove cache for product object
    $self->_cleanCache($Product->id);

    # Check if threads were successsful.
    is( $ResultThread1, 'success', "Result thread 1" );
    is( $ResultThread2, 'success', "Result thread 2" );

    SynchronizeCache();

    my $ObjectID = $Product->id;
    UpdateCache($self->dbi->cacheKey, 'Object', $ObjectID);
    UpdateCache($self->dbi->cacheKey, 'InfoObject', $ObjectID);
    UpdateCache($self->dbi->cacheKey, 'InfoContentObject', $ObjectID);
    UpdateCache($self->dbi->cacheKey, 'ObjectByParentID', $ObjectID);

    my $hExpectedValue = 25;
    is(
        LoadObjectByPath(PATH)->get('StockLevel'),
        $hExpectedValue,
        "Check stocklevel after two changes"
    );

    return;
}

#========================================================================================
# function     test_set_negative_stocklevel
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test_set_negative_stocklevel()
#----------------------------------------------------------------------------------------
# description  Test case to verify, that a negative stocklevel can be set successfully.
#========================================================================================
sub test_set_negative_stocklevel {
    my $self = shift;

    my $Product = LoadObjectByPath(PATH);

    # Set stocklevel to 100
    $Product->set({'StockLevel'=> -100 });

    $self->_cleanCache($Product->id);

    my $hExpectedValue = -100;

    is(
        LoadObjectByPath(PATH)->get('StockLevel'),
        $hExpectedValue,
        "Check negative stocklevel");
}

#========================================================================================
# function     test_negative_stocklevel_decrease
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test_negative_stocklevel_decrease()
#----------------------------------------------------------------------------------------
# description  Test case to verify, that a negative stocklevel decrease will add stocklevel.
#========================================================================================
sub test_negative_stocklevel_decrease {
    my $self = shift;
    my $Product = LoadObjectByPath(PATH);

    $Product->set({'StockLevel'=> 100 });

    $Product->substractFromStock(-100);
    my $hExpectedValue = 200;

    $self->_cleanCache($Product->id);

    SynchronizeCache();

    my $ObjectID = $Product->id;
    UpdateCache($self->dbi->cacheKey, 'Object', $ObjectID);
    UpdateCache($self->dbi->cacheKey, 'InfoObject', $ObjectID);
    UpdateCache($self->dbi->cacheKey, 'InfoContentObject', $ObjectID);
    UpdateCache($self->dbi->cacheKey, 'ObjectByParentID', $ObjectID);

    is(
        LoadObjectByPath(PATH)->get('StockLevel'),
        $hExpectedValue,
        "Check negative decrease stocklevel");
}

#========================================================================================
# function     init
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->init()
#----------------------------------------------------------------------------------------
# description  Init method to create a shop with a product.
#========================================================================================
sub init {
    my $self = shift;

    cleanup();

    my $Folder = LoadRootObject()->insertChild('Object', { Alias => ROOTPATH });
    my $Shop = $Folder->insertChild('Shop', { Alias => SHOP_ALIAS });

    $Shop->featureMaxValue('Products', 1);
    my $ProductFolder = $Shop->get('ProductFolder');
    $ProductFolder->insertChild('Product', { Alias => PRODUCT_ALIAS });

    return 1;
}

#========================================================================================
# function     cleanup
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->cleanup()
#----------------------------------------------------------------------------------------
# description  Cleanup method to delete shop, if it exist.
#========================================================================================
sub cleanup {
    my $self = shift;

    LoadObjectByPath(ROOTPATH)->delete if ExistsObjectByPath(ROOTPATH);

    return 1;
}

#========================================================================================
# function     _runThreadOnStore
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_runThreadOnStore('Store', sub {...})
#----------------------------------------------------------------------------------------
# description  Helper method to create and run a thread on a given store.
#----------------------------------------------------------------------------------------
# input        $Store | The store the thread will run on | object
# input        $rSub | The code that will be run in the thread | ref.sub
# return       $ReturnValue | Created thread returns error code or 'success' otherwiese | string
#========================================================================================
sub _runThreadOnStore {
    my $self = shift;
    my ( $Store, $rSub ) = @_;

    return threads->create(
        sub {
            my $ReturnValue;
            RunOnStore(
                Store => $Store,
                Sub   => sub {
                    eval { $rSub->(); };
                    if ( ExistsError() ) {
                        print "\n\n\nError found\n\n\n";
                        my $Error = GetError();
                        LogError($Error);
                        $ReturnValue = $Error->code;
                    }
                    else {
                        $ReturnValue = 'success';
                    }
                    SynchronizeCache();
                }
            );
            return $ReturnValue;
        }
    );
}

#========================================================================================
# function     _cleanCache
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_cleanCache($ObjectID)
#----------------------------------------------------------------------------------------
# description  Helper method to delete the cache of the product.
#----------------------------------------------------------------------------------------
# input        ObjectID | The ID of the object, which cache should be deleted. | int
#========================================================================================
sub _cleanCache() {
    my $self = shift;

    my $ObjectID = @_;

    UpdateCache($self->dbi->cacheKey, 'Object', $ObjectID);
    UpdateCache($self->dbi->cacheKey, 'InfoObject', $ObjectID);
    UpdateCache($self->dbi->cacheKey, 'InfoContentObject', $ObjectID);
    UpdateCache($self->dbi->cacheKey, 'ObjectByParentID', $ObjectID);

    return 1;
}

__PACKAGE__->runScript();
1;
