#========================================================================================
# package      DE_EPAGES::SIA::API::Request::Signature
# state        public
# description  Creating signature needed for communication with payment provider.
#========================================================================================
package DE_EPAGES::SIA::API::Request::Signature;

use Crypt::OpenSSL::RSA;
use DE_EPAGES::SIA::API::SIAConfig;
use DE_EPAGES::SIA::API::Tools  qw (GetToolsInstance);
use Digest::SHA                 qw (sha256_hex);
use strict;

use Moose;

has SiaConfig => is => 'ro', isa => 'DE_EPAGES::SIA::API::SIAConfig', required => 1;

# All SIA Attributes
has IDVERSION       => is => 'ro', isa => 'Str', default   => '';
has IDTERMINAL      => is => 'ro', isa => 'Str', default   => '';
has IDENTIDAD       => is => 'ro', isa => 'Str', default   => '';
has IDCOMERCIO      => is => 'ro', isa => 'Str', default   => '';
has IDSERVICIO      => is => 'ro', isa => 'Str', default   => '';
has IDUSUARIO       => is => 'ro', isa => 'Str', default   => '';
has IDTM            => is => 'ro', isa => 'Str', default   => '';
has OPCTOKEN        => is => 'ro', isa => 'Str', default   => '';
has IDPEDIDO        => is => 'ro', isa => 'Str', default   => '';
has NOMTITULAR      => is => 'ro', isa => 'Str', default   => '';
has CODPAIS         => is => 'ro', isa => 'Str', default   => '';
has PAN             => is => 'ro', isa => 'Str', default   => '';
has FECCAD          => is => 'ro', isa => 'Str', default   => '';
has CVC             => is => 'ro', isa => 'Str', default   => '';
has IMPTRANS        => is => 'ro', isa => 'Str', default   => '';
has CODMON          => is => 'ro', isa => 'Str', default   => '';
has TIMETRANS       => is => 'ro', isa => 'Str', default   => '';
has IDIOMA          => is => 'ro', isa => 'Str', default   => '';
has IPORIGEN        => is => 'ro', isa => 'Str', default   => '';
has TOKEN           => is => 'ro', isa => 'Str', default   => '';
has DESTOKEN        => is => 'ro', isa => 'Str', default   => '';
has fechProOri      => is => 'ro', isa => 'Str', default   => '';
has numsiaori       => is => 'ro', isa => 'Str', default   => '';
has EMAILUSUARIO    => is => 'ro', isa => 'Str', default   => '';
has sesionComercio  => is => 'ro', isa => 'Str', default   => '';

#========================================================================================
# function     getSha256HexString
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getSha256HexString();
#----------------------------------------------------------------------------------------
# description  Provides signature hash needed for firmToken.
#----------------------------------------------------------------------------------------
# return       $sha256HexString | hash | string
#========================================================================================
sub getSha256HexString {
    my $self = shift;

    my $Fields = join( '',
        $self->IDVERSION, $self->IDTERMINAL, $self->IDENTIDAD, $self->IDCOMERCIO,   $self->IDSERVICIO,
        $self->IDUSUARIO, $self->IDTM,       $self->OPCTOKEN,  $self->IDPEDIDO,     $self->NOMTITULAR,
        $self->CODPAIS,   $self->PAN,        $self->FECCAD,    $self->CVC,          $self->IMPTRANS,
        $self->CODMON,    $self->TIMETRANS,  $self->IDIOMA,    $self->IPORIGEN,     $self->TOKEN,
        $self->DESTOKEN,  $self->fechProOri, $self->numsiaori, $self->EMAILUSUARIO, $self->sesionComercio );

    return sha256_hex($Fields);
}

#TODO Check with test key
#========================================================================================
# function     getFirmToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getFirmToken();
#----------------------------------------------------------------------------------------
# description  Generates the signature that has to be used in the firmToken field. SIA
#               demanded that we use the public key to create the signature. SIA ensured
#               that the public key is only known by ePages and SIA.
#----------------------------------------------------------------------------------------
# return       $firmToken | firmToken signature | string
#========================================================================================
sub getFirmToken {
    my $self = shift;

    my $Sha256             = $self->getSha256HexString();
    my $RSA                = Crypt::OpenSSL::RSA->new_public_key( $self->SiaConfig->getCertificate() );

    # use pkcs1 padding because sia expects this
    $RSA->use_pkcs1_padding();

    my $EncryptedSha256    = $RSA->encrypt($Sha256);
    my $EncryptedHexSha256 = GetToolsInstance()->stringToHexString($EncryptedSha256);

    return $EncryptedHexSha256;
}

no Moose;
__PACKAGE__->meta->make_immutable;

1;
