#==================================================================================================
# package      DE_EPAGES::SIA::t::unit::MockupObjects
# state        public
#--------------------------------------------------------------------------------------------------
# description  Mock objects used by unit tests
#==================================================================================================
package DE_EPAGES::SIA::t::unit::MockupObjects;
use base Exporter;

use Crypt::OpenSSL::RSA;
use DateTime;
use DE_EPAGES::SIA::API::Object::RegistrationTransaction;
use DE_EPAGES::SIA::API::SIAConfig;
use DE_EPAGES::Shop::API::Url qw ( BuildDefaultShopUrl );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::MockObject::Extends;


use strict;

## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
## no critic (ProhibitImplicitNewlines)

our @EXPORT_OK = qw (
    CreateMockedLineItemPayment
    CreateMockedPaymentMethod
    CreateRegistrationTransaction
    CreateMockedRegistrationTransaction
    CreateMerchantRegistrationDataHash
    CreateMockedSIAConfig
    CreateMockedServlet
    TEST_MERCHANT_ID
    TEST_MERCHANT_FUC
    TEST_TERMINAL_NUMBER
    TEST_CURRENCY_ID
    TEST_TRANS_ID
    TEST_SHOP_NAME
    TEST_REMOTE_ADDRESS
);

use constant TEST_MERCHANT_ID       => '055720051';
use constant TEST_MERCHANT_FUC      => '004123123123123';
use constant TEST_TERMINAL_NUMBER   => '00000001';
use constant TEST_SHOP_NAME         => 'UnitTestShop';
use constant TEST_SHOP_PATH         => '/Shops/'.TEST_SHOP_NAME;
use constant TEST_SHOP_URL          => 'http://unit-test.epages.de/epages/'.TEST_SHOP_NAME.'.sf';
use constant TEST_SHOP_EMAIL        => 'unit-test@epages.de';
use constant TEST_ORDER_CURRENCY    => 'EUR';
use constant TEST_CURRENCY_ID       => '978';
use constant TEST_ORDER_TOTAL       => '80';
use constant TEST_TRANS_ID          => '566004DBD5809AA32A41';
use constant TEST_TRANS_ID2         => '567890';
use constant TEST_REMOTE_ADDRESS    => '192.168.0.1';

#==================================================================================================
# function     CreateMockedLineItemPayment
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       CreateMockedLineItemPayment();
#--------------------------------------------------------------------------------------------------
# description  Creates a Mocked System and returns the LineItemPayment
#--------------------------------------------------------------------------------------------------
# return       LineItemPayment | mocked line item payment object | object
#==================================================================================================
sub CreateMockedLineItemPayment {
    my $MockedDALSystem             = CreateNewMockedSystem();
    my $MockedPaymentMethodSIA      = ExtendMockPaymentMethodSIA($MockedDALSystem);
    my $MockedLineItemPaymentSIA    = ExtendMockLineItemPaymentSIA($MockedDALSystem,$MockedPaymentMethodSIA);

    return $MockedLineItemPaymentSIA;
}

#==================================================================================================
# function     CreateMockedPaymentMethod
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       CreateMockedPaymentMethod();
#--------------------------------------------------------------------------------------------------
# description  Creates a Mocked System and returns the Payment
#--------------------------------------------------------------------------------------------------
# return       LineItemPayment | mocked payment object | object
#==================================================================================================
sub CreateMockedPaymentMethod {
    my $MockedDALSystem             = CreateNewMockedSystem();
    my $MockedPaymentMethodSIA      = ExtendMockPaymentMethodSIA($MockedDALSystem);
    ExtendMockLineItemPaymentSIA($MockedDALSystem,$MockedPaymentMethodSIA);

    return $MockedPaymentMethodSIA;
}

sub CreateNewMockedSystem {
    my $MockedSystem            = NewMockedSystem();
    my $MockedShop              = NewMockedShop();
    my $MockedPaymentMethod     = NewMockedPaymentMethodSIA();

    # Add Shop as Child from System
    AddShopToSystem($MockedSystem,$MockedShop);
    # Add Payment as Child from Shop
    AddPaymentToShop($MockedSystem,$MockedPaymentMethod);

    # Now recursivly create Order
    my $MockedOrder             = NewMockedOrder($MockedSystem);

    # Add Order to Shop
    AddOrderToShop($MockedSystem,$MockedOrder);

    return $MockedSystem;
}

sub NewMockedSystem {
    my $MockedSystem =
        DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Object::API::Object::Site',
            {
                'TimeZone'          => 'Europe/Berlin',
                'Child'             => {
                    'SIATestConfig'     => NewMockedSIATestConfig(),
                    'SIAConfig'         => NewMockedSIAConfig(),
                    'Shops'             => DE_EPAGES::Test::Mock::EpagesObject->new(
                        'DE_EPAGES::Object::API::Object::Object',
                        {
                            'Alias'     => 'Shops',
                            'Path'      => '/Shops',
                            'Child'     => undef,
                        }
                    )
                }
            }
        );

    $MockedSystem->set({'Site' => $MockedSystem});

    return $MockedSystem;
}

sub NewMockedSIAConfig {

    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::SIA::API::SIAConfig',
        {
            'IMCompany'                     => '0001',
            'CompanyCode'                   => 'test',
            'HoldingCode'                   => 'test',
            'AcquiringFinancialEntity'      => 'test',
            'MerchantFUCPrefix'             => '004',
            'EntityID'                      => 'ACEN',
            'SaleWebserviceEndpointURL'     => 'todo',
            'RefundWebserviceEndpointURL'   => 'todo',
            'CancelWebserviceEndpointURL'   => 'todo',
            'UseSSL'                        => '1',
            'SIACertificate'                => '',
        }
    );
}

sub NewMockedSIATestConfig {

    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::SIA::API::SIAConfig',
        {
            'IMCompany'                     => '0001',
            'CompanyCode'                   => '0001',
            'HoldingCode'                   => '0001',
            'AcquiringFinancialEntity'      => 'ADYN',
            'MerchantFUCPrefix'             => '004',
            'EntityID'                      => 'ACEN',
            'SaleWebserviceEndpointURL'     => 'http://216.177.213.245:6990/servlet/ServletDirector',
            'RefundWebserviceEndpointURL'   => 'http://216.177.213.245:6982/Payment/refund',
            'CancelWebserviceEndpointURL'   => 'http://216.177.213.245:6982/Payment/cancel',
            'UseSSL'                        => '0',
            'SIACertificate' => '-----BEGIN PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAjlZ002IVEtaOyDjmHWMb
q+kq3RoKmPjhcA7cbHpmNoHYzOOiSK7wH727OftY3l9AwPbYKLc956dcYueaEagA
XbTWRZl5WjyTUgSiBSvwvqYVOl8XX2yDlMooqMHd6mp3x6NnwcRXDlza+yNVBDU3
v03WW6y0gPtwiwtImWz2Wokkqc4nQrSsgu2QNrri9ZocaycQtqQpmxa8W0pjufwH
DUIYkz/UfPSbfE42HzH2z1Hp75IkPKXWO8Axun234EudtoArVo5wT+bpbbjNRXWg
h/DQtgiP/p/FBDnDTspJiGou3IblCIBKtSbt6yix1dfVId/5CJRWy5Bxv89+d/vt
uwIDAQAB
-----END PUBLIC KEY-----',
        }
    );
}

sub NewMockedShop {
    my ($MockedSystem) = @_;

    my $MockedShop =
        DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop',
            {
                'ID'                    => '12345',
                'Alias'                 => TEST_SHOP_NAME,
                'LanguageID'            => 1,
                'CurrencyID'            => 'EUR',
                'instanceOf'            => 'Shop',
                'Path'                  => TEST_SHOP_PATH,
                'PathString'            => TEST_SHOP_PATH,
                'WebServerScriptUrl'    => TEST_SHOP_URL,
                'WebServerScriptUrlSSL' => TEST_SHOP_URL,
                'Address'               => NewMockedShopAddress($MockedSystem),
                'Currencies'            => [
                                                { 'CurrencyID' => 'EUR', 'NumCode' => '978', 'MinorUnit' => 2 },
                                                { 'CurrencyID' => 'GBP', 'NumCode' => '826', 'MinorUnit' => 2 },
                ],

            }
        );

    $MockedShop->mock('getShopUrl',sub {
        my $self = shift;
        my ($hParams, $hOptions, $Shop) = @_;

        return BuildDefaultShopUrl($self, $hParams, $hOptions, $Shop);
    }
    );

    return $MockedShop;
}

sub NewMockedShopAddress {

    return
        DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Address::API::Object::Address',
            {
                'Alias'                 => 'ShopAddress',
                'EMail'                 => TEST_SHOP_EMAIL,
                'LastName'              => 'TestLastName',
                'FirstName'             => 'TestFirstName',
                'FullName'              => 'TestFirstName TestLastName',
                'City'                  => 'TestCity',
                'CountryID'             => '276',
            }
        );
}

sub NewMockedPaymentMethodSIA {

    my $PaymentMethodSia = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::SIA::API::Object::PaymentMethodSIA',
        {
            'Alias'                             => 'SIA',
            'ID'                                => 'SIAID',
            'IsInTestMode'                      => 1,
            'CurrentRegistrationTransaction'    => 'TestRegistrationTransaction1',
            'Child'                             => undef,
        }
    );

    $PaymentMethodSia->set('Child' => {
        'RegistrationTransactions' => DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Object::API::Object::Object',
            {
                'Alias'     => 'RegistrationTransactions',
                    'Child'     => {
                        'TestRegistrationTransaction1' =>
                            CreateMockedRegistrationTransaction(CreateRegistrationTransaction(
                                {'Alias' => 'TestRegistrationTransaction1','Payment' => $PaymentMethodSia})),
                        'TestRegistrationTransaction2' =>
                            CreateMockedRegistrationTransaction(CreateRegistrationTransaction(
                                {'Alias' => 'TestRegistrationTransaction2','Payment' => $PaymentMethodSia})),
                    }
            }
        )
    });

    return $PaymentMethodSia;
}

sub NewMockedOrder {
    my ($MockedSystem) = @_;

    my $MockedShop = $MockedSystem->get('Shop');

    my $MockedOrder =
        DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Order::API::Object::CustomerOrder',
            {
                'Alias'             => 'Order',
                'instanceOf'        => 'Order',
                'ID'                => '15917',
                'GUID'              => '56261623-D358-D467-C2AC-D5809AA3DE8F',
                'Shop'              => $MockedShop,
                'PaidOn'            => undef,
                'CancelledOn'       => undef,
                'LineItemContainer' => undef,
                'BillingAddress'    => NewMockedBillingAddress($MockedShop),
                'Site'              => $MockedShop,
            }
        );

    $MockedSystem->set({'OrderSIA' => $MockedOrder});
    $MockedOrder->set({'LineItemContainer' => NewMockedLineItemContainer($MockedSystem)});

    return $MockedOrder;
}

sub NewMockedLineItemContainer {
    my ($MockedSystem) = @_;

    my $MockedShop              = $MockedSystem->get('Shop');
    my $MockedLineItemPayment   = NewMockedLineItemPayment($MockedSystem);

    my $MockedLineItemContainer =
        DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Order::API::Object::LineItemContainer',
            {
                'Alias'             => 'LineItemContainer',
                'ID'                => '17890',
                'Shop'              => $MockedShop,
                'CurrencyID'        => TEST_ORDER_CURRENCY,
                'GrandTotal'        => TEST_ORDER_TOTAL,
                'Payment'           => $MockedLineItemPayment,
                'BillingCountryID'  => '276',
                'BasketOrOrder'     => $MockedSystem->get('OrderSIA'),
                'Parent'            => $MockedSystem->get('OrderSIA'),
                'LanguageID'        => '1',
            }
        );

    $MockedSystem->set({'LineItemContainerSIA' => $MockedLineItemContainer});
    $MockedLineItemPayment->set({'LineItemContainer' => $MockedLineItemContainer});

    return $MockedLineItemContainer;
}

sub NewMockedBillingAddress {
    my ($MockedShop) = @_;

    my $MockedBillingAddress =
        DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Address::API::Object::Address',
            {
                'Alias'             => 'BillingAddress',
                'Site'              => $MockedShop,
                'EMail'             => 'test@epages.de',
                'FullName'          => 'Test User',
            }
        );

    return $MockedBillingAddress;
}

sub NewMockedLineItemPayment {
    my ($MockedSystem) = @_;

    my $MockedShop = $MockedSystem->get('Shop');

    my $MockedLineItemPayment =
        DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::SIA::API::Object::LineItemPaymentSIA',
            {
                'Alias'             => 'LineItemPaymentSIA',
                'Shop'              => $MockedShop,
                'GUID'              => '11111111-1111-1111-1111-111111111111',
                'TransID'           => TEST_TRANS_ID,
                'TransID2'          => TEST_TRANS_ID2,
                'CurrencyID'        => TEST_ORDER_CURRENCY,
                'Amount'            => TEST_ORDER_TOTAL,
                'UserIP'            => TEST_REMOTE_ADDRESS,
                'TransStatus'       => 'STATUS_ACCEPTED',
                'TransTime'         => '2015-11-26T07:24:31',
                'TransTimeMod'      => undef,
                'PaymentMethod'     => $MockedSystem->get('PaymentMethodSIA'),
                'SiaCallNumber'     => undef,
                'Field1'            => '2541',
                'Field2'            => '150629102556',
                'Field3'            => '',
                'CancelledOn'       => '2015-12-08T07:24:31',
                'Parent'            => $MockedSystem->get('OrderSIA'),
            }
        );

    $MockedSystem->set({'LineItemPaymentSIA' => $MockedLineItemPayment});

    return $MockedLineItemPayment;
}

sub AddOrderToShop {
    my ($MockedSystem,$MockedOrder) = @_;
    my $MockedShop = $MockedSystem->get('Shop');


    $MockedShop->set({
        'Site'      => $MockedShop,
        'Orders'    => [ $MockedOrder ],
    });
}

sub AddPaymentToShop {
    my ($MockedSystem,$MockedPayment) = @_;

    my $MockedShop = $MockedSystem->get('Shop');

    $MockedShop->set({
        'Child'                 => {
            'PaymentMethods'    => DE_EPAGES::Test::Mock::EpagesObject->new(
                'DE_EPAGES::Object::API::Object::Object',
                {
                    'Alias'     => 'PaymentMethods',
                    'Path'      => '/Shops/TestShop/PaymentMethods',
                    'Child'     => {
                        'SIA' => $MockedPayment,
                    }
                }
            )
        }
    });

    $MockedSystem->set({'PaymentMethodSIA' => $MockedPayment});
}

sub AddShopToSystem {
    my ($MockedSystem,$MockedShop) = @_;

    $MockedSystem->child('Shops')->set({'UnitTestShop' => $MockedShop});

    $MockedSystem->set({
        'Shop'      => $MockedShop,
    });
}

#==================================================================================================
# function     CreateMockedRegistrationTransaction
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       $MockedRegistrationTransaction = CreateMockedRegistrationTransaction();
#--------------------------------------------------------------------------------------------------
# description  Creates a mocked RegistrationTransaction object
#--------------------------------------------------------------------------------------------------
# input        $MerchantRegistrationData | merchant registration data | object
# return       $MockedRegistrationTransaction | mocked registration transaction object | object
#==================================================================================================
sub CreateMockedRegistrationTransaction {
    my ($MerchantRegistrationData) = @_;

    my $ExtendedMerchantRegistrationData = Test::MockObject::Extends->new($MerchantRegistrationData);

    $ExtendedMerchantRegistrationData->mock(
        'tleHash',
        sub {
            my $self = shift;
            return $ExtendedMerchantRegistrationData->convertToHash();
        }
    );
}

#==================================================================================================
# function     CreateMockedSIAConfig
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       CreateMockedSIAConfig();
#--------------------------------------------------------------------------------------------------
# description  Creates a SIAConfig mocked object
#--------------------------------------------------------------------------------------------------
# return       $MockedSIAConfig | mocked sia config object | object
#==================================================================================================
sub CreateMockedSIAConfig {
    my $RSA = Crypt::OpenSSL::RSA->generate_key(1024);
    $RSA->use_pkcs1_padding;

    my $SIAConfig       = DE_EPAGES::SIA::API::SIAConfig->new( 'IsInTestMode' => 1 );
    my $MockedSIAConfig = Test::MockObject::Extends->new($SIAConfig);

    $MockedSIAConfig->mock(
        'getCertificate',
        sub {
            my $self = shift;
            return $RSA->get_public_key_string();
        }
    );

    return ( $MockedSIAConfig, $RSA );
}

#==================================================================================================
# function     CreateMerchantRegistrationDataHash
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       CreateMerchantRegistrationDataHash();
#--------------------------------------------------------------------------------------------------
# description  Creates merchant registration data hash.
#--------------------------------------------------------------------------------------------------
# input        $Alias | alias inserted in result hash | string
# input        $Payment | payment inserted in result hash | object
# return       $hRegistrationData | registration data | ref.hash
#==================================================================================================
sub CreateMerchantRegistrationDataHash {
    my ($Alias,$Payment)   = @_;

    return {
        Alias                       => $Alias,
        Payment                     => $Payment,
        IMCompany                   => '0001',
        WorkOrderID                 => '00000000008',
        RequestCode                 => 'IMN',
        RequestSubCode              => '01',
        SiaEntityCode               => 'SCNP',
        SiaCompanyCode              => 'SCNP',
        HoldingCode                 => 'SCNP',
        MerchantFUC                 => TEST_MERCHANT_FUC,
        MerchantID                  => TEST_MERCHANT_ID,
        MerchantName                => 'NOMBRE DEMO 3',
        City                        => 'POBLACION DEMO 3',
        Country                     => '724',
        ApplicationPaymentNetwork   => 'Telefonica',
        WorkOrderDate               => '20150820',
        MerchantTaxId               => '12345678Z',
        MerchantLanguage            => 'ES',
        Currency                    => 'EUR',
        ApplicationPaymentNetwork   => '110001',
        PrivateCardFlag             => '0',
        DccFlag                     => '0',
        MarkupDcc                   => '000000',
        TypeOfCurrencyDCC           => '00',
        TaxFreeFlag                 => '0',
        TaxFree                     => '00',
        TaxFreeConcept              => '',
        KindOfLocation              => 'AP',
        RegionCode                  => 'ES',
        KindOfStreet                => 'CL',
        StreetName                  => 'DIR DEMO 3    56789012345678901234567980',
        StreetNumber                => '    20',
        Building                    => 'XXXX',
        Stairs                      => 'ESCA',
        Floor                       => 'PISO',
        Door                        => 'PUER',
        OtherInformationInAddress   => '1234567890123456789012345678901234567890',
        ZipCode                     => '12530',
        PostOfficeBoxCode           => 'APTCORREOS',
        PhoneNumber1                => '12345678901234567890',
        PhoneNumber2                => '12345678901234567890',
        MobileNumber1               => '12345678901234567890',
        MobileNumber2               => '12345678901234567890',
        FaxNumber                   => '12345678901234567890',
        Email                       => 'SANTIAGO.DIAZ@tecnocom.es',
        ActivityCode                => '5119',
        RsaVerification             => 'N',
        RecurrentAdmitted           => 'N',
        ThreeDSecureConfiguration   => 'S',
        RecurrentMaximum            => '0010',
        TokenDaysOfExpiration       => '0999',
        TokenGenerationOptions      => 'N',
        QuarantineTime              => '0050',
        CaptchaOption               => 'N',
        TimeStampTime               => '0100',
        DaysHistory                 => '0180',
        GetIssuerBankCountry        => 'S',
        GetCountryIp                => 'S',
        WebLanguage                 => 'ES',
        RouterLevel                 => '1',
        CancellationOptions         => '0',
        RefundOptions               => '1',
        RefundDays                  => '0180',
        UrlOk                       => 'http://es-d-j9x1z3j:9208/',
        UrlKo                       => 'http://es-d-j9x1z3j:9208/',
        AcquiringFinancialEntity    => '0182',
        AcquiringMerchant           => '015778814',
        TerminalNumber              => '00000001',
        AdyenAccount                => 'Telefonica_Tecnocom',
        RFU                         => 'test',
    };
}

#==================================================================================================
# function     CreateRegistrationTransaction
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       CreateRegistrationTransaction();
#--------------------------------------------------------------------------------------------------
# description  Creates registration transaction.
#--------------------------------------------------------------------------------------------------
# input        $hParams                 | Alias which can be set   | hash.ref
# return       $RegistrationTransaction | registration transaction | object
#==================================================================================================
sub CreateRegistrationTransaction {
    my ($hParamas) = @_;

    my $RegistrationTransaction = DE_EPAGES::SIA::API::Object::RegistrationTransaction->new();
    my $PartlyMockedRegTrans    = Test::MockObject::Extends->new($RegistrationTransaction);

    my $hMockedRegistrationData  = CreateMerchantRegistrationDataHash($hParamas->{'Alias'},$hParamas->{'Payment'});

    $PartlyMockedRegTrans->mock(
        'convertToHash',
        sub {
            return $hMockedRegistrationData;
        }
    );
    $PartlyMockedRegTrans->mock(
        'getTerminalNumber',
        sub {
            $hMockedRegistrationData->{'TerminalNumber'};
        }
    );
    $PartlyMockedRegTrans->mock(
        'getMerchantID',
        sub {
            $hMockedRegistrationData->{'MerchantID'};
        }
    );
    $PartlyMockedRegTrans->mock(
        'getMerchantFUC',
        sub {
            $hMockedRegistrationData->{'MerchantFUC'};
        }
    );

    return $PartlyMockedRegTrans;
}

sub ExtendMockCancelRequest {
    my ($MockedDALSystem,$MockedLineItemPaymentSIA) = @_;

    my $CancelRequestObject         = DE_EPAGES::SIA::API::Request::Cancel->new('LineItemPayment' => $MockedLineItemPaymentSIA);
    my $MockedCancelRequestObject   = Test::MockObject::Extends->new($CancelRequestObject);

    $MockedLineItemPaymentSIA->mock('_getCancelRequestDataObject',sub {
        my $self = shift;
        return $MockedCancelRequestObject;
    }
    );
    $MockedCancelRequestObject->mock('getLanguageCode',sub {
        my $self = shift;

        return 'de';
    }
    );
    $MockedCancelRequestObject->mock('_loadRootObject',sub {
        my $self = shift;

        return $MockedDALSystem;
    }
    );

    return $MockedCancelRequestObject;
}

sub ExtendMockSaleRequest {
    my ($MockedDALSystem,$MockedLineItemPaymentSIA) = @_;

    my $SaleRequestObject           = DE_EPAGES::SIA::API::Request::Sale->new('LineItemPayment' => $MockedLineItemPaymentSIA);
    my $MockedSaleRequestObject     = Test::MockObject::Extends->new($SaleRequestObject);

    $MockedSaleRequestObject->mock('getLanguageCode',sub {
        my $self = shift;

        return 'de';
    }
    );

    $MockedSaleRequestObject->mock('_loadRootObject',sub {
        my $self = shift;

        return $MockedDALSystem;
    }
    );

    $MockedLineItemPaymentSIA->mock('_getSaleRequestDataObject',sub {
        my $self = shift;
        return $MockedSaleRequestObject;
    }
    );

    return $MockedSaleRequestObject;
}

sub ExtendMockLineItemPaymentSIA {
    my ($MockedDALSystem,$MockedPaymentMethodSIA) = @_;

    #mock lineitem payment
    my $LineItemPaymentSIA          = DE_EPAGES::SIA::API::Object::LineItemPaymentSIA->new();
    my $MockedLineItemPaymentSIA    = Test::MockObject::Extends->new($LineItemPaymentSIA);

    my $MockedDALShop                  = $MockedDALSystem->get('Shop');
    my $MockedDALOrder                 = $MockedDALSystem->get('OrderSIA');
    my $MockedDALLineItemPayment       = $MockedDALOrder->get('LineItemContainer')->get('Payment');

    $MockedLineItemPaymentSIA->mock('get',sub {
        my $self = shift;
        my ($attrName) = @_;

        return $MockedPaymentMethodSIA if $attrName eq 'PaymentMethod';
        return $MockedDALLineItemPayment->get($attrName);
    }
    );
    $MockedLineItemPaymentSIA->mock('getSite',sub {
        my $self = shift;
        return $MockedDALShop;
    }
    );

    ExtendMockSaleRequest($MockedDALSystem,$MockedLineItemPaymentSIA);
    ExtendMockCancelRequest($MockedDALSystem,$MockedLineItemPaymentSIA);

    return $MockedLineItemPaymentSIA;
}

sub ExtendMockPaymentMethodSIA {
    my ($MockedDALSystem) = @_;

    my $MockedDALShop                  = $MockedDALSystem->get('Shop');
    my $MockedDALPaymentMethod         = $MockedDALSystem->get('PaymentMethodSIA');

    # mock paymentmethod
    my $PaymentMethodSIA            = DE_EPAGES::SIA::API::Object::PaymentMethodSIA->new();
    my $MockedPaymentMethodSIA      = Test::MockObject::Extends->new($PaymentMethodSIA);

    # mock config
    my $SIAConfig                   = DE_EPAGES::SIA::API::SIAConfig->new('IsInTestMode' => 1);
    my $MockedSIAConfig             = Test::MockObject::Extends->new($SIAConfig);

    # mock functions
    $MockedPaymentMethodSIA->mock(
        'get',
        sub {
            my $self = shift;
            my ($attrName) = @_;

            return $MockedDALPaymentMethod->get($attrName);
        }
    );

    $MockedPaymentMethodSIA->mock(
        'set',
        sub {
            my $self = shift;
            my ($hAttr) = @_;

            foreach my $key (keys %{$hAttr}) {
                $MockedDALPaymentMethod->set($key => $hAttr->{$key});
            }
            return;
        }
    );

    $MockedPaymentMethodSIA->mock(
        'getSite',
        sub {
            my $self = shift;

            return $MockedDALShop;
        }
    );
    $MockedPaymentMethodSIA->mock(
        'child',
        sub {
            my $self = shift;
            my ($ChildAlias) = @_;

            return $MockedDALPaymentMethod->child($ChildAlias);
        }
    );

    $MockedPaymentMethodSIA->mock(
        'folder',
        sub {
            my $self = shift;
            my ($FolderAlias) = @_;

            return $self->child($FolderAlias);
        }
    );

    $MockedSIAConfig->mock('_loadRootObject',sub {
        my $self = shift;
        return $MockedDALSystem;
    });

    $MockedPaymentMethodSIA->mock('getConfig',sub {
        my $self = shift;

        return $MockedSIAConfig;
    });

    $MockedPaymentMethodSIA->mock('_getInfoLanguage',sub {
            my $self = shift;
            my ($Shop) = @_;
            return 'de';
        }
    );

    return $MockedPaymentMethodSIA;
}

#==================================================================================================
# function     CreateMockedServlet
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       CreateMockedServlet();
#--------------------------------------------------------------------------------------------------
# description  Creates mocked servlet.
#--------------------------------------------------------------------------------------------------
# return       $MockedServlet | mocked servlet | object
#==================================================================================================
sub CreateMockedServlet {
    my $self = shift;

    my $ShopObject = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', {} );
    my $MockifyServletBasket = DE_EPAGES::Test::Mock::MockifyServlet->new($ShopObject);

    return $MockifyServletBasket->getMockObject();
}

1;
