#==============================================================================
# package      DE_EPAGES::Shipping::UI::ShippingMethod
# state        private
#==============================================================================
package DE_EPAGES::Shipping::UI::ShippingMethod;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File  qw ( DeleteFile ExistsFile );

sub _EnableShippingMethod {
    my ($Product, $ShippingMethod, $EnabledByDefault, $aShopShippingMethods) = @_;

    my $aProductShippingMethods = $Product->get('ShippingMethods');

    if ( scalar(@$aProductShippingMethods) && $EnabledByDefault ) {
        push(@$aProductShippingMethods, $ShippingMethod);
    }
    elsif ( !$EnabledByDefault ) {
        my @NewProductShippingMethods = ();
        my $aOldProductShippingMethods;
        if ( scalar(@$aProductShippingMethods) ) {
            $aOldProductShippingMethods = $aProductShippingMethods;
        }
        else {
            $aOldProductShippingMethods = $aShopShippingMethods;
        }

        foreach my $ProductShippingMethod (@$aOldProductShippingMethods) {
            if( $ProductShippingMethod->id != $ShippingMethod->id ) {
                push(@NewProductShippingMethods, $ProductShippingMethod);
            }
        }
        $aProductShippingMethods = \@NewProductShippingMethods;
    }

    #if all shipping methods are enabled then this attribute should be undef
    if (scalar(@$aShopShippingMethods) <= scalar(@$aProductShippingMethods)) {
        $Product->set({'ShippingMethods' => undef});
    }
    else {
        $Product->set({'ShippingMethods' => $aProductShippingMethods});
    }
}

sub Save {
    my $self = shift;
    my ($Servlet) = @_;
    my $ShippingMethod = $Servlet->object;
    my $IsDefault      = $Servlet->params('IsDefault');
    my $IsActivated    = $Servlet->params('IsActivated');
    my $Shop = $ShippingMethod->getSite();

    if( $Shop->featureMaxValue('ShippingMethodsPerProduct') ) {
        my $EnabledByDefaultUpdateProducts = $Servlet->form->value('EnabledByDefaultUpdateProducts');
        my $EnabledByDefault = $Servlet->form->value('EnabledByDefault');
        if ( defined $EnabledByDefaultUpdateProducts && $EnabledByDefaultUpdateProducts ) {
            my $aShopShippingMethods = $Shop->get('ShippingMethods');
            # go through all products and set the shipping method as defined
            my $ProductFolder = $Shop->get('ProductFolder');
            my $aProductIDs = defined $ProductFolder ? $ProductFolder->findChildren : [];
            foreach my $ProductID ( @$aProductIDs ) {
                my $Product = $Shop->loadObject($ProductID, 'Product');
                # If the product is a master product, also add the subproducts to the array of product ids
                if ($Product->get('IsMaster') == 1) {
                    push(@$aProductIDs, map {$_->id} @{$Product->get('SubProducts')});
                }
                _EnableShippingMethod($Product, $ShippingMethod, $EnabledByDefault, $aShopShippingMethods);
            }
        }
    }

    # test if default is visible and shipping method settings are complete
    if (defined $IsActivated and $IsActivated == 1) {
        my $aShopCurrencies = $Shop->get('Currencies');
        my @ActiveCurrencies = grep { $_->{'IsActivated'} == 1 } @$aShopCurrencies;
        my $hVars = { 'TaxModel'   => $Shop->get('TaxModel') };
        my $ShippingFulfillmentMethod = $ShippingMethod->get('SelectedShippingFulfillmentMethod');
        my $FulfillmentUsable;

        if(defined($ShippingFulfillmentMethod)){
            $FulfillmentUsable = $ShippingFulfillmentMethod->isUsable();
        } else {
            my $ShippingProviderMethod = $ShippingMethod->get('SelectedShippingProviderMethod');
            $FulfillmentUsable = defined($ShippingProviderMethod) ? $ShippingProviderMethod->isUsable() : 1;
        }

        foreach my $ActiveCurrency (@ActiveCurrencies) {
            $hVars->{'CurrencyID'} = $ActiveCurrency->{'CurrencyID'};
            my $SettingsComplete = $ShippingMethod->usableAtStorefront($hVars);
            if ($SettingsComplete == 0 || $FulfillmentUsable == 0) {
                $Servlet->form->executeFormError({
                    'Reason'=> 'SETTINGS_INCOMPLETE',
                    'Name'  => 'IsActivated',
                    'Value' => $ShippingMethod->alias,
                });
            }
        }
    }
    if (not $IsActivated and $IsDefault) {
        $Servlet->form->executeFormError({
            'Reason'=> 'DEACTIVATE_DEFAULT',
            'Name'  => 'IsActivated',
        }) ;
    }

    my ($OldDefault) = grep { $_->get('IsDefault') } @{$Shop->get('ShippingMethods')};
    $OldDefault->set({'IsDefault'=>0}) if defined $OldDefault and $IsDefault and $OldDefault->id != $ShippingMethod->id;

    # store old image maybe we have to delete it
    my $OldImage = $Servlet->object()->get('Image');

    $self->SUPER::Save($Servlet);

    # remove old image if new attribute and old image not equal
    if ($OldImage and ($OldImage ne $ShippingMethod->get('Image'))) {
        my $OldShippingImage = join '/', $ShippingMethod->get('PublicPath'), $OldImage;
        DeleteFile($OldShippingImage) if ExistsFile($OldShippingImage);
    }

    # We force the change to the General tab in case the object it's not still configured.
    my $IsUsable = $ShippingMethod->get( 'UsableAtStorefront' ) ;
    my $CurrentViewAction = $Servlet->vars( 'ViewAction' ) ;

    if ( (not $IsUsable) and (defined $CurrentViewAction) and ($CurrentViewAction eq 'MBO-ViewSettings') ) {
        $Servlet->vars( 'ViewAction', 'MBO-ViewGeneral' ) ;
    }

    return ;

}

sub AddCustomerGroup {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShippingMethod = $Servlet->object;

    return unless $Servlet->params('AddCustomerGroupID');
    my $CustomerGroup = $ShippingMethod->getSite()->loadObject( $Servlet->params('AddCustomerGroupID'), 'CustomerGroup' );

    my $Form = $Servlet->form;
    eval {
        $ShippingMethod->addCustomerGroup($CustomerGroup);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code eq 'CustomerGroupAssignedToAnotherShippingMethodAtThisTimeFrame';
        $Form->mapFormError({'Name' => 'AddCustomerGroupID'}, $Error);
    }
    $Form->executeFormError;
    return;
}

sub RemoveCustomerGroups {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShippingMethod   = $Servlet->object;
    my @CustomerGroupIDs = $Servlet->form->value('ListObjectID');
    my $aCustomerGroups  = $ShippingMethod->getSite()->loadObjects( \@CustomerGroupIDs, 'CustomerGroup' );

    foreach my $CustomerGroup (@$aCustomerGroups) {
        $ShippingMethod->removeCustomerGroup($CustomerGroup);
    }

    return;
}

1;
