#========================================================================================
# package      DE_EPAGES::Shop::API::Resource::Formatter
# state        public
# description  formatter class for shop-dependend REST resources.
#========================================================================================
package DE_EPAGES::Shop::API::Resource::Formatter;

use strict;
use DE_EPAGES::Core::API::TestParameter qw (TestObject TestString);
use DE_EPAGES::Shop::API::Constants qw (TAX_MODEL_GROSS);
use JSON::XS;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Formatter = DE_EPAGES::Shop::API::Resource::Formatter->new($Shop, $LocaleID);
# example      $Formatter = DE_EPAGES::Shop::API::Resource::Formatter->new($Object->getSite,'de_DE');
#----------------------------------------------------------------------------------------
# description  Create a new Formatter object for a given shop to provide some format methods.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGET::Shop::API::Object::Shop
# input        $LocaleID | locale string i.e. 'de_DE' | string
# return       $Formatter | formatter | DE_EPAGES::Shop::API::Resource::Formatter
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop, $LocaleID) = @_;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    TestString('LocaleID', $LocaleID, 5, 5); #locale should have exactly 5 chars

    return bless {
        Shop => $Shop,
        Locale => $Shop->siteLocale($LocaleID),
    }, $class;
}

#========================================================================================
# function     shop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Shop = $Formatter->shop;
#----------------------------------------------------------------------------------------
# description  Returns the shop.
#----------------------------------------------------------------------------------------
# return       $Shop | shop of the current request | DE_EPAGES::Shop::API::Object::Shop
#========================================================================================
sub shop {
    return shift->{'Shop'};
}

#========================================================================================
# function     locale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Locale = $Formatter->locale;
#----------------------------------------------------------------------------------------
# description  Returns the locale.
#----------------------------------------------------------------------------------------
# return       $Locale | locale of the formatter | DE_EPAGES::Object::API::Object::SiteLocale
#========================================================================================
sub locale {
    return shift->{'Locale'};
}

#========================================================================================
# function     price
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hPrice = $Formatter->price($Amount, $CurrencyID, $TaxModel);
# example      $hPrice = $Formatter->price(27.25, 'EUR', undef);
# example      $hPrice = $Formatter->price(1, 'GBP', TAX_MODEL_GROSS);
#----------------------------------------------------------------------------------------
# description  Returns a formatted price of amount and currency symbol
#               as defined for the formatter locale
#----------------------------------------------------------------------------------------
# input        $Amount | amount value | float
# input        $CurrencyID | currency string i.e. 'EUR' | string
# input        $TaxModel | locale string i.e. 'de_DE' | integer
# return       $hPrice | formated price for use in a JSON structure, hash keys are:
#                   <ul><li>amount - price value - float</li>
#                       <li>currency - currency id, e.g. 'USD' - string</li>
#                       <li>locale - locale id, e.g. 'en_US'</li>
#                       <li>taxType - tax included? GROSS/NET/NONE</li>
#                   </ul> | ref.hash
#========================================================================================
sub price {
    my $self = shift;
    my ($Amount, $CurrencyID, $TaxModel) = @_;

    return undef unless defined $Amount;
    return {
        amount => $Amount,
        currency => $CurrencyID,
        formatted => defined $Amount ? $self->locale->formatMoney($Amount, $CurrencyID) : undef,
        taxType => defined $TaxModel ? $TaxModel == TAX_MODEL_GROSS ? 'GROSS' : 'NET' : 'NONE',
    };
}

#========================================================================================
# function     bool
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Bool = $Formatter->bool($Value);
# example      $hStruct = { isAvaliabe => $Formatter->bool($IsAvailable) };
# example      $Bool = $Formatter->bool( $x < $y );
#----------------------------------------------------------------------------------------
# description  Returns a formatted boolean in json format
#----------------------------------------------------------------------------------------
# input        $Value | unformatted boolean value | boolean
# return       $Bool | formated boolean value | JSON::XS::true or JSON::XS::false
#========================================================================================
sub bool {
    my $self = shift;
    my ($Value) = @_;

    return undef unless defined $Value;
    return $Value ? JSON::XS::true : JSON::XS::false;
}

#========================================================================================
# function     quantity
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hQuantity = $Formatter->quantity($Amount);
# syntax       $hQuantity = $Formatter->quantity($Amount, $Unit);
# example      $hStruct = { quantity => $Formatter->quantity(1) }
# example      $hStruct = { quantity => $Formatter->quantity(1,$Unit) }
#----------------------------------------------------------------------------------------
# description  Returns a formatted qunatity in json format
#----------------------------------------------------------------------------------------
# input        $Amount | amount value | float
# input        $Unit | (optional) unit of measurement | DE_EPAGES::Object::API::Object::UnitOfMeasurement
# return       $hQuantity | formated quantity value | JSON::true or JSON::false
#========================================================================================
sub quantity {
    my $self = shift;
    my ($Amount, $Unit) = @_;

    return undef unless defined $Amount;
    return {
        amount => $Amount,
        unit => $self->unit( $Unit ),
    }
}

#========================================================================================
# function     unit
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UnitName = $Formatter->unit($Unit);
# example      $hStruct = { unit => $Formatter->unit($Unit) }
#----------------------------------------------------------------------------------------
# description  Returns a formatted unit representation
#----------------------------------------------------------------------------------------
# input        $Unit | unit of measurement | DE_EPAGES::Object::API::Object::UnitOfMeasurement
# return       $UnitName | representation of unit (symbol or name) | string
#========================================================================================
sub unit {
    my $self = shift;
    my ($Unit) = @_;
    return undef unless defined $Unit;

    my $Locale = DE_EPAGES::Object::API::Object::Locale->load($self->locale->get('LocaleID'));
    return $Unit->get('Abbreviation') // $Unit->get('NameOrAlias', $Locale->get('LanguageID'));
}

#========================================================================================
# function     percentage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hPercentage = $Formatter->percentage($Percentage);
# example      $hStruct = { percentage => $Formatter->percentage($Percentage) }
#----------------------------------------------------------------------------------------
# description  Returns a formatted percentage representation
#----------------------------------------------------------------------------------------
# input        $Percentage | percentage value | integer
# return       $hPercentage | formated percentage for use in a JSON structure, hash keys are:
#                   <ul><li>percentage - percentage value - float</li>
#                       <li>formatted - value with percentage symbol - string</li>
#                   </ul> | ref.hash
#========================================================================================
sub percentage {
    my $self = shift;
    my ($PercentageValue) = @_;

    return {
        percentage => $PercentageValue,
        formatted => defined $PercentageValue ? $self->locale->formatFloat($PercentageValue, 2) . ' %' : undef
    };
}

1;
