#========================================================================================
# package      DE_EPAGES::ShopWizard::API::ShopWizard
# state        private
# description  Functions to save data from shop wizard
#========================================================================================
package DE_EPAGES::ShopWizard::API::ShopWizard;
use base qw( Exporter );

use strict;

our @EXPORT_OK = qw(
    SaveWizard
    ChangeDesignStyle
);

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Design::API::Style qw ( AddStyle ExistsStyle Style );
use DE_EPAGES::Object::API::Template qw ( ProcessFileTemplate );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadRootObject LoadObject Transaction );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguage );
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::Template;

#========================================================================================
# function     SaveWizard
# state        private
#----------------------------------------------------------------------------------------
# syntax       SaveWizard( $Shop, $hShopWizardAttributes );
# example      SaveWizard( $Shop, $hShopWizardAttributes );
#----------------------------------------------------------------------------------------
# description  Saves the given data from shopwizard.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $hShopWizardAttributes | shop attributes | ref.hash
#========================================================================================
sub SaveWizard {
    my ($Shop, $hShopWizardAttributes) = @_;

    # remove and add languages
    my $aFormLanguages = [ map { $_->{'ShopLanguageID'} } @{$hShopWizardAttributes->{'ShopLanguageID'}} ];
    _SaveLanguages($Shop, $aFormLanguages);

    # set address attributes
    my %AddressAttributes = ();
    $AddressAttributes{'Alias'} = 'Address';
    for my $attr (qw(City Company FirstName LastName CountryID EMail Street Zipcode Phone Fax VATID CommercialRegister State)) {
        $AddressAttributes{$attr} = $hShopWizardAttributes->{$attr};
    }
    my $Address = _SaveAddressData($Shop, \%AddressAttributes);
    my $ImprintAddress = _GetImprintAddress($Address);
    _SaveImprintPage($Shop, $Address, $ImprintAddress);

    # remove and add locales
    my $hCountry = $Address->get('Country');
    my $CountryID = defined $hCountry ? $hCountry->{'CountryID'} : 0;

    # for unity we try to find a matching locale for the language and the shop locale
    # e.g. if we have German as language and switzerland as address we try to add
    # de_CH as locale and not de_DE
    if($Shop->get('IsUnityRegistered')){
        _SaveUnityLocales($Shop, $aFormLanguages);
        _CleanupLanguagesAndCountries($Shop);

    } else {
        _SaveLocales($Shop, $aFormLanguages, $CountryID);    
    }
  
    # set MailFrom, TaxModel, InputTaxModel, Logo, SelectedStyleGroup
    my $hShopAttributes = {
        'MailFrom'  => $Address->get('EMail'),
    };
    if (exists $hShopWizardAttributes->{'TaxModel'}) {
        $hShopAttributes->{'TaxModel'} = $hShopWizardAttributes->{'TaxModel'};
        $hShopAttributes->{'InputTaxModel'} = $hShopWizardAttributes->{'TaxModel'};
    }
    $hShopAttributes->{'Logo'} = $hShopWizardAttributes->{'Logo'} if defined $hShopWizardAttributes->{'Logo'};
    $hShopAttributes->{'SelectedStyleGroup'} = LoadObject($hShopWizardAttributes->{'StyleGroup'}) if defined($hShopWizardAttributes->{'StyleGroup'});
    $Shop->set($hShopAttributes);

    #set shop attributes
    my %ShopData;
    my $ShopLanguageID = $Shop->get('LanguageID');
    foreach my $Attribute ( qw( Name Slogan ) ) {
        $ShopData{$Attribute} = $hShopWizardAttributes->{$Attribute} if defined $hShopWizardAttributes->{$Attribute};
    }
    my $Signature = "--<br/><br/>\n" . join ("<br />\n", map { defined($_) && $_ =~ /\S/  ?  $_ : () } (
        $Shop->get('Name', $ShopLanguageID),
        $Address->get('FullName'),
        $ImprintAddress,
        $Address->get('EMail')
    ) );
    $ShopData{Signature} = $Signature;
    foreach my $LanguageID ( @{$Shop->languages} ) {
        my %ShopDataForLanguage = %ShopData;
        $Shop->set(\%ShopDataForLanguage, $LanguageID)
            if $LanguageID == $ShopLanguageID or $Shop->get('InitialShopWizardStartup') or not defined $Shop->get('Name', $ShopLanguageID);
    }

    $Shop->set({ 'HasCreatedInitialShortURLs' => 1 }) if defined $Shop->get('DomainName');
    return;
}

sub _GetImprintAddress {
    my ($Address) = @_;

    # build the address for the signature and imprint page
    my $City = $Address->get('City');
    my $ZipCode = $Address->get('Zipcode');
    my $ZipCode_City = (defined $City && defined $ZipCode) ? $ZipCode . ' ' . $City : '';
    my $hCountry = $Address->get('Country');
    my $Country = defined $hCountry ? $hCountry->{'Native'} : '';
    my $ImprintAddress = join ("<br />\n", map { defined($_) && $_ =~ /\S/  ?  $_ : () } (
        $Address->get('Street'),
        $Address->get('Street2'),
        $ZipCode_City,
        $Country,
        $Address->get('State')
    ) );

    return $ImprintAddress;
}

sub _SaveAddressData {
    my ($Shop, $hAddressData)  = @_;
    my $Address;
    if ($Shop->existsChild('Address')) {
        $Address = $Shop->child('Address');
        $Address->set($hAddressData);
    } else {
        $Address = $Shop->insertChild( 'ShopAddress', $hAddressData );
    }

    return $Address;
}

sub _SaveImprintPage {
    my ($Shop, $Address, $ImprintAddress) = @_;

    #set imprint address and additional information
    my $Imprint = $Shop->get('Imprint');
    if (defined $Imprint) {
        my $hVars = $Shop->tleHash;

        my $FileName = DE_EPAGES::Presentation::API::Template->new->templateFileName(
            LoadPageTypeByAlias('MBO-ShopWizard')->template( 'INC-Imprint-AdditionalInformation' )
        );

        foreach my $LanguageID ( @{$Shop->languages} ) {
            # process template for imprint additional information
            my $Text = ProcessFileTemplate($FileName, GetCodeByLanguageID($LanguageID), $hVars);
            $Imprint->set({
                'Address'       => $ImprintAddress,
                'Email'         => $Address->get('EMail'),
                'Company'       => $Address->get('Company'),
                'ContactPerson' => $Address->get('FullName'),
                'Phone'         => $Address->get('Phone'),
                'Fax'           => $Address->get('Fax'),
                'Text'          => $Text,
            }, $LanguageID);
        }
    }
    return;
}

sub _SaveLanguages {
    my ($Shop, $aLanguages) = @_;
    my $aShopLanguages = $Shop->languages;

    # faster access to selected languages
    my %SelectedLanguages = map { $_ => 1 } @$aLanguages;

    # remove not longer used languages
    foreach my $ShopLanguage (@$aShopLanguages) {
        $Shop->removeLanguage($ShopLanguage) unless exists $SelectedLanguages{$ShopLanguage};
    }
    # add new languages
    foreach my $FormLanguage (@$aLanguages) {
        $Shop->addLanguage($FormLanguage) unless (ElementOf($FormLanguage, @$aShopLanguages));
    }
    return;
}

sub _SaveLocales {
    my ($Shop, $aLanguages, $CountryID) = @_;
    # activate matching locales
    my $System = LoadRootObject();
    my %SelectedLanguages = map { $_ => 1 } @$aLanguages;

    # locales speaking any of the selected languages within the country of the shop
    my @CountryLocales = map { $_->get('LocaleID') }
        grep { $_->get('CountryID') == $CountryID }
        grep { exists $SelectedLanguages{$_->get('LanguageID')} }
        @{$System->get('Locales')};

    # visible locales matching new available languages ?
    my @ExistingShopLocales = map {$_->alias} @{$Shop->get('SiteLocales')};
    my @VisibleLocales = grep { $_->get('Language')->{'DefaultLocaleID'} eq $_->get('LocaleID') ||
                                ElementOf($_->get('LocaleID'), @ExistingShopLocales)
                         }
                         grep { exists $SelectedLanguages{$_->get('LanguageID')} }
                         @{$Shop->get('Locales')};
    if (not scalar @VisibleLocales) {
        if (scalar @CountryLocales) {
            @VisibleLocales = ($Shop->siteLocale($CountryLocales[0]));
        } else {
            @VisibleLocales = map { $Shop->siteLocale($_) }
                              grep { defined $_ }
                              map { $_->{'DefaultLocaleID'} }
                              map { GetInfoLanguage($_); }
                              @$aLanguages;
        }
    }

    # add/remove locales
    my %VisibleLocaleIDs = map { $_->get('LocaleID') => 1 } @VisibleLocales;
    foreach my $LocaleID (keys %VisibleLocaleIDs) {
        $Shop->addLocale($LocaleID) unless $Shop->existsLocale($LocaleID);
    }
    foreach my $LocaleID ( map { $_->get('LocaleID') } @{$Shop->get('SiteLocales')} ) {
        $Shop->removeLocale($LocaleID) unless exists $VisibleLocaleIDs{$LocaleID};
    }

    if (not exists $VisibleLocaleIDs{$Shop->get('LocaleID')}) {
        my $LocaleID = scalar @CountryLocales
            ? $CountryLocales[0]
            : $VisibleLocales[0]->get('LocaleID');
        $Shop->addLocale($LocaleID) unless $Shop->existsLocale($LocaleID);
        $Shop->set({'LocaleID' => $LocaleID});
    }
    return;
}

sub _SaveUnityLocales{
    my ($Shop, $aLanguages) = @_;

    my $ShopLanguageID = $Shop->get('LanguageID');
    # find the matching locale and add to shop
    my $ShopLanguage = shift @{LoadClassByAlias('ShopLanguage')->getObjects('ByShopIDLanguageID', [$Shop->id, $ShopLanguageID])};
    my $LocaleIDForNewShop = $Shop->localeForLanguageByShopLocaleIDAndLanguage($ShopLanguage);

    # be sure have no locales...
    foreach my $LocaleID ( map { $_->get('LocaleID') } @{$Shop->get('SiteLocales')} ) {
        $Shop->removeLocale($LocaleID);
    }
    # now add the matching locale to the shop
    $Shop->addLocale($LocaleIDForNewShop);

}

sub _CleanupLanguagesAndCountries {
    my ($Shop) = @_;

    # remove obsolete Languages
    my $ahLanguages = $Shop->get('Languages');
    my $ShopLanguageID = $Shop->get('LanguageID');

    foreach my $hLanguage (@$ahLanguages){
        my $LanguageID = $hLanguage->{'LanguageID'};
        next if $LanguageID eq $ShopLanguageID;
        $Shop->removeLanguage($LanguageID) if $Shop->existsLanguage($LanguageID);         
    }

    # remove obsolet shop countries but not the country from the SiteLocale (shop address country)
    # and set this country as default
    my $SiteLocaleCountryID = $Shop->get('SiteLocale')->get('Country')->{'CountryID'};
    foreach my $CountryID (@{$Shop->countries}) {
        $Shop->removeCountry($CountryID);
    }

    $Shop->addCountry($SiteLocaleCountryID,10,1);

}

#========================================================================================
# function     ChangeDesignStyle
# state        private
#----------------------------------------------------------------------------------------
# syntax       ChangeDesignStyle( $Shop, $StyleTemplateID );
# example      ChangeDesignStyle( $Shop, $StyleTemplateID );
#----------------------------------------------------------------------------------------
# description  Sets or adds the new style to the shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $StyleTemplateID | style template identifier | integer
# return       $WantedStyle | created style object | object
#========================================================================================
sub ChangeDesignStyle {
    my ($Shop, $StyleTemplateID) = @_;

    my $StyleTemplate = LoadObject($StyleTemplateID);
    #delete old NavBarElements(if Style exists) and override/add Style
    my $WantedStyle;
    Transaction ( sub {
        $WantedStyle = ExistsStyle ($Shop, $StyleTemplate)
            ? Style($Shop,$StyleTemplate)
            : AddStyle($Shop, $StyleTemplate);
    });
    return $WantedStyle;
}

1;
