#========================================================================================
# package      DE_EPAGES::TaxUSSystemDB::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::TaxUSSystemDB::API::Cartridge;
use base DE_EPAGES::Trigger::API::TriggerInstaller;

use strict;
use DE_EPAGES::Core::API::File qw ( DeleteFile ExistsFile );
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Database::API::Config qw ( DatabaseConfig DatabaseConfigFileName );
use DE_EPAGES::Database::API::Connection qw (Transaction);
use DE_EPAGES::TaxUSSystemDB::API::Zip2TaxUS qw (
    UpdateZip2TaxUSTable
    UpdateZip2TaxUSMetaTable
    LoadSystemDBTable
);
use DE_EPAGES::TaxUSSystemDB::API::Table::Zip2TaxUSMeta qw ( GetNextZip2TaxUSVersion );

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::TaxUSSystemDB::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::TaxUSSystemDB.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/TaxUSSystemDB',
        'Version'            => '7.11.0',
        'Patches'            => ['6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;

    $self->finishInstall if $ENV{SKIP_UPDATE_ZIP2TAXUS};

    my $RecreateZip2TaxUSTables = $ENV{RECREATE_ZIP2TAXUS_TABLES};

    if (not $RecreateZip2TaxUSTables) {
    Transaction(sub {
            my $BU = _GetSystemDBImportBU();
            if (LoadSystemDBTable('zip2taxus', $BU)) {
                $RecreateZip2TaxUSTables = 1 unless
                    LoadSystemDBTable('zip2taxusmeta', $BU);
            } else {
                $RecreateZip2TaxUSTables = 1;
            }
            DeleteFile(DatabaseConfigFileName($BU));
        });
    }

    $self->finishInstall unless $RecreateZip2TaxUSTables;

    Transaction(sub {
        UpdateZip2TaxUSMetaTable(
            GetNextZip2TaxUSVersion(),
            'http://tables.zip2tax.com/?usr=epagestg&pwd=gme89wu&id=1');
        UpdateZip2TaxUSTable();
    });

    $self->finishInstall;
    return;
}

#========================================================================================
# function     _GetSystemDBImportBU
# state        private
#----------------------------------------------------------------------------------------
# syntax       $BU = _GetSystemDBImportBU();
#----------------------------------------------------------------------------------------
# description  Copies the 'Database.d/Backup.conf', prepares the copy for the import of the
#               system db CSV dump and returns the name of the business unit.
#----------------------------------------------------------------------------------------
# return       $BU | the name of the business unit from the config copy | string
#========================================================================================
sub _GetSystemDBImportBU {
    my $OriginalBusinessUnit = 'Backup';
    my $TempBusinessUnitName = _GetTempBusinessUnitName('SystemDBImport');
    
    my $IniConfig = DatabaseConfig($OriginalBusinessUnit);
    
    my $DataSource = $IniConfig->get('Connection', 'datasource');
    unless ($DataSource =~ m/mysql_local_infile=1/) {
        $DataSource .= ';mysql_local_infile=1';
        $IniConfig->set('Connection', 'datasource', $DataSource);
    }

    my $ConfigPath = DatabaseConfigFileName($OriginalBusinessUnit);
    $ConfigPath =~ s/$OriginalBusinessUnit.conf/$TempBusinessUnitName.conf/;
    $IniConfig->write($ConfigPath);

    return $TempBusinessUnitName;
}

#========================================================================================
# function     _GetTempBusinessUnitName
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $TempBusinessUnitName = _GetTempBusinessUnitName('SystemDBImport');
#----------------------------------------------------------------------------------------
# description  Use this method to get the name of the business unit to be used for
#               the import of the system db. If the business unit already exist, the
#               given value will be incremented.
#----------------------------------------------------------------------------------------
# input        $BaseName | the first part of the import business unit | string
# return       $BusinessUnitName | the name of the import business unit | string
#========================================================================================
sub _GetTempBusinessUnitName ($) {
    my ($BaseName) = @_;

    my $ConfigFileName = $BaseName;
    my $ConfigFilePath = DatabaseConfigFileName($ConfigFileName);
    for (my $i = 2; ExistsFile($ConfigFilePath); $i++) {
        $ConfigFileName = $BaseName . $i;
        $ConfigFilePath = DatabaseConfigFileName($ConfigFileName);
    }

    return $ConfigFileName;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

1;
