#========================================================================================
# package      DE_EPAGES::Test::API::TranslationJS
# base         DE_EPAGES::Test::API::StoreUnitTest
# state        public
#----------------------------------------------------------------------------------------
# description  Checks translation files (de/en) for correct encoding: iso-8859-1
#========================================================================================
package DE_EPAGES::Test::API::TranslationJS;
use base DE_EPAGES::Test::API::StoreUnitTest;

use strict;

use Config;
use Test::More;
use DE_EPAGES::Core::API::Cartridge;
use DE_EPAGES::Core::API::File qw ( CorrectPathSlashes ExistsDirectory fsplit GetFileContent GetFileNames );
use DE_EPAGES::Core::API::SystemCommand qw ( SystemCommand );
use DE_EPAGES::Database::API::Connection qw( GetTestSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use JSON qw( from_json );

our $JSREGEX = '("|\')\{([^\s\{\'"]*?)\}("|\')|(?:\.translate\(("|\')(.*?)("|\')\))';
our $TMPLREGEX = '((?:^|[^\$\{])\{([^\{\s\'"]*?)\})';


sub test {
    my $self = shift;

    # Get storetype.
    my $StoreType = GetTestSite();

    # Get current ePages version.
    my $InstalledVersion = LoadRootObject()->get('EpagesVersion');

    # Correct slashes.
    my $DictConverterPath = CorrectPathSlashes("$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/JQuery/Scripts/DictionaryConverter.pl");
    my $PERL = $Config{'osname'} eq 'MSWin32' ? "$ENV{'EPAGES_PERL'}\\bin\\perl.exe" : $ENV{'PERL'};

    # Generate translation files (json).
    SystemCommand("$PERL $DictConverterPath -storetype $StoreType -version $InstalledVersion -cartridge ALL");

    # Test if translation-generation worked.
    my $TestFolder = $ENV{'EPAGES_WEBROOT'}."/StoreTypes/$InstalledVersion/$StoreType";
    die "Version '$InstalledVersion' not found in $ENV{'EPAGES_WEBROOT'}/Storetypes/" unless ExistsDirectory($TestFolder);

    # Get possible javascript module, dictionary files.
    my @PossibleFiles = (@{GetFileNames($ENV{'EPAGES_WEBROOT'}."/StoreTypes/$InstalledVersion/$StoreType/lib/de_epages", {'recursive' => 1})}, @{GetFileNames($ENV{'EPAGES_WEBROOT'}."/StoreTypes/$InstalledVersion/$StoreType/lib/ep", {'recursive' => 1})});
    my @Dicts = grep { $_ =~ m/.*?\.[a-z]{2}\.json$/ } @PossibleFiles;
    my @JSFiles  = grep { $_ !~ m/-test/ } grep { $_ =~ m/\.js$/ } @PossibleFiles;

    # Build datastructure foreach js module
    my $hJS2Dict = _GetJSDictRequire(\@JSFiles, $InstalledVersion, $StoreType);

    # group dictionaries by name e.g. {'dict' => ['dict.en.json','dict.de.json']};
    my $hDicts = {};
    foreach my $Dict (@Dicts) {
        my $UniformDict = $Dict;
        $UniformDict =~ s/(.*?)\.[a-z]{2}\.json$/$1/;
        $hDicts->{$UniformDict} = [];
        push @{$hDicts->{$UniformDict}}, $Dict;
    }

    # Get dict contents.
    my $hCompleteDictsWithInheritance = {};
    my $hCompleteDictsWithOutInheritance = {};
    foreach my $Dict (@Dicts) {
        # Solve dict dependencies (merge dicts).
        $hCompleteDictsWithInheritance->{$Dict} = _LoadDict($Dict, $StoreType, $InstalledVersion, {}, 1);
        # dict content without inheritance.
        $hCompleteDictsWithOutInheritance->{$Dict} = _LoadDict($Dict, $StoreType, $InstalledVersion, {}, 0);
    }

    # General tests: module should require only 1 dict, require a dict if translations are used.
    CheckGeneralUseOfDictionaries($hJS2Dict, $hDicts);

    # Datastructure foreach dictionary.
    my $hDict2JSFile = {};
    foreach my $Dict (@Dicts) {
        my $UniformDict = $Dict;
        $UniformDict =~ s/(.*?)\.[a-z]{2}\.json$/$1/;
        my ($Path, $DictName, undef) = fsplit($Dict);
        # if global dict -> add all jsmodules with templates etc. .
        if($DictName =~ m/^dict\./ and $Path =~ m/ep$/) {
            $hDict2JSFile->{$Dict} = $hJS2Dict;
        }
        # if cartridge dictionary -> add all jsmodules etc. of this cartridge.
        elsif($DictName =~ m/^dictionary\./ and $Path =~ m/de_epages$/) {
            foreach my $JSFile (keys %$hJS2Dict) {
                $hDict2JSFile->{$Dict}->{$JSFile} = $hJS2Dict->{$JSFile} if $JSFile =~ m/^$Path/;
            }
        } else {
            # if module dictionary -> add only the module.
            if(exists $hJS2Dict->{"$UniformDict.js"}) {
                $hDict2JSFile->{$Dict}->{"$UniformDict.js"} = $hJS2Dict->{"$UniformDict.js"} if exists $hJS2Dict->{"$UniformDict.js"};
            }
        }
    }

    # Check if js module and module template use unknown translation.
    CheckMissingTranslations($hJS2Dict, $hDicts, $hCompleteDictsWithInheritance);
    # Check if dictionaries have unnecessary translation.
    CheckUnUsedTranslations($hDict2JSFile, $hCompleteDictsWithOutInheritance);

    done_testing();
}

#========================================================================================
# function     CheckGeneralUseOfDictionaries
#----------------------------------------------------------------------------------------
# syntax       CheckGeneralUseOfDictionaries($hJS2Dict, $hDicts);
#----------------------------------------------------------------------------------------
# description  Check the general use of dictionaries in js modules.
#----------------------------------------------------------------------------------------
# input        $hJS2Dict | hash of js modules paths as keys and values see
#                           _GetJSDictRequire | hash.refs
# input        $hDicts   | hash of dictionaries | hash.ref
#========================================================================================
sub CheckGeneralUseOfDictionaries {
    my ($hJS2Dict, $hDicts) = @_;
    # Check foreach module.
    foreach (keys %$hJS2Dict) {
        my $Okay = ($hJS2Dict->{$_}->{'hasTranslations'} and scalar @{$hJS2Dict->{$_}->{'dicts'}} == 1) ? 1 : 0;
        my $ExistsDict = 0;
        ok($Okay, "js file: '$_' requires only one dict and need translations.");
        my $AllowedDict = 0;
        if($Okay) {
            ok((not $hJS2Dict->{$_}->{'dublicatedDicts'}), "js file: '$_' requires the dict only one time.");
            my $Dict = $hJS2Dict->{$_}->{'dicts'}->[0];
            $ExistsDict = 1 if exists $hDicts->{$Dict};
            ok($ExistsDict, "dictionary '$Dict' used in '$_' exists.");
            $AllowedDict = _DictAllowedForModule($_, $Dict);
            ok($AllowedDict, "dictionary '$Dict' is allowed to use in module '$_'.");
        }
        # Remove jsmodules which are not necessary for further tests.
        delete $hJS2Dict->{$_} if not $Okay or not $AllowedDict;
    }
}

#========================================================================================
# function     CheckMissingTranslations
#----------------------------------------------------------------------------------------
# syntax       CheckMissingTranslations($hJS2Dict, $hDicts, $hCompleteDicts);
#----------------------------------------------------------------------------------------
# description  Check if js module and module template use unknown translation.
#----------------------------------------------------------------------------------------
# input        $hJS2Dict | hash of js modules paths as keys and values see
#                           _GetJSDictRequire | hash.refs
# input        $hDicts   | hash of dictionaries | hash.ref
# input        $hCompleteDicts | hash with translations foreach dictionary | hash.ref
#========================================================================================
sub CheckMissingTranslations {
    my ($hJS2Dict, $hDicts, $hCompleteDicts) = @_;

    # Foreach module:
    foreach my $JSFile (keys %$hJS2Dict) {
        # Get dictionary.
        my $Dict = $hJS2Dict->{$JSFile}->{'dicts'}->[0];
        # Get translations in all languages.
        my %CompleteDicts = map {$_ => $hCompleteDicts->{$_}} @{$hDicts->{$Dict}};
        my $JSContent = ${GetFileContent($JSFile)};
        # Check found translations in js module.
        while($JSContent =~ m/$JSREGEX/sg) {
            if(defined $2 and $2 ne '') {
                foreach my $CompleteDict (keys %CompleteDicts) {
                    ok((defined $CompleteDicts{$CompleteDict}->{$2}), "translation found for \"$2\" (file: \"$JSFile\") in dictionary: \"$CompleteDict\".");
                }
            }
        }

        # Loop over each file and check used translations.
        foreach my $Tmpl (@{$hJS2Dict->{$JSFile}->{'tmpls'}}) {
            my $FileContent = ${GetFileContent($Tmpl)};
            # Check found translations in js module templates.
            while($FileContent =~ m/$TMPLREGEX/sg) {
                # ugly fix for facebook widget where {2} is used.
                if(defined $2 and $2 ne '' and $2 ne '2') {
                    foreach my $CompleteDict (keys %CompleteDicts) {
                        ok((defined $CompleteDicts{$CompleteDict}->{$2}), "translation found for \"$2\" (file: \"$Tmpl\") in dictionary: \"$CompleteDict\".");
                    }
                }
            }
        }
    }
}

#========================================================================================
# function     CheckUnUsedTranslations
#----------------------------------------------------------------------------------------
# syntax       CheckUnUsedTranslations($hJS2Dict, $hDicts, $hCompleteDicts);
#----------------------------------------------------------------------------------------
# description  Check if dictionaries have unnecessary translation.
#----------------------------------------------------------------------------------------
# input        $hDict2JSFile | hash of dictionary paths as keys and values are arrays of
#                               return value of _GetJSDictRequire | hash.ref
# input        $hCompleteDictsWithOutInheritance | hash with translations foreach
#                                                   dictionary | hash.ref
#========================================================================================
sub CheckUnUsedTranslations {
    my ($hDict2JSFile, $hCompleteDictsWithOutInheritance) = @_;

    # Foreach dictionary:
    foreach my $Dict (keys %$hDict2JSFile) {
        # Get translations.
        my $hDictContent = $hCompleteDictsWithOutInheritance->{$Dict};
        # Foreach modules using this dict:
        foreach my $JSModule (keys %{$hDict2JSFile->{$Dict}}) {
            # Check the used translations in the module.
            my $JSContent = ${GetFileContent($JSModule)};
            foreach my $Translation (keys %{$hDictContent->{$Dict}}) {
                $hDictContent->{$Translation} = 0 if $hDictContent->{$Translation} != 1;
                if($JSContent =~ m/("|')\{\Q$Translation\E\}("|')|\.translate\(("|')\Q$Translation\E("|')\)/s) {
                    # Translation in use.
                    $hDictContent->{$Translation} = 1;
                    last;
                }
            }
            # Check the used translations in the templates of the module.
            foreach my $Tmpl (@{$hDict2JSFile->{$Dict}->{$JSModule}->{'tmpls'}}) {
                my $TmplContent = ${GetFileContent($Tmpl)};
                foreach my $Translation (keys %{$hDictContent->{$Dict}}) {
                    if($TmplContent =~ m/\{\Q$Translation\E\}/s) {
                        # Translation in use.
                        $hDictContent->{$Translation} = 1;
                        last;
                    }
                }
            }
        }
        # Loop over all translations and check their usage.
        foreach my $Translation (keys %$hDictContent) {
            ok($hDictContent->{$Translation}, "translation \"$Translation\" in use (dictionary: \"$Dict\").");
        }
    }
}

#----------------------------------------------------------------------------------------
# private helper
#----------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------
# Build datastructure for all jsmodules:
#----------------------------------------------------------------------------------------
# hash: key   = js module path
#       value = 'dicts' array.ref
#               'tmpls' array.ref
#               'dublicatedDicts' boolean
#               'hasTranslations' boolean
# example:
# {
#   '/srv/.../alert.js' => {
#                               'dicts' => ['/srv/.../ep/dict'],
#                               'tmpls' => [],
#                               'dublicatedDicts' => 0,
#                               'hasTranslations' => 1,
#                          },
#   ...
# }
#----------------------------------------------------------------------------------------
sub _GetJSDictRequire {
    my ($aJSFiles, $InstalledVersion, $StoreType) = @_;

    my $hJS2Dict = {};
    # Foreach js module.
    foreach my $JSFile (@$aJSFiles) {
        my $Content = ${GetFileContent($JSFile)};
        ok(CheckTranslationList($Content), "correct \@dictionary and \@translation comments in file $JSFile")
        # exclude JQuery translation module
            unless $JSFile =~ m/lib\/ep\/dict\.js$/;
        # Build datastructure with defaults.
        $hJS2Dict->{$JSFile} = {
            'dicts' => [],
            'tmpls' => [],
            'dublicatedDicts' => 0,
            'hasTranslations' => 0,
        };
        # Search for required dicts.
        while($Content =~ m/("|')\$dict!(.*?)("|')/g) {
            my $Dict = $2;
            # If dict found set values in datastructure.
            if(defined $Dict) {
                # Get absolute path for the dict.
                my $AbsoluteDictPath = _GetAbsoluteDictPath($JSFile, $Dict, $InstalledVersion, $StoreType);
                # Check if dict already required.
                $hJS2Dict->{$JSFile}->{'dublicatedDicts'} = 1 if $AbsoluteDictPath ~~ $hJS2Dict->{$JSFile}->{'dicts'};
                # Add dict to dict array.
                push @{$hJS2Dict->{$JSFile}->{'dicts'}}, $AbsoluteDictPath if (defined $AbsoluteDictPath and not $AbsoluteDictPath ~~ $hJS2Dict->{$JSFile}->{'dicts'});
            }
        }
        # Check if module uses translations.
        $hJS2Dict->{$JSFile}->{'hasTranslations'} = 1 if $Content =~ m/$JSREGEX/sg;

        my ($JSPath, $FileName, undef) = fsplit($JSFile);
        $FileName .= ''; # avoid perl critic error.

        # Find templates for the js module which use translations.
        my @Tmpls = grep {$_ =~ m/.*?\/\Q$FileName\E(|-[^\/]*?)\.tmpl\.html$/} @{GetFileNames($JSPath)};
        my $TmplUsesTranslations = 0;
        foreach my $Tmpl (@Tmpls) {
            my $TmplContent = ${GetFileContent($Tmpl)};
            # Check if
            if($TmplContent =~ m/$TMPLREGEX/sg) {
                # ugly fix for facebook widget where {2} is used.
                if(defined $2 and $2 ne '' and $2 ne '2') {
                    push @{$hJS2Dict->{$JSFile}->{'tmpls'}}, $Tmpl;
                    $hJS2Dict->{$JSFile}->{'hasTranslations'} = 1;
                    $TmplUsesTranslations = 1 unless $TmplUsesTranslations;
                }
            }
        }
        # Oh template has translations and wooah no dictionary required.. check if subfolder in cartridge directory
        # Automatically use cartridge dict if this exists else global dict.
        if($TmplUsesTranslations and scalar @{$hJS2Dict->{$JSFile}->{'dicts'}} == 0 and $JSFile =~ m/\/de_epages\/(.+?)\/.+\/.+\.js$/) {
            my $Cartridge = $1;
            my $AbsoluteDictPath = _GetAbsoluteDictPath($JSFile, "de_epages/$Cartridge/dictionary", $InstalledVersion, $StoreType);
            my ($CartridgePath, undef, undef) = fsplit($AbsoluteDictPath);
            my $CountCartridgeDicts = scalar (grep { $_ =~ m/\/dictionary\.[a-z]{2}\.json$/ } @{GetFileNames($CartridgePath)});

            $AbsoluteDictPath = _GetAbsoluteDictPath($JSFile, 'ep/dict', $InstalledVersion, $StoreType) unless $CountCartridgeDicts;
            push @{$hJS2Dict->{$JSFile}->{'dicts'}}, $AbsoluteDictPath if (defined $AbsoluteDictPath);
        }
    }
    # Filter results: return only modules which use translations or have required a dict,
    %$hJS2Dict = map { $_ => $hJS2Dict->{$_} } grep { scalar @{$hJS2Dict->{$_}->{'dicts'}} > 0 or $hJS2Dict->{$_}->{'hasTranslations'} }  keys %$hJS2Dict;
    return $hJS2Dict;
}

#----------------------------------------------------------------------------------------
# Returns 1 if the module is allowed to require the dict else 0.
#----------------------------------------------------------------------------------------
sub _DictAllowedForModule {
    my ($JSModule, $Dict) = @_;

    # Get js module, dict path and name.
    my ($ModulePath, $ModuleName, undef) = fsplit($JSModule);
    my ($DictPath, $DictName, undef) = fsplit($Dict);

    # Check if dictionary is cartridge dictionary and module is in this cartridge.
    if($DictName eq 'dictionary' and $ModulePath =~ m/\Q$DictPath\E.*?/) {
        return 1;
    }
    # Check if module dictionary and both files are in the same directory.
    elsif ($DictName eq $ModuleName and $ModulePath eq $DictPath) {
        return 1;
    }
    # Check if global dict.
    elsif ($DictName eq 'dict' and $DictPath =~ m/lib\/ep$/) {
        return 1
    } else {
        return 0;
    }
}

#----------------------------------------------------------------------------------------
# Get absolute path to a dict -> in module "../../dictionary" => "/srv/epages/eproot/Shared/WebRoot/.../dictionary".
#----------------------------------------------------------------------------------------
sub _GetAbsoluteDictPath {
    my ($JSFile, $Dict, $InstalledVersion, $StoreType) = @_;
    return unless (defined $Dict and defined $InstalledVersion and defined $StoreType);

    # Split path at "/".
    my @DictParts = split(/\//, $Dict);
    # Get js module path.
    my ($AbsoluteDictPath, undef, undef) = fsplit($JSFile);
    # Foreach pathpart do:
    foreach my $DictPart (@DictParts) {
        # Resolve "../" in the dict path.
        if($DictPart =~ m/\.\./) {
            # Go in parent directory.
            $AbsoluteDictPath =~ s/(.*)\/.*/$1/;
        } elsif($DictPart =~ m/^ep$/) {
            # Move rootdir from e.g. de_epages to ep.
            $AbsoluteDictPath = $ENV{'EPAGES_WEBROOT'}."/StoreTypes/$InstalledVersion/$StoreType/lib/ep";
        } elsif($DictPart eq 'de_epages') {
            $AbsoluteDictPath = $ENV{'EPAGES_WEBROOT'}."/StoreTypes/$InstalledVersion/$StoreType/lib/de_epages";
        } elsif($DictPart !~ m/^\.$/ and $DictPart ne '') {
            # Add dictname.
            $AbsoluteDictPath .= "/$DictPart";
        }
    }

    return $AbsoluteDictPath;
}

#----------------------------------------------------------------------------------------
# Load content of a dictionary (also recuresive to solve inheritance).
#----------------------------------------------------------------------------------------
sub _LoadDict {
    my ($Dict, $StoreType, $InstalledVersion, $hContent, $Recursive) = @_;

    # Save content of recursive calls.
    $hContent = {} unless defined $hContent;
    # Get current file content.
    my $Content = ${GetFileContent($Dict)};
    # Check if content is an array.
    if(ref(from_json($Content)) eq 'ARRAY') {
        my $aContent = from_json($Content);
        # Extract dict language.
        my $Language = $Dict;
        $Language =~ s/.*\.([a-z]{2})\.json$/$1/;
        # Check if there is a parent dict.
        my $ParentDictName = $aContent->[0];
        my $HasParentDict = (defined $ParentDictName and ref($ParentDictName) ne 'HASH') ? 1 : 0;
        # If parent dict exists.
        if($HasParentDict and $Recursive) {
            # Load and store parent dict content.
            %$hContent = (%{_LoadDict($ENV{'EPAGES_WEBROOT'}."/StoreTypes/$InstalledVersion/$StoreType/lib/$ParentDictName.$Language.json", $StoreType, $InstalledVersion, $hContent, $Recursive)}, %{$aContent->[1]});
            return $hContent;
        } elsif(not $Recursive and $HasParentDict) {
            return $aContent->[1];
        }
        # Return translations.
        return $aContent->[0];
    }
}

#========================================================================================
# function     CheckTranslationList
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckTranslationList($rContent);
# example      if(!$CheckTranslationList($rFileContent)) { print "error ..."; }
#----------------------------------------------------------------------------------------
# description  Applies to the ePages JavaScript API, based on jQuery. If a JS module
#               uses a function call to our dictionary like:
#                   this.dict.translate(...)
#               the module MUST contain a declaration of the used language tags like:
#                   /*
#                    * @dictionary  ep.dict
#                    *
#                    * @translation
#                    *  {Today}
#                    *  {Done}
#                    */
#----------------------------------------------------------------------------------------
# input        $rContent | javascript source code file content | ref.string
# return       0 or 1 | 1 if no problem could be found, otherwise 0 | boolean
#========================================================================================
sub CheckTranslationList {
    my ($rContent) = @_; # entire file content, hash to string

    if($rContent =~ /\.translate\(/xg && $rContent =~ /\$dict\!/) {
        if($rContent !~ /\@dictionary/x || $rContent !~ /\@translation/x) {
            return 0;
        }
    }
    return 1;
}

1;