use strict;

use DE_EPAGES::Core::API::Iterator qw ( ArrayIterator );
use DE_EPAGES::Core::API::PerlTools qw ( GetIPAddress trim );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Error qw ( HandleError );
use DE_EPAGES::WebInterface::API::ServerConfig qw ( ServerConfig );
use English qw ( -no_match_vars );    ## no critic ( DE_EPAGES::Style::RequireCorrectImports )
use Getopt::Long;
use IO::Socket::INET6;
use Sys::HostIP;
use threads;
use threads::shared;
use Time::HiRes qw ( usleep );

use constant REQUEST => "GET /\r\nX-epages-EXTRAINFO: %s\r\nuser-agent: epages Service\r\n\r\n";

RunScript( 'Sub' => \&Main, 'Message' => 'long' );

sub Main {
    local $OUTPUT_AUTOFLUSH = 1;    ## no critic ( ProhibitLocalVars )
    my ( $Help, $Port, $HostName, $Sleep, $PoolName, $All, $Reload, @ReloadModules, $ForceSequential, $Timeout, $MaxThreads );
    GetOptions(
        'sleep=s'      => \$Sleep,
        'help'         => \$Help,
        'all'          => \$All,
        'pool=s'       => \$PoolName,
        'host=s'       => \$HostName,
        'port=s'       => \$Port,
        'reload'       => \$Reload,
        'reloadmodules=s{1,}' => \@ReloadModules,
        'seq'          => \$ForceSequential,
        'timeout=s'    => \$Timeout,
        'maxthreads=s' => \$MaxThreads,
    );
    usage() if $Help;
    usage() unless $All || $Port || $HostName || $PoolName;

    $ForceSequential //= 0;
    $Reload          //= 0;
    $Timeout         //= ( $Reload ? 11 : 3 );                                  ## no critic qw(ProhibitMagicNumbers)
    $Sleep           //= ( $ForceSequential ? 2 : ( $Reload ? 0.1 : 0.5 ) );    ## no critic qw(ProhibitMagicNumbers)
    $MaxThreads      //= 100;

    my $Servers = expandServerList( $PoolName, $HostName, $Port );
    my $Options = {
        'Servers'    => $Servers,
        'Reload'     => $Reload,
        'Sleep'      => $Sleep,
        'Timeout'    => $Timeout,
        'Sequential' => $ForceSequential,
        'ReloadModules' => \@ReloadModules,
        'MaxThreads' => $MaxThreads,
    };
    my $CanUseThreads = eval { use threads; 1; };
    HandleError(
        OnError => sub {
            $CanUseThreads = 0;
            return 'FIXED';
        },
    );
    if ( $CanUseThreads and not $ForceSequential ) {
        sendParallel($Options);
    } else {
        sendSequential($Options);
    }
    print "\n";
}

sub expandServerList {
    my ( $PoolName, $HostName, $Port ) = @_;

    my $Servers;
    my $ServerConfig = ServerConfig();
    my @Groups       = values %{ $ServerConfig->appServerGroups };

    if ($PoolName) {
        my @PoolNames = map { trim($_) } split( /,/, $PoolName );
        @Groups = grep { $_->{PoolName} ~~ @PoolNames } @Groups;
    }
    if ($HostName) {
        if ( $HostName =~ /^(ipv6-)?localhost$/ ) {

            # TODO: Sys::HostIP->ips does not return IPv6 addresses
            my %LocalIPs = map { $_ => undef } @{ Sys::HostIP->ips };
            @Groups = grep { exists $LocalIPs{ GetIPAddress( $_->{Host} ) } } @Groups;
        } else {
            @Groups = grep { $_->{Host} eq $HostName } @Groups;
        }
    }
    foreach my $hGroup (@Groups) {
        my $Ports = $hGroup->{'Ports'};

        my @Ports;
        foreach my $Range ( split( /,/, $Ports ) ) {
            my @RangePorts
                = $Range =~ /(\d+)\s*-\s*(\d+)/x
                ? $1 .. $2
                : $Range;
            push @Ports, @RangePorts;
        }
        push @$Servers, map { { Host => $hGroup->{'Host'}, Port => $_ } } @Ports;
    }
    if ($Port) {
        $Servers = [ grep { $_->{Port} == $Port } @$Servers ];
    }

    return $Servers;
}

sub sendSequential {
    my ($Options) = @_;

    return unless defined $Options->{'Servers'};

    printf "Sequential processing.\n";
    my $Servers = $Options->{'Servers'};
    foreach my $hServer (@$Servers) {
        my @Params = ( $hServer->{'Host'}, $hServer->{'Port'}, $Options->{'Timeout'}, $Options->{'Sequential'}, $Options->{'ReloadModules'} );
        if ( $Options->{'Reload'} ) {
            SendRequest( 'reload', @Params );
        } elsif( scalar(@{$Options->{'ReloadModules'}}) ) {
            SendRequest( 'reloadmodules', @Params );
        } else {
            SendRequest( 'shutdown', @Params );
        }
        usleep( int( 1_000_000 * $Options->{'Sleep'} ) ) if scalar @$Servers > 1;    ## no critic qw(ProhibitMagicNumbers)
    }
}

sub sendParallel {
    my ($Options) = @_;

    return unless defined $Options->{'Servers'};

    my $Sleep        = $Options->{'Sleep'};
    my $MaxThreads   = $Options->{'MaxThreads'};
    my $Servers      = $Options->{'Servers'};
    my $CountServers = scalar @$Servers;

    printf "Restarting %s AS processes (package size:%s, delay: %ss)...\n", scalar @$Servers, $MaxThreads, $Sleep;
    my $ASIterator         = ArrayIterator($Servers);
    my $CurrentThreadCount = 0;
    my $ASRestarted        = 0;
    my $PackageSize        = $CountServers < $MaxThreads ? $CountServers : $MaxThreads;
    while ( my $hServer = <$ASIterator> ) {
        if ( $CurrentThreadCount < $MaxThreads ) {
            $CurrentThreadCount++;
            printf "\rCreating Thread %s/%s", $CurrentThreadCount, $PackageSize;
            my @Params = ( $hServer->{'Host'}, $hServer->{'Port'}, $Options->{'Timeout'}, $Options->{'Sequential'}, $Options->{'ReloadModules'} );
            if ( $Options->{'Reload'} ) {
                @Params = ( 'reload', @Params );
            } elsif( scalar(@{$Options->{'ReloadModules'}}) ) {
                @Params = ( 'reloadmodules', @Params );
            } else {
                @Params = ( 'shutdown', @Params );
            }
            threads->create( { 'context' => 'scalar' }, \&SendRequest, @Params );
            usleep( int( 1_000_000 * $Sleep ) );    ## no critic qw(ProhibitMagicNumbers)
            $ASRestarted++;
        }
        if ( $CurrentThreadCount == $MaxThreads || $ASRestarted == scalar @$Servers ) {
            while ( scalar threads->list(threads::all) ) {
                my $Run = threads->list(threads::running);
                printf "\nWaiting for %s threads to finish execution...\r", $Run;
                while ( ( scalar threads->list(threads::joinable) == 0 ) and ( scalar threads->list(threads::running) > 0 ) ) {
                    usleep(100_000);                ## no critic qw(ProhibitMagicNumbers)
                }
                print $_->join() foreach ( threads->list(threads::joinable) );
            }
            $CurrentThreadCount = 0;
            $PackageSize = $CountServers - $ASRestarted if ( $CountServers - $ASRestarted < $PackageSize );
        }
    }
}

sub SendRequest ($$$$$$) {
    my ( $Request, $HostName, $Port, $Timeout, $Sequential, $aReloadModules ) = @_;

    $Request .= "\r\nX-epages-MODULELIST: ".join(',',@$aReloadModules) if scalar(@$aReloadModules);

    my $IP = GetIPAddress($HostName);
    my $StartMsg = sprintf "send $Request to [%s]:%d (%s) Timeout %ss -> ", $HostName, $Port, '' . localtime(), $Timeout;
    print $StartMsg if $Sequential;

    my $Socket = IO::Socket::INET6->new(
        PeerAddr => $IP,
        PeerPort => $Port,
        Type     => SOCK_STREAM,
        Proto    => 'tcp',
        Timeout  => $Timeout,
    );
    my $EndMsg;

    if ( defined $Socket ) {
        $Socket->autoflush();
        $Socket->send( sprintf( REQUEST, $Request ), 0 );
        if ($OS_ERROR) {
            $EndMsg = sprintf "send FAILED [%s]:%d : %s!\n", $HostName, $Port, $OS_ERROR;
        } else {

            # the Socket->Timeout only works for connect() and others, not for receive due to a bug
            # in IO::Socket and glibc (https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=155100)
            # So we need to set in manually otherwise recv will wait forever (if connected)
            $Socket->setsockopt( SOL_SOCKET, SO_RCVTIMEO, pack( 'l!l!', $Timeout, 0 ) );
            my $BUF;
            $Socket->recv( $BUF, 1 );
            $Socket->close;
        }
    }
    if ($OS_ERROR) {
        $EndMsg = sprintf "recv [%s]:%d : %s!\n", $HostName, $Port, $OS_ERROR;
    } else {
        $EndMsg = sprintf "recv [%s]:%d : OK\n", $HostName, $Port;
    }
    print $EndMsg if $Sequential;
    return $StartMsg . $EndMsg;
}

sub usage {
    print <<END_USAGE;
Description:
        Stops/Reloads all application servers one by one or in parallel (default).
        Can also be used to reload perl modules by a given package to the running application server.
        If threads cannot be used it fails back to sequential processing.
        The servers will be restarted automatically by the epages Service.
        Note that 'reload' takes approx. 10 seconds per AS.

Usage:
    perl $PROGRAM_NAME [flags] [options]

options:
    -all        restart all processes (optional)
    -pool       (comma separated list) restart all processes associated with this pool(s) (optional)
    -host       restart only processes on this server
                (either IP or 'localhost' for all local IPs) (optional)
    -port       restart only the process using this port (optional)
    -sleep      wait n seconds before sending to the next server (default=2 if sequential)
    -sleep      wait n seconds before starting the next thread (default=0.5 if threaded)
    -timeout    socket recv() and connect() timeout in seconds (default: 3 for restart, 11 for reload ) 0 = unlimited
    -reloadmodules  comma separated list of package names or relative filenames to reload without restarting AS (e.g. DE_EPAGES::Presentation::UI::Object or DE_EPAGES/Presentation/UI/Object.pm )
    -maxthreads max perl threads (default=100)
flags:
    -help       show the command line options
    -reload     do not restart the processes, just reload pool assignments
    -seq        force sequential processing (otherwise tries to run in threaded mode if possible)

Example:
    perl $PROGRAM_NAME -pool DefaultPool
    perl $PROGRAM_NAME -host localhost -seq -timeout 1 -sleep 0
    perl $PROGRAM_NAME -host localhost -sleep 0.2
    perl $PROGRAM_NAME -host localhost -reloadmodules DE_EPAGES::Presentation::UI::Object

END_USAGE
    exit 2;
}
